unit Unit1;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaImageList, OmegaScreen, OmegaInput, OmegaBitmapFont,
  D3D8Math, OmegaTimer, OmegaUtils, LMGen, D3DX8, {$I UseD3D8.inc};

type
  TForm1 = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaTimer1: TOmegaTimer;
    OmegaBitmapFont1: TOmegaBitmapFont;
    OmegaInput1: TOmegaInput;
    procedure FormCreate(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
  private
    procedure SetMaterials;
    procedure SetupMatrices;
    function InitVertexBuffer: boolean;
    procedure AnimateLight;
    procedure UpdateLightMap;
  end;

var Form1: TForm1;

implementation
{$R *.DFM}

type
  // The structure of the custom vertex.
  TCustomVertex = record
                    Position : TD3DXVector3; // Position.
                    DColor : dWord;          // Diffuse color.
                    U, V     : Single; // U and V for texture.
                    U2, V2   : single; // U and V for light map.
                  end;

var
  // Mesh
  VertexBuffer : IDirect3DVertexBuffer8 = NIL;
  MeshSource   : array[0..3] of TCustomVertex;

  // Texture
  Texture  : IDirect3DTexture8 = NIL;
  LightMap : IDirect3DTexture8 = NIL;
  canmulti, drawmulti: boolean;

  // Matrices
  matWorld : TD3DXMatrix; // World
  matView  : TD3DXMatrix; // View
  matProj  : TD3DXMatrix; // Projection

  // Ambiental light.
  AmbColor : integer = $000000;   // Black
  // Omnidirectional lights.
  Omni1 : TPointlight;
  Omni2 : TPointlight;
  Omni3 : TPointlight;
  // Spot lights
  Spot1 : TSpotlight;
  l1, l2, l3, l4, showlm: boolean;

const
  // Flexible vector format (FVF) description.
  // Describes the custom vertex structure (TCustomVertex).
  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZ or D3DFVF_DIFFUSE or
                        D3DFVF_TEX1 or D3DFVF_TEX2;
  USETEXFILTER = true; // Use texture filtering.
  TEXTFILENAME = 'WALL.PNG';
  LIGHTMAPRES  = 16; // 16x16 16-bit lightmaps.

procedure TForm1.SetMaterials;
var _mat : TD3DMaterial8;
begin
  // Set Material
  FillChar(_mat, sizeof(_mat), 0 );
  with _mat do
    begin
      Ambient.r := 1.0;
      Ambient.g := 1.0;
      Ambient.b := 1.0;
      Ambient.a := 1.0;
      OmegaScreen1.Device.SetMaterial(_mat);
    end;
end;

procedure TForm1.SetupMatrices;
var _v1, _v2, _v3 : TD3DXVector3;
begin
  // World.
  D3DXMatrixIdentity(matWorld);
  OmegaScreen1.Device.SetTransform(D3DTS_WORLD, matWorld);
  // Camera
  _v1 := D3DXVector3(0.0, 0.0, -2.0);
  _v2 := D3DXVector3(0.0, 0.0, 0.0);
  _v3 := D3DXVector3(0.0, 1.0, 0.0);
  // View.
  D3DXMatrixLookAtLH(matView, _v1, _v2, _v3);
  OmegaScreen1.Device.SetTransform(D3DTS_VIEW, matView);
  // Projection.
  D3DXMatrixPerspectiveFovLH(matProj, PI/4, 1{512/384}, 0.5, 10.0);
  OmegaScreen1.Device.SetTransform(D3DTS_PROJECTION, matProj);
end;

procedure TForm1.AnimateLight;
var _time : dWord;
begin
 _time := GetTickCount;
 // Omni 1
 Omni1.Position.x := cos(_time/150) * 0.5;
 Omni1.Position.y := cos(_time/100) * 0.5;
 // Spot 1
 Spot1.Target.x := cos(_time/250) * 1.0;
end;

procedure TForm1.UpdateLightMap;
var _d3dr : TD3DLocked_Rect;
    _cursor : pWord;
    _x, _y  : longint;
    _u, _v  : single;
    _21x, _31x, _21y, _31y, _21z, _31z : single;
    _point  : TD3DXVector3;
    _normal : TD3DXVector3;
    _color  : word;
    _colorT : word;
    _useOmni1 : boolean;
    _useOmni2 : boolean;
    _useOmni3 : boolean;
    _useSpot1 : boolean;
    _lineinc : longint;
begin
  // Get face normal.
  _normal := FaceNormal(MeshSource[0].Position,
                        MeshSource[1].Position,
                        MeshSource[2].Position);

  // Wich lights must be used with this face ?
  _useOmni1 := (l1) and (PointFacesPlane(
                Omni1.Position, MeshSource[0].Position, _normal));
  _useOmni2 := (l2) and (PointFacesPlane(
                Omni2.Position, MeshSource[0].Position, _normal));
  _useOmni3 := (l3) and (PointFacesPlane(
                Omni3.Position, MeshSource[0].Position, _normal));
  _useSpot1 := (l4) and (PointFacesPlane(
                Spot1.Position, MeshSource[0].Position, _normal));

  // Initialize data for using GetPointInFace function.
  PIFInit(MeshSource[0].Position,
          MeshSource[1].Position,
          MeshSource[2].Position,
          _21x, _31x, _21y, _31y, _21z, _31z);

  // Make the lightmap.
  if not failed(LightMap.LockRect(0, _d3dr, NIL, 0)) then
    begin
      _cursor  := _d3dr.pBits;
      _lineinc := (_d3dr.Pitch shr 1) - LIGHTMAPRES;
      for _y := 0 to LIGHTMAPRES - 1 do
        begin
          for _x := 0 to LIGHTMAPRES - 1 do
            begin
              _u := (1.0 * _x) / (LIGHTMAPRES - 1);
              _v := (1.0 * _y) / (LIGHTMAPRES - 1);

              _point := PointInFace(MeshSource[0].Position,
                                    _21x, _31x, _21y, _31y, _21z, _31z,
                                    _u, _v);

              _color := AmbColor; // Get ambiental color.
              if _useOmni1 then // Omnidirectional light 1
                begin
                  _colorT := RGB8toRGB5(GetOmniColor(_point, Omni1));
                  _color := SumRGB5Colors(_color, _colorT);
                end;
              if _useOmni2 then // Omnidirectional light 2
                begin
                  _colorT := RGB8toRGB5(GetOmniColor(_point, Omni2));
                  _color := SumRGB5Colors(_color, _colorT);
                end;
              if _useOmni3 then // Omnidirectional light 3
                begin
                  _colorT := RGB8toRGB5(GetOmniColor(_point, Omni3));
                  _color := SumRGB5Colors(_color, _colorT);
                end;
              if _useSpot1 then // Spotlight 1
                begin
                  _colorT := RGB8toRGB5(GetSpotColor(_point, Spot1));
                  _color := SumRGB5Colors(_color, _colorT);
                end;
              _cursor^ := _color;
              inc(_cursor);
            end;
          inc(_cursor, _lineinc);
        end;
      LightMap.UnlockRect(0);
    end;
end;

function CustomVertex(_px, _py, _pz, _u, _v, _u2, _v2: single;
                      _color : dWord): TCustomVertex;
begin
  with Result do
   begin
     Position.x := _px;
     Position.y := _py;
     Position.z := _pz;
     DColor := _color;
     U := _u;
     V := _v;
     U2 := _u2;
     V2 := _v2;
   end;
end;


function TForm1.InitVertexBuffer: boolean;
var _ptr : pByte;
begin
  Result := false;
  // Initialize the vertices.
  MeshSource[0] := CustomVertex( -1.0,  1.0,  0.0, 0, 0, 0, 0, $FFFFFFFF);
  MeshSource[1] := CustomVertex(  1.0,  1.0,  0.0, 2, 0, 1, 0, $FFFFFFFF);
  MeshSource[2] := CustomVertex( -1.0, -1.0,  0.0, 0, 2, 0, 1, $FFFFFFFF);
  MeshSource[3] := CustomVertex(  1.0, -1.0,  0.0, 2, 2, 1, 1, $FFFFFFFF);
  // Create a vertex buffer.
  OmegaScreen1.Device.CreateVertexBuffer(SizeOf(MeshSource), 0,
                                       D3DFVF_CUSTOMVERTEX,
                                       D3DPOOL_MANAGED,
                                       VertexBuffer);

  // Lock, fill and unlock vertex buffer.
  if failed(VertexBuffer.Lock(0, SizeOf(MeshSource), _ptr, 0)) then exit;
  Move(MeshSource, _ptr^, SizeOf(MeshSource));
  VertexBuffer.Unlock;

  // All right
  Result := true;
end;

procedure TForm1.FormCreate(Sender: TObject);
var _filt : dWord;
begin
 self.ClientWidth := 800;
 self.ClientHeight := 600;
 OmegaScreen1.Width := self.ClientWidth;
 OmegaScreen1.Height := self.ClientHeight;
 OmegaScreen1.Init;
 with OmegaScreen1.Device do
 begin
  // Initialize VertexBuffer
  InitVertexBuffer;
  // Set Material
  SetMaterials;
  InitVertexBuffer;
  l1 := true;
  l2 := true;

  // Device supports multitexturing ?
  canmulti := OmegaScreen1.FD3DDEVCAPS8.MaxSimultaneousTextures > 1;
  // Device supports bilinear filter ?
  _filt := (D3DPTFILTERCAPS_MAGFLINEAR or D3DPTFILTERCAPS_MINFLINEAR);

  if (USETEXFILTER)
   and (OmegaScreen1.FD3DDEVCAPS8.TextureFilterCaps and _filt = _filt) then
    begin
      SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
      SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
      if (OmegaScreen1.FD3DDEVCAPS8.MaxSimultaneousTextures > 1) then
        begin
          SetTextureStageState(1, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
          SetTextureStageState(1, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
        end;
    end;
  // Turn off D3D lighting, since we are providing
  // our own vertex colors.
  SetRenderState(D3DRS_LIGHTING, 0);
  // Enable dithering
  SetRenderState(D3DRS_DITHERENABLE, 1);
  // Create texture for lighmap.
  D3DXCreateTexture(OmegaScreen1.Device, LIGHTMAPRES, LIGHTMAPRES, 1,
                          0, D3DFMT_A1R5G5B5, D3DPOOL_MANAGED, LightMap);
  // Load texture.
  D3DXCreateTextureFromFileA(OmegaScreen1.Device, TEXTFILENAME, Texture);
  // Initialize ommnidirectional lights.
  Omni1 := Pointlight(D3DXVector3( 0.0,  0.0, -0.25), 0.5, 1.0, $FFFFFF);
  Omni2 := Pointlight(D3DXVector3( 1.0,  1.0, -0.1),  0.5, 1.5, $00FF00);
  Omni3 := Pointlight(D3DXVector3(-1.0, -1.0, -0.1),  0.5, 1.5, $0000FF);
  // Initialize spot lights.
  Spot1 := Spotlight(D3DXVector3( 0.0, -1.5, -0.25),
                     D3DXVector3( 0.0,  0.0,  0.0),
                     PI/32, PI/8, 1.0, 3.5, $FFFFFF);
 end;
 OmegaTimer1.Enabled := True;
end;

procedure TForm1.OmegaTimer1Timer(Sender: TObject);
begin
 if not OmegaScreen1.CanDraw then exit;
 OmegaInput1.Update;
 if oisbutton1 in OmegaInput1.keyboard.statesclicked then
 begin
  OmegaScreen1.ToggleFullScreen;
  with OmegaScreen1.device do
  begin
   SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
   SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
   if (OmegaScreen1.FD3DDEVCAPS8.MaxSimultaneousTextures > 1) then
   begin
    SetTextureStageState(1, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
    SetTextureStageState(1, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
   end;
   SetRenderState(D3DRS_DITHERENABLE, 1);
  end;
 end; 
 OmegaScreen1.ClearScreen(0, 0, 0);
 OmegaScreen1.BeginRender;

 with OmegaScreen1.device do
 begin
  // Set Transform states.
  SetupMatrices;
  // Animate light
  AnimateLight;
  // Update lightmap.
  UpdateLightMap;
  // Set the stream source.
  SetStreamSource(0, VertexBuffer, sizeof(TCustomVertex));
  // Set the vertex shader.
  SetVertexShader(D3DFVF_CUSTOMVERTEX);

  // Draw only lightmap ?
  if showlm then
  begin
   SetRenderState(D3DRS_ALPHABLENDENABLE, 0);
   SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 1);
   SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
   SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
   SetTexture(0, LightMap);
   DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
  end else
  begin
   // Draw using multi-pass
   if (canmulti and drawmulti) then
   begin
    // Draw the first textures normally.
    // Use the 1st set of tex coords.
    SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
    SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
    SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
    SetTexture(0, Texture);
    DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
    // Draw the lightmap using blending,
    // with the 2nd set of tex coords
    SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 1);
    SetRenderState(D3DRS_ALPHABLENDENABLE, 1);
    SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ZERO);
    SetRenderState(D3DRS_DESTBLEND, D3DBLEND_SRCCOLOR);
    SetTexture(0, LightMap);
    DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
    // Restore state
    SetRenderState(D3DRS_ALPHABLENDENABLE, 0);
   end else
   begin // Draw using multi-texture
    // Texture States for first texture (0); the wall
    SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
    SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
    SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
    SetTexture(0, Texture);
    // Texture States for second texture (1); the lightmap
    SetTextureStageState(1, D3DTSS_TEXCOORDINDEX, 1);
    SetTextureStageState(1, D3DTSS_COLORARG1, D3DTA_TEXTURE);
    SetTextureStageState(1, D3DTSS_COLORARG2, D3DTA_CURRENT);
    SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_MODULATE);
    SetTexture(1, LightMap);
    // Draw mesh.
    DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
    // Restore state
    SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
   end;
  end;
  SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
  SetRenderState(D3DRS_ALPHABLENDENABLE, cardinal(TRUE));
  SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
  SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );
 end;
 OmegaScreen1.SetBlendMode(bmNormal);
 OmegaBitmapFont1.Print(44, 480, 'Press "space" to toggle fullscreen');
 OmegaBitmapFont1.Print(44, 500, 'Press "A" to toggle ambient light');
 OmegaBitmapFont1.Print(44, 520, 'Press "R" to randomize ambient light color');
 OmegaBitmapFont1.Print(44, 540, 'Press "L" to toggle lightmap display');
 OmegaBitmapFont1.Print(44, 560, 'Press "M" to toggle multi-texture drawing');
 OmegaBitmapFont1.Print(44, 580, 'Press "1" to "4" to toggle different lights');
 OmegaScreen1.EndRender;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 OmegaTimer1.Enabled := False;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if key = 27 then close;
 if key = ord('M') then drawmulti := not drawmulti;
 if key = ord('R') then
  AmbColor := random($FFFFFF) and $1F1F1F;
 if key = ord('A') then
 begin
  if AmbColor = 0 then
   AmbColor := ($FFFFFF) and $1F1F1F
  else AmbColor := $000000;
 end;
 if key = ord('1') then l1 := not l1;
 if key = ord('2') then l2 := not l2;
 if key = ord('3') then l3 := not l3;
 if key = ord('4') then l4 := not l4;
 if key = ord('L') then showlm := not showlm;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 LightMap := NIL;
 Texture := NIL;
 // Free Vertex Buffer
 VertexBuffer := NIL;
end;

end.


