unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaFont, OmegaImageList, OmegaSprite, OmegaTimer, OmegaScreen,
  OmegaSound3D, OmegaMusic, OmegaBitmapFont, OmegaInput, OmegaMultimedia,
  OmegaFader;

type
  // our different scenes in the game
  TGameScene = (gs_None, gs_Menu, gs_Game, gs_GameOver);

  TStar = record
    X, Y: Integer;  // position
    Alpha: Byte;    // low = star is far away high = star is close
  end;

  // out player =)
  TPlayer = class(TSprite)
  public
    procedure OnCollision(const Sprite: TSprite; const colX, colY: Integer); override;
    procedure Move(const MoveCount: Single); override;
  end;

  // an enemy =(
  TEnemy = class(TSprite)
  public
    AI: Integer;         // what kind of attack pattern will the enemy have
    Curve: Integer;      // used for the sinus curve (AI = 1)
    OriginalX: Integer;  // used for the sinus curve (AI = 1)
    procedure OnCollision(const Sprite: TSprite; const colX, colY: Integer); override;
    procedure Move(const MoveCount: Single); override;
  end;

  // bullet that the player can fire =)
  TBullet = class(TSprite)
  public
    YSpeed: Integer;                              // speed of bullet
    procedure Move(const MoveCount: Single); override;
  end;

type
  TMainForm = class(TForm)
    OmegaScreen: TOmegaScreen;
    OmegaTimer: TOmegaTimer;
    OmegaSprite: TOmegaSprite;
    OmegaImageList: TOmegaImageList;
    OmegaInput: TOmegaInput;
    OmegaMultimedia: TOmegaMultimedia;
    OmegaBitmapFont: TOmegaBitmapFont;
    OmegaFader: TOmegaFader;
    procedure OmegaTimerTimer(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    Scene: TGameScene;   // current scene
    Next: TGameScene;    // change to this scene

    Flash: Boolean;      // flash when player dies
    Score: Integer;      // guess what ;-)

    Stars: array[0..49] of TStar;

    procedure ChangeScene(NewScene: TGameScene); // change the scene
    procedure EndScene;                          // end current scene
    procedure DrawMenu;                          // draw menu scene
    procedure DrawGame;                          // draw game scene
    procedure DrawGameOver;                      // draw gameover scene
    procedure StartGame;                         // start game scene
  end;

var
  MainForm: TMainForm;
  Player: TPlayer;

implementation

{$R *.dfm}

// if the Player collides with a sprite this procedure is called
//--------------------------------------------------------------
procedure TPlayer.OnCollision(const Sprite: TSprite; const colX, colY: Integer);
begin
  if Sprite is TEnemy then // have we collided with an enemy?
  begin
    MainForm.Flash := True;
    MainForm.OmegaFader.SetDestinationColor(OmegaColor(clWhite));
    MainForm.OmegaFader.PrepareFadeOut(255, 255, 255, 1);
    Dead;
    MainForm.OmegaMultimedia.Media.Items[0].Play;
  end;
end;

// Move the player
// ---------------
procedure TPlayer.Move(const MoveCount: Single);
var
  Bullet: TBullet;
begin
  // if we press space, fire of a bullet
  if oisButton1 in MainForm.OmegaInput.Keyboard.StatesClicked then
  begin
    Bullet := TBullet.Create(MainForm.OmegaSprite);
    Bullet.YSpeed := -5;
    Bullet.Width := 5;
    Bullet.Height := 15;
    Bullet.X := X + (Width div 2) - (Bullet.Width div 2);
    Bullet.Y := Y - 16;
    Bullet.Image := MainForm.OmegaImageList.ImageList.Items[1];
    Bullet.DoCollision := True;
    Bullet.DoPixelCheck := True;
    Bullet.Red := 255;
    Bullet.Green := 255;
    Bullet.Blue := 255;
    Bullet.Alpha := 255;
    Bullet.ScaleX := 0;
    Bullet.ScaleY := 0;
  end;

  // if we press right, go right
  if oisRight in MainForm.OmegaInput.Keyboard.States then
  begin
    X := X + 2;
    if X + Width > 640 then X := 640 - Width;
  end;

  // if we press left, go left
  if oisLeft in MainForm.OmegaInput.Keyboard.States then
  begin
    X := X - 2;
    if X < 0 then X := 0;
  end;

  // if we press up, go upp
  if oisUp in MainForm.OmegaInput.Keyboard.States then
  begin
    Y := Y - 2;
    if Y < 0 then Y := 0;
  end;

  // if we press down, go down
  if oisDown in MainForm.OmegaInput.Keyboard.States then
  begin
    Y := Y + 2;
    if Y + Height > 480 then Y := 480 - Height;
  end;

  // if we press escape change scene
  if oisButton2 in MainForm.OmegaInput.Keyboard.StatesClicked then
  begin
    MainForm.Next := gs_Menu;
  end;
end;

// if the enemy collides with anything deal with it
// ------------------------------------------------            
procedure TEnemy.OnCollision(const Sprite: TSprite; const colX, colY: Integer);
begin
  if Sprite is TBullet then // check if the sprites is a bullet
  begin
    Dead;
    Sprite.Dead;
    MainForm.Score := Mainform.Score + 100 + (AI * 50);
    MainForm.OmegaMultimedia.Media.Items[0].Play;
  end;
end;

// move the enemy
// --------------    
procedure TEnemy.Move(const MoveCount: Single);
begin
  // AI = 0 just move the enemy downwards
  // AI = 1 move the enemy down in a nice sinus curve
  // AI = 2 make the enemy hunt the player

  case AI of
    0: begin
      Y := Y + 2;
    end;
    1: begin
      Y := Y + 1;
      Inc(Curve);
      X := OriginalX + Round(50 * sin(2 * PI * Curve / 360));
    end;
    2: begin
      Y := Y + 2;
      if Player.X > X then X := X + 1;
      if Player.X < X then X := X - 1;
    end;
  end;

  if Y > 480 then Dead; // if the enemy goes out of screen clear it out of memory
end;

// Move the bullet
// ---------------
procedure TBullet.Move(const MoveCount: Single);
begin
  Y := Y + YSpeed;
  if Y > 480 then Dead; // if the bullet goes out of screen clear it out of memory
  if Y < 0 then Dead;   // if the bullet goes out of screen clear it out of memory
end;

// change the current screen
// -------------------------
procedure TMainForm.ChangeScene(NewScene: TGameScene);
begin
  EndScene;

  Scene := NewScene;

  // start scenes
  case Scene of
    gs_Menu: begin
      // none
    end;
    gs_Game: begin
      StartGame;
    end;
    gs_GameOver: begin
      // none
    end;
  end;
end;

// end current scene
// -----------------
procedure TMainForm.EndScene;
var
  i: Integer;
begin
  case Scene of
    gs_Menu: begin
      // nothing
    end;
    gs_Game: begin
      // delete all sprites
      for i := 0 to OmegaSprite.Count - 1 do
          TSprite(OmegaSprite.Items[i]).Dead;
      OmegaSprite.Dead;
    end;
    gs_GameOver: begin
      // nothing
    end;
  end;
end;

// start the game scene
// --------------------
procedure TMainForm.StartGame;
var
  i: Integer;
begin
  // create the player
  Player := TPlayer.Create(OmegaSprite);
  Player.Width := 24;
  Player.Height := 32;
  Player.X := 320 - Player.Width div 2;
  Player.Y := 400;
  Player.Image := OmegaImageList.ImageList.Items[0];
  Player.DoCollision := True;
  Player.DoPixelCheck := True;
  Player.Red := 255;
  Player.Blue := 255;
  Player.Green := 255;
  Player.Alpha := 255;
  Player.ScaleX := 0;
  Player.ScaleY := 0;

  // make sure "flash" isn't enabled when we go come in the scene
  Flash := False;
  MainForm.OmegaFader.Stop;
  MainForm.OmegaFader.Reset;

  // set score to zero
  Score := 0;

  // new stars
  for i := 0 to 49 do
  begin
    Stars[i].X := Random(640-8);
    Stars[i].Y := Random(480-8);
    Stars[i].Alpha := Random(156) + 100;
  end;
end;
     
// Draw the main menu
// ------------------
procedure TMainForm.DrawMenu;
begin
  OmegaBitmapFont.Print(10, 10, 0.0, 1.0, 1.0, OmegaColor(clWhite), 'Press [SPACE] to start game');

  // if Space is pressed start the game
  if oisButton1 in OmegaInput.Keyboard.StatesClicked then
  begin
    Next := gs_Game;
  end;
  // if escape is pressed exit the game
  if oisButton2 in OmegaInput.Keyboard.StatesClicked then
  begin
    Close;
  end;
end;

// Draw the game
// -------------
procedure TMainForm.DrawGame;
var
  RandEnemy: Integer;
  Enemy: TEnemy;
  i: Integer;
begin
  OmegaImageList.BeginDraw;

  // draw the stars
  for i := 0 to 49 do
      OmegaImageList.ImageList.Items[3].Draw(Stars[i].X, Stars[i].Y, 0, 0, 0, 1, 1, 255, 255, 255, Stars[i].Alpha, False, 0);

  // sprites procedures
  OmegaSprite.Move(0);  
  OmegaSprite.Collision;
  // Imortant!
  // Make sure that OmegaSprite.Dead comes directly after
  // OmegaSprite.Collision. Otherwise strange things can happen.
  OmegaSprite.Dead;
  OmegaSprite.Draw;

  // if the player has died make a flash
  if Flash then
  begin
    OmegaFader.Draw;
    if OmegaFader.FadeEnded then Next := gs_GameOver;
  end;

  // create new enemies
  for RandEnemy := 0 to 3 do
    if Random(100) = 0 then
    begin
      Enemy := TEnemy.Create(OmegaSprite);
      with Enemy do
      begin
        Width := 24;
        Height := 32;
        X := Random(640 - Width);
        Y := - Height - Random(100);
        Image := OmegaImageList.ImageList.Items[2];
        AI := Random(3);
        Red := 255;
        Green := 255;
        Blue := 255;
        Alpha := 255;
        DoCollision := True;
        DoPixelCheck := True;
        OriginalX := Round(X);
      end;
    end;

  OmegaImageList.EndDraw;
end;

// draw the gameover scene
// -----------------------
procedure TMainForm.DrawGameOver;
begin
  // let the player know s/he failed
  OmegaBitmapFont.Print(210, 200, 0.0, 1.0, 1.0, OmegaColor(clRed), 'OH-NO! GAME OVER!!!');

  // if the user press escape exit the game
  if oisButton2 in OmegaInput.Keyboard.StatesClicked then
  begin
    Close;
  end;
  // if the player press space go back to the main menu
  if oisButton1 in OmegaInput.Keyboard.StatesClicked then
  begin
    Next := gs_Menu;
  end;
end;

// the mainloop
//-------------
procedure TMainForm.OmegaTimerTimer(Sender: TObject);
begin
  // if we can't draw then exit
  if not OmegaScreen.CanDraw then Exit;
  // I always forget this line and wonder why nothing happens when I press a key ;-)
  OmegaInput.Update;

  OmegaScreen.BeginRender;
  OmegaScreen.ClearScreen(0, 0, 0, 255);

  case Scene of
    gs_Menu: begin
      DrawMenu;
    end;
    gs_Game: begin
      DrawGame;
    end;
    gs_GameOver: begin
      DrawGameOver;
    end;
  end;

  // make sure we change scene if we changed the value of Next
  if Next <> gs_None then
  begin
    ChangeScene(Next);
    Next := gs_None;
  end;

  // print fps and score
  OmegaBitmapFont.Print(520, 400, 0.0, 1.0, 1.0, OmegaColor(clWhite), 'FPS = ' + IntToStr(OmegaTimer.FPS));
  OmegaBitmapFont.Print(520, 420, 0.0, 1.0, 1.0, OmegaColor(clWhite), 'Score = ' + IntToStr(Score));

  OmegaScreen.EndRender;
end;

// close the game
//---------------
procedure TMainForm.FormDestroy(Sender: TObject);
begin
  EndScene;
  OmegaTimer.Enabled := False;
end;

// create the game
//----------------
procedure TMainForm.FormCreate(Sender: TObject);
begin
  Randomize;
  Cursor := crNone;

  // start with the menu scene
  Scene := gs_None;
  Next := gs_Menu;

  // usual initialization of Omega components
  OmegaScreen.Init;
  OmegaBitmapFont.MakeFont('verdana', DEFCHARACTERS, 10, 0);
  OmegaMultimedia.Init;
  OmegaInput.Initialize;

  OmegaTimer.Enabled := True;
end;

end.
