unit uMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, OmegaImageList, OmegaScreen, OmegaTimer,
  OmegaBitmapFont, OmegaSprite, OmegaUtils, OmegaInput,
  OmegaPrimitives, {$I UseD3D8.inc};

type
  TfrmMain = class(TForm)
    oScreen: TOmegaScreen;
    oIL: TOmegaImageList;
    oTimer: TOmegaTimer;
    oBitmapFont: TOmegaBitmapFont;
    oInput: TOmegaInput;
    oSprite: TOmegaSprite;
    oPrimitives: TOmegaPrimitives;
    procedure FormCreate(Sender: TObject);
    procedure oTimerTimer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure oScreenInit(Sender: TObject);
  end;

  TSpaceShip = class(TSprite)
  public
    constructor Create(const AParent: TOmegaSprite); override;
    procedure Draw; override;
//    procedure Move(const MoveCount: single); override;
//    procedure OnCollision(const Sprite: TSprite; const X, Y: integer); override;
  end;
  TSpaceDisc = class(TSimpleAnimSprite)
  public
    constructor Create(const AParent: TOmegaSprite); override;
    procedure Draw; override;
//    procedure OnCollision(const Sprite: TSprite; const X, Y: integer); override;
  end;

  var frmMain: TfrmMain;
      pixelCheck, showBorders: boolean;
      hor, ver: boolean;
      rCenter: single;

implementation
{$R *.DFM}

constructor TSpaceShip.Create(const AParent: TOmegaSprite);
begin
 inherited;
 Image := frmMain.oIL.ImageList.Find('spaceShip');
 X := 300; Y := 300;
end;
constructor TSpaceDisc.Create(const AParent: TOmegaSprite);
begin
 inherited;
 Image := frmMain.oIL.ImageList.Find('spaceDisc');
 AnimLooped:= true; DoAnimate:= true;
 AnimCount:= 6; AnimSpeed:= 5;
 X := 200; Y := 200;
end;

procedure TSpaceShip.Draw;
  var q: TQuad;
begin
 inherited;
 MirrorHorizontally:= hor; MirrorVertically:= ver;
 CenterX:= rCenter; CenterY:= rCenter;
 if showBorders then begin
   q:= GetQuad;
   frmMain.oPrimitives.Polygon(q, OmegaColor($FFFFFFFF));
   end;
end;
procedure TSpaceDisc.Draw;
  var q: TQuad;
begin
 inherited;
 CenterX:= rCenter; CenterY:= rCenter;
 MirrorHorizontally:= hor; MirrorVertically:= ver;
 if showBorders then begin
   q:= GetQuad;
   frmMain.oPrimitives.Polygon(q, OmegaColor($FFFFFFFF));
   end;
end;

procedure TfrmMain.oTimerTimer(Sender: TObject);
  var p: TPoint;
  targetPoint: TPoint;
  aimedSprite: TSprite;
  i: integer;
begin
 if not oScreen.CanDraw then exit;
 oInput.Update;
 oSprite.Move(oTimer.DeltaSecs);

 if oisbutton1 in oInput.keyboard.statesclicked then oScreen.ToggleFullScreen;
 //change Rotation of all sprites in oSprite
 if oisLeft in oInput.keyboard.statesclicked then
    for i:= 0 to oSprite.Count - 1 do
      oSprite.Items[i].Rotation:= oSprite.Items[i].Rotation - 5;
 if oisRight in oInput.keyboard.statesclicked then
    for i:= 0 to oSprite.Count - 1 do
      oSprite.Items[i].Rotation:= oSprite.Items[i].Rotation + 5;
 if oisUp in oInput.keyboard.statesclicked then
    rCenter:= rCenter + 0.1;
 if oisDown in oInput.keyboard.statesclicked then
    rCenter:= rCenter - 0.1;
 if rCenter > 1 then rCenter:= 1;
 if rCenter < 0 then rCenter:= 0;

 oScreen.ClearScreen(0, 0, 0, 0);
 oScreen.BeginRender;

 //targetPoint coords depend on fullScreen/non-FS mode
 if not oScreen.FullScreen then begin
  p := ScreenToClient(Mouse.CursorPos);
  targetPoint.x := p.x; targetPoint.y := p.Y;
 end else begin
  targetPoint.x := Mouse.CursorPos.X;
  targetPoint.y := Mouse.CursorPos.Y;
 end;

 oSprite.Draw;

 oBitmapFont.Print(40, 40, 'FPS: ' + IntToStr(oTimer.FPS));
 oBitmapFont.Print(40, 60, 'Target.X: ' + IntToStr(targetPoint.X) + '; target.Y: ' + IntToStr(targetPoint.Y));
 oBitmapFont.Print(40, 80, 'Rotation Center: ' + FloatToStr(rCenter));
 oBitmapFont.Print(40, 380, 'Press "Space" to toggle fullscreen');
 oBitmapFont.Print(40, 400, 'Press "H" to toggle horizontal mirroring');
 oBitmapFont.Print(40, 420, 'Press "V" to toggle vertical mirroring');
 oBitmapFont.Print(40, 440, 'Press "B" to toggle border display');
 oBitmapFont.Print(40, 460, 'Press "<-" to decrease rotation');
 oBitmapFont.Print(40, 480, 'Press "->" to encrease rotation');
 oBitmapFont.Print(40, 500, 'Press "v" to decrease rotation Center value');
 oBitmapFont.Print(40, 520, 'Press "^" to encrease rotation Center value');
 oBitmapFont.Print(40, 540, 'Press "P" to toggle pixel checking in GetSpriteAt');
 oBitmapFont.Print(40, 560, 'Press "Esc" to exit');

 //aimedSprite is a sprite under mouse cursor
 aimedSprite:= oSprite.GetSpriteAt(targetPoint.X, targetPoint.Y, pixelCheck, false);
 if aimedSprite <> nil then
   oBitmapFont.Print(40, 100, 'Aimed sprite: ' + aimedSprite.Image.Name);

 oScreen.EndRender;
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 oTimer.Enabled := False;
end;

procedure TfrmMain.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if Key = 27 then Close;
 if Key = ord('B') then showBorders:=not showBorders;
 if Key = ord('H') then hor := not hor;
 if Key = ord('V') then ver := not ver;
 if Key = ord('P') then pixelCheck := not pixelCheck;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
 self.ClientWidth := 800; self.ClientHeight := 600;
 oScreen.Width := self.ClientWidth; oScreen.Height := self.ClientHeight;
 oScreen.Init; oTimer.Enabled := true;
// oScreen.SetAntiAliasing(true);

 TSpaceDisc.Create(oSprite);
 TSpaceShip.Create(oSprite);
 hor := false; ver := false;
 showBorders := true; pixelCheck:= true;
end;

procedure TfrmMain.oScreenInit(Sender: TObject);
begin
 oBitmapFont.MakeFont('Courier New', DEFCHARACTERS, 10, 0);
end;

end.



