unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  OmegaScreen, OmegaInput, OmegaTimer, ExtCtrls, OmegaSurface,
  OmegaImageList, OmegaFont, OmegaSprite, OmegaBitmapFont,
  OmegaAnimationList, OmegaSound3D;

type
  TfrmMain = class(TForm)
    scr: TOmegaScreen;
    inp: TOmegaInput;
    tmr: TOmegaTimer;
    spr: TOmegaSprite;
    iml: TOmegaImageList;
    anl: TOmegaAnimationList;
    procedure FormCreate(Sender: TObject);
    procedure tmrTimer(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    x, y: Single;
    sprOmega, sprSmiley: TAdvancedAnimationSprite;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  self.ClientWidth := 640;
  self.ClientHeight := 480;
  scr.Width := self.ClientWidth;
  scr.Height := self.ClientHeight;
  scr.Init;
  inp.Initialize;
  iml.Init;
  //Here are the animations! I used '-1' where values were not used by the
  //component anyway because of the specified actions. You don't have to do this
  //it is just here to illustrate better what parameters are important.

  //As an introduction: A simple smiley animation that first fades in,
  //then animates and then keeps doing the same while rolling around
  with anl.Animations.Add('smiley').Frames do begin
    AddPlay(4.000, 'fadein', [aaImage, aaAlpha],  0, 'smiley', -1, -1,   0);
    AddPlay(0.250, 'animate',[aaImage, aaAlpha],  0, 'smiley', -1, -1, 255);
    AddPlay(0.250, '',       [aaImage],           1, 'smiley');
    AddPlay(0.250, '',       [aaImage],           2, 'smiley');
    AddPlay(0.250, '',       [aaImage],           3, 'smiley');
    AddPlay(0.250, '',       [aaImage],           4, 'smiley');
    AddPlay(0.250, '',       [aaImage],           5, 'smiley');
    AddPlay(0.250, '',       [aaImage],           6, 'smiley');
    AddPlay(0.250, '',       [aaImage],           7, 'smiley');
    AddPlay(0.250, 'start',  [aaImage, aaRotate], 0, 'smiley', -1, -1, 255,   0);
    AddPlay(0.250, '',       [aaImage, aaRotate], 1, 'smiley', -1, -1,  -1,  45);
    AddPlay(0.250, '',       [aaImage, aaRotate], 2, 'smiley', -1, -1,  -1,  90);
    AddPlay(0.250, '',       [aaImage, aaRotate], 3, 'smiley', -1, -1,  -1, 135);
    AddPlay(0.250, '',       [aaImage, aaRotate], 4, 'smiley', -1, -1,  -1, 180);
    AddPlay(0.250, '',       [aaImage, aaRotate], 5, 'smiley', -1, -1,  -1, 225);
    AddPlay(0.250, '',       [aaImage, aaRotate], 6, 'smiley', -1, -1,  -1, 270);
    AddPlay(0.250, '',       [aaImage, aaRotate], 7, 'smiley', -1, -1,  -1, 315);
    AddPlay(0.250, '',       [aaRotate],          -1,      -1, -1, -1,  -1, 360); //notice that it spins back here in 0.250 secs. If you want it to roll continuously, just set 0.250 to 0 !
    AddGotoFrame('start');
  end;

  //O M E G A  showing slowly animation
  with anl.Animations.Add('blinkslow').Frames do begin
    AddPlay(0.400, 'start', [aaImage, aaPosition, aaAlpha], 0, 'omegatext',   0,  0, 255);
    AddPlay(0.000, '',      [aaImage, aaPosition, aaAlpha], 0, 'omegatext',   0, 16,   0);
    AddPlay(0.400, '',      [aaImage, aaPosition, aaAlpha], 1, 'omegatext',  32,  0, 255);
    AddPlay(0.000, '',      [aaImage, aaPosition, aaAlpha], 1, 'omegatext',  32, 16,   0);
    AddPlay(0.400, '',      [aaImage, aaPosition, aaAlpha], 2, 'omegatext',  64,  0, 255);
    AddPlay(0.000, '',      [aaImage, aaPosition, aaAlpha], 2, 'omegatext',  64, 16,   0);
    AddPlay(0.400, '',      [aaImage, aaPosition, aaAlpha], 3, 'omegatext',  96,  0, 255);
    AddPlay(0.000, '',      [aaImage, aaPosition, aaAlpha], 3, 'omegatext',  96, 16,   0);
    AddPlay(0.400, '',      [aaImage, aaPosition, aaAlpha], 4, 'omegatext', 128,  0, 255);
    AddPlay(0.000, '',      [aaImage, aaPosition, aaAlpha], 4, 'omegatext', 128, 16,   0);
    AddPlay(0.000, '',      [aaAlpha],                     -1,          -1,  -1, -1, 255); //Quickly go back to visible
    AddLoop('start', 2);
    AddGotoAnim(2); //We need to use a number here as 'blinkfast' which this one refers to is not known yet at this point!
  end;
  //O M E G A  showing very fast animation
  with anl.Animations.Add('blinkfast').Frames do begin
    AddPlay(0.200, 'start', [aaImage, aaPosition], 0, 'omegatext',   0, 0);
    AddPlay(0.100, '',      [aaImage, aaPosition], 1, 'omegatext',  32, 0);
    AddPlay(0.100, '',      [aaImage, aaPosition], 2, 'omegatext',  64, 0);
    AddPlay(0.100, '',      [aaImage, aaPosition], 3, 'omegatext',  96, 0);
    AddPlay(0.200, '',      [aaImage, aaPosition], 4, 'omegatext', 128, 0);
    AddPlay(0.000, '',      [aaPosition],         -1,          -1, 160, 0);
    AddLoop('start', 10);
    AddGotoAnim('blinkslow');
  end;
  //Generic rotating animation
  with anl.Animations.Add('rotate').Frames do begin
    AddPlay(2.000, 'start', [aaRotate], -1, -1, -1, -1, -1,   0);
    AddPlay(0.000, ''     , [aaRotate], -1, -1, -1, -1, -1, 360);
    AddGotoFrame('start');
  end;
  //Generic scaling animation
  with anl.Animations.Add('scale').Frames do begin
    AddPlay(1.000, 'start', [aaScale], -1, -1, -1, -1, -1, -1,  1.0,  1.0);
    AddPlay(0.500, '',      [aaScale], -1, -1, -1, -1, -1, -1, 10.0, 10.0);
    AddPlay(0.500, '',      [aaScale], -1, -1, -1, -1, -1, -1,  1.0,  1.0);
    AddPlay(0.500, '',      [aaScale], -1, -1, -1, -1, -1, -1,  4.0,  4.0);
    AddGotoFrame('start');
  end;

  //Setup sprites
  sprOmega := TAdvancedAnimationSprite.Create(spr);
  sprOmega.Image:=iml.ImageList.Items[0];
  sprOmega.PlayerCount := 2;
  sprOmega.Players[0].Animation := anl.Animations.Find('blinkslow');
  //Make omega sprite somwhat bigger to fill screen
  sprOmega.ScaleX := 4;
  sprOmega.ScaleY := 4;

  sprSmiley := TAdvancedAnimationSprite.Create(spr);
  sprSmiley.Image:=iml.ImageList.Items[1];
  sprSmiley.PlayerCount := 2;
  sprSmiley.Players[0].Animation := anl.Animations.Find('smiley');
  //Put smiley in center and make it big
  sprSmiley.BaseX := 304;
  sprSmiley.BaseY := 224;
 
  //Start timer
  tmr.Enabled := True;
end;

procedure TfrmMain.tmrTimer(Sender: TObject);
begin
  scr.BeginRender;
  scr.ClearScreen(0,0,0,255);

  //Move
  inp.Update;
  if oisLeft in inp.Keyboard.States then x := x - tmr.Delta * 100;
  if oisRight in inp.Keyboard.States then x := x + tmr.Delta * 100;
  if oisUp in inp.Keyboard.States then y := y - tmr.Delta * 100;
  if oisDown in inp.Keyboard.States then y := y + tmr.Delta * 100;
  //Look how easy we add another 'layer' of animation to add effects!
  if oisButton1 in inp.Keyboard.States then
    sprOmega.Players[1].Animation := anl.Animations.Find('rotate');
  if oisButton2 in inp.Keyboard.States then
    sprSmiley.Players[1].Animation := anl.Animations.Find('scale');
  //Update base x and y position of omega sprite to illustrate movement
  //independant of animation positions
  sprOmega.BaseX := x;
  sprOmega.BaseY := y;

  iml.BeginDraw;

  spr.Move(tmr.Delta);
  spr.Draw;
  iml.EndDraw;
  scr.EndRender;
end;

end.
