unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, OmegaMusic, ExtCtrls;

type
  TMainForm = class(TForm)
    lbxFiles: TListBox;
    btnAdd: TButton;
    btnRemove: TButton;
    OpenFile: TOpenDialog;
    OmegaMusic: TOmegaMusic;
    btnPlay: TButton;
    btnStop: TButton;
    btnPause: TButton;
    cbxLoop: TCheckBox;
    scrVolume: TScrollBar;
    scrPan: TScrollBar;
    lblVol: TLabel;
    lblPan: TLabel;
    tmrLoop: TTimer;
    lblIsPlaying: TLabel;
    scrPos: TScrollBar;
    procedure FormCreate(Sender: TObject);
    procedure tmrLoopTimer(Sender: TObject);
    procedure btnPlayClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure btnPauseClick(Sender: TObject);
    procedure cbxLoopClick(Sender: TObject);
    procedure scrVolumeChange(Sender: TObject);
    procedure scrPanChange(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure btnRemoveClick(Sender: TObject);
    procedure lbxFilesClick(Sender: TObject);
    procedure scrPosScroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  // call before anything else
  OmegaMusic.Init;
end;

procedure TMainForm.tmrLoopTimer(Sender: TObject);
var
  Selected: Integer;
  d1,d2: Double;
begin
  // loop the songs
  OmegaMusic.Loop;

  // find which item we are trying to access
  Selected := lbxFiles.ItemIndex;
  if Selected < 0 then Exit;

  // if we are playing the current song let us know
  if (OmegaMusic.Songs.Items[Selected].IsPlaying) and (OmegaMusic.Songs.Items[Selected].Position > 0) then
     lblIsPlaying.Caption := 'it''s playing' else lblIsPlaying.Caption := 'not playing';

  // find position and duration of current song
  d1 := OmegaMusic.Songs.Items[Selected].Position;
  d2 := OmegaMusic.Songs.Items[Selected].Duration;

  // change the position of the scrollbar so that it displays current position
  scrPos.Position := Trunc(100 * (d1 / d2));
end;

procedure TMainForm.btnPlayClick(Sender: TObject);
var
  Selected: Integer;
begin
  // find which item we are trying to access
  Selected := lbxFiles.ItemIndex;
  if Selected < 0 then Exit;

  // play song
  OmegaMusic.Songs.Items[Selected].Play;
end;

procedure TMainForm.btnStopClick(Sender: TObject);
var
  Selected: Integer;
begin
  // find which item we are trying to access
  Selected := lbxFiles.ItemIndex;
  if Selected < 0 then Exit;

  // stop song
  OmegaMusic.Songs.Items[Selected].Stop;
end;

procedure TMainForm.btnPauseClick(Sender: TObject);
var
  Selected: Integer;
begin
  // find which item we are trying to access
  Selected := lbxFiles.ItemIndex;
  if Selected < 0 then Exit;

  // pause song
  OmegaMusic.Songs.Items[Selected].Pause;
end;

procedure TMainForm.cbxLoopClick(Sender: TObject);
var
  Selected: Integer;
begin
  // find which item we are trying to access
  Selected := lbxFiles.ItemIndex;
  if Selected < 0 then Exit;

  // loop song
  OmegaMusic.Songs.Items[Selected].Loop := cbxLoop.Checked;
end;

procedure TMainForm.scrVolumeChange(Sender: TObject);
var
  Selected: Integer;
begin
  // find which item we are trying to access
  Selected := lbxFiles.ItemIndex;
  if Selected < 0 then Exit;

  // change volume of song
  OmegaMusic.Songs.Items[Selected].Volume := scrVolume.Position;
  lblVol.Caption := IntToStr(scrVolume.Position);
end;

procedure TMainForm.scrPanChange(Sender: TObject);
var
  Selected: Integer;
begin
  // find which item we are trying to access
  Selected := lbxFiles.ItemIndex;
  if Selected < 0 then Exit;

  // change pan of song
  OmegaMusic.Songs.Items[Selected].Pan := scrPan.Position;
  lblPan.Caption := IntToStr(scrPan.Position);
end;

procedure TMainForm.btnAddClick(Sender: TObject);
begin
  if OpenFile.Execute then
  begin
    // new item
    OmegaMusic.Songs.Add;
    // load file
    OmegaMusic.Songs.Items[OmegaMusic.Songs.Count-1].Load(OpenFile.FileName);
    // initialise song
    OmegaMusic.Songs.Items[OmegaMusic.Songs.Count-1].Init;
    // add to listbox
    lbxFiles.Items.Add(OpenFile.FileName);
  end;
end;

procedure TMainForm.btnRemoveClick(Sender: TObject);
var
  Selected: Integer;
begin
  // find which item we are trying to access
  Selected := lbxFiles.ItemIndex;
  if Selected < 0 then Exit;

  // delete song
  OmegaMusic.Songs.Delete(Selected);
  lbxFiles.Items.Delete(Selected);
end;

procedure TMainForm.lbxFilesClick(Sender: TObject);
var
  selected: Integer;
begin
  // find which item we are trying to access
  Selected := lbxFiles.ItemIndex;
  if Selected < 0 then Exit;

  // change values so that they display the current song's properties
  scrVolume.Position := OmegaMusic.Songs.Items[Selected].Volume;
  scrPan.Position := OmegaMusic.Songs.Items[Selected].Pan;
  cbxLoop.Checked := OmegaMusic.Songs.Items[Selected].Loop;
end;

procedure TMainForm.scrPosScroll(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: Integer);
var
  d: Double;
  selected: Integer;
begin
  // find which item we are trying to access
  Selected := lbxFiles.ItemIndex;
  if Selected < 0 then Exit;

  // change position of song
  d := OmegaMusic.Songs.Items[Selected].Duration;
  OmegaMusic.Songs.Items[Selected].Position := (d/100) * ScrollPos;
end;

end.
