unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaImageList, OmegaScreen, OmegaTimer,
  OmegaBitmapFont, OmegaSprite, OmegaParticleEngine, OmegaUtils, OmegaInput;

type
  TStar = class(TParticle)
  public
    xpos, ypos, zpos, speed: single;
    procedure Update(MoveCount: Single); override;
    procedure Reset; override;
  end;

  TForm1 = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaImageList1: TOmegaImageList;
    OmegaTimer1: TOmegaTimer;
    OmegaBitmapFont1: TOmegaBitmapFont;
    OmegaParticleEngine1: TOmegaParticleEngine;
    OmegaSprite1: TOmegaSprite;
    OmegaInput1: TOmegaInput;
    procedure FormCreate(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure OmegaScreen1Init(Sender: TObject);
  end;

var Form1: TForm1;
    sfacx, sfacy, angle: single;
    bm, sp, MCenterX, MCenterY: integer;
    rot, MaxParticles: integer;
    showhelp, colored, moving: boolean;
    LastTick: DWORD;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
 Randomize;
 moving := false;
 colored := false;
 showhelp := true;
 sp := 50;
 bm := 0;
 rot := 1;
 self.ClientWidth := 800;
 self.ClientHeight := 600;
 OmegaScreen1.Width := self.ClientWidth;
 OmegaScreen1.Height := self.ClientHeight;
 OmegaScreen1.Init;
 OmegaInput1.Initialize;
 with OmegaParticleEngine1.EngineList.Items[0] do
 begin
  Sprite.Image := OmegaImageList1.ImageList.Items[3];
  Sprite.ImageIndex := -1;
  Sprite.BlendMode := bmIgnore;
  UseCustomParticle(TStar);
  NumOfParticles := 10;
  Particle.LifeTime := 1;
  Particle.Decay := 0;
 end;
 MCenterX := 400;
 MCenterY := 300;
 MaxParticles := 300;
 OmegaParticleEngine1.Init;
 sfacx := 1;
 sfacy := 1;
 OmegaTimer1.Enabled := True;
end;

procedure TForm1.OmegaTimer1Timer(Sender: TObject);
var s, c,  // sine and cosine variables
    t, t2: single; // temporary variables
    Tick: DWORD;
begin
 if not OmegaScreen1.CanDraw then exit;
 OmegaInput1.update;
 if oisbutton1 in OmegaInput1.keyboard.statesclicked then
  OmegaScreen1.ToggleFullScreen;
 Tick := GetTickCount - LastTick;
 LastTick := GetTickCount;
 OmegaParticleEngine1.UpdateAll(Tick * 0.1);
 with OmegaParticleEngine1.Enginelist.Items[0] do
 begin
  while NumOfParticles < MaxParticles do NumOfParticles := NumOfParticles + 1;
  while NumOfParticles > MaxParticles do NumOfParticles := NumOfParticles - 1;
 end;

 OmegaScreen1.ClearScreen(0, 0, 0, 0);
 OmegaScreen1.BeginRender;
 if moving then
 begin
  angle := angle + 0.5;
  if angle >= 360 then angle := angle - 360;
  s := sin(angle * pi / 180);
  c := cos(angle * pi / 180);
  t  := 400 + 200 * s;
  t2 := 300 + 150 * c;
  MCenterX := round(t);
  MCenterY := round(t2);
 end;

 OmegaScreen1.SetAntiAliasing(true);
 OmegaScreen1.SetBlendMode(bm);
 OmegaImageList1.ImageList.Items[1].Draw(MCenterX - 50, MCenterY - 50, -1, bmIgnore);
 OmegaSprite1.Draw;
 OmegaSprite1.Move(0);
 OmegaSprite1.Dead;

 if showhelp then
 begin
  OmegaScreen1.SetAntiAliasing(false);
  OmegaBitmapFont1.Print(60, 340, '"F" for a cool effect');
  OmegaBitmapFont1.Print(60, 360, '"B" for different blending modes');
  OmegaBitmapFont1.Print(60, 380, '"1" to "4" for different stars');
  OmegaBitmapFont1.Print(60, 400, '"M" for movement on/off');
  OmegaBitmapFont1.Print(60, 420, '"C" for coloring on/off');
  OmegaBitmapFont1.Print(60, 440, '"Q" and "W" for -/+ stars');
  OmegaBitmapFont1.Print(60, 460, '"A" and "S" for -/+ speed');
  OmegaBitmapFont1.Print(60, 480, '"Y" and "X" for -/+ size');
  OmegaBitmapFont1.Print(60, 500, '"R" for random');
  OmegaBitmapFont1.Print(60, 520, '"H" to toggle this help on/off');
  OmegaBitmapFont1.Print(60, 540, '"Space" for fullscreen');
 end;
 OmegaScreen1.EndRender;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 OmegaTimer1.Enabled := False;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var a: integer;
begin
 if key = 27 then close;
 if key = ord('C') then colored := not colored;
 if key = ord('H') then showhelp := not showhelp;
 if key = ord('M') then
 begin
  moving := not moving;
  if not moving then
  begin
   MCenterX := 400;
   MCenterY := 300;
  end;
 end;
 if key = ord('1') then
 begin
  OmegaParticleEngine1.EngineList.Items[0].Sprite.Image := OmegaImageList1.ImageList.Items[1];
  sfacx := 0.5;
  sfacy := 0.5;
 end;
 if key = ord('2') then
 begin
  OmegaParticleEngine1.EngineList.Items[0].Sprite.Image := OmegaImageList1.ImageList.Items[0];
  sfacx := 3;
  sfacy := 3;
 end;
 if key = ord('3') then
 begin
  OmegaParticleEngine1.EngineList.Items[0].Sprite.Image := OmegaImageList1.ImageList.Items[2];
  sfacx := 0.7;
  sfacy := 0.7;
 end;
 if key = ord('4') then
 begin
  OmegaParticleEngine1.EngineList.Items[0].Sprite.Image := OmegaImageList1.ImageList.Items[3];
  sfacx := 1;
  sfacy := 1;
 end;
 if key = ord('B') then bm := (bm + 1) mod 5;
 if key = ord('A') then if sp > 10 then sp := sp - 10;
 if key = ord('Y') then
 begin
  sfacx := sfacx - 0.1;
  sfacy := sfacy - 0.1;
 end;
 if key = ord('X') then
 begin
  sfacx := sfacx + 0.1;
  sfacy := sfacy + 0.1;
 end;
 if key = ord('S') then if sp < 100 then sp := sp + 10;
 if key = ord('R') then
 begin
  MCenterX := 400;
  MCenterY := 300;
  sfacx := 0.3 + random * 9;
  bm := random(6); if bm = 5 then bm := 2;
  sp := 10 + random(100);
  MaxParticles := random(1400) + 100;
  a := random(4);
  OmegaParticleEngine1.EngineList.Items[0].Sprite.Image := OmegaImageList1.ImageList.Items[a];
  if (a = 1) or (a = 3) then sfacx := sfacx * 0.3;
  sfacy := sfacx + random * 0.3;
  moving := random < 0.5;
  colored := random < 0.5;
  rot := random(3);
 end;
 if key = ord('W') then
  if MaxParticles < 1500 then MaxParticles := MaxParticles + 50;
 if key = ord('Q') then
  if MaxParticles > 50 then MaxParticles := MaxParticles - 50;
 if key = ord('F') then
 begin
  OmegaParticleEngine1.EngineList.Items[0].Sprite.Image := OmegaImageList1.ImageList.Items[3];
  sfacx := 20 + random(30);
  sfacy := sfacx + random(10);
  bm := 4;
  sp := 150;
  MCenterX := 400;
  MCenterY := 300;
  MaxParticles := 70;
  moving := true;
  colored := true;
  rot := random(10);
  OmegaParticleEngine1.EngineList.Items[0].Clear;
 end;
end;

{ TStar }

procedure TStar.Reset;
begin
 inherited;
 xpos := (-15 + (30 * random)) * 4000;
 ypos := (-15 + (30 * random)) * 4000;
 zpos := 254;
 speed := 0.2 + random * 2;
 speed := speed *  (sp / 50);
 alpha := 0;
 ScaleX := 40;
 ScaleY := 40;
 Rotation := random(360);
 if colored then
 begin
  red := random(256);
  green := random(256);
  blue := random(256);
 end else
 begin
  red := 255;
  green := 255;
  blue := 255;
 end;
end;

procedure TStar.Update(MoveCount: Single);
var tempx, tempy: integer;
begin
 inherited;
 zpos := zpos - speed;
 if (zpos <= -10) then
 begin
  zpos := -10;
  reset;
 end;
 Rotation := Rotation + rot;
 tempx := round(xpos / zpos) + MCenterX;
 tempy := round(ypos / zpos) + MCenterY;
 if (tempx < -300) or (tempx > 1200)
 or (tempy < -300) or (tempy > 1100) then reset;
 x := tempx;
 y := tempy;
 alpha := 255 - round(zpos);
 ScaleX := (alpha / 200) * sfacx;
 ScaleY := (alpha / 200) * sfacy;
end;

procedure TForm1.OmegaScreen1Init(Sender: TObject);
begin
 OmegaBitmapFont1.MakeFont('Arial', DEFCHARACTERS, 10, D3DFONT_BOLD);
 ColorToAlpha(OmegaImageList1.ImageList.Items[0].image, 1, 0, 0, 0);
 ColorToAlpha(OmegaImageList1.ImageList.Items[1].image, 1, 0, 0, 0);
 ColorToAlpha(OmegaImageList1.ImageList.Items[2].image, 1, 0, 0, 0);
 ColorToAlpha(OmegaImageList1.ImageList.Items[3].image, 1, 0, 0, 0);
end;

end.


