unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, OmegaSound3D, StdCtrls, Menus, ImgList;

type
  TMovingSound = class
    Xpos, Zpos: Single;
    Dir: Integer;
    procedure Draw(Canv: TCanvas);
  end;

  TStationarySound = class
    Xpos, Zpos: Single;
    procedure Draw(Canv: TCanvas);
  end;

  TPlayer = class
    Xpos, Zpos: Single;
    procedure Draw(Canv: TCanvas);
  end;

type
  TForm1 = class(TForm)
    Sound3D: TOmegaSound3D;
    PaintBox1: TPaintBox;
    Timer1: TTimer;     
    Label1: TLabel;
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormActivate(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  Player: TPlayer;
  MovingSound: TMovingSound;
  StationarySound: TStationarySound;
  Keys: array[0..255] of Boolean;

implementation

{$R *.dfm}

procedure TMovingSound.Draw(Canv: TCanvas);
var
  x,y,w,h: Integer;
begin
  x := Trunc(Xpos-10);
  y := Trunc(Zpos-10);
  w := 40;
  h := 20;
  Canv.Rectangle(Bounds(x,y,w,h));
  Canv.TextOut(x+2, y+2, 'hit.wav');
end;

procedure TStationarySound.Draw(Canv: TCanvas);
var
  x,y,w,h: Integer;
begin
  x := Trunc(Xpos-15);
  y := Trunc(Zpos-10);
  w := 50;
  h := 20;
  Canv.Rectangle(Bounds(x,y,w,h));
  Canv.TextOut(x+2, y+2, 'test.mid');
end;

procedure TPlayer.Draw(Canv: TCanvas);
var
  x,y,w,h: Integer;
begin
  x := Trunc(Xpos-10);
  y := Trunc(Zpos-10);
  w := 20;
  h := 20;
  Canv.Ellipse(Bounds(x,y,w,h));
  Canv.TextOut(x+2, y+2, 'YOU!');
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  if Keys[VK_LEFT] then Player.Xpos := Player.Xpos - 2;
  if Keys[VK_RIGHT] then Player.Xpos := Player.Xpos + 2;
  if Keys[VK_UP] then Player.Zpos := Player.Zpos - 2;
  if Keys[VK_Down] then Player.Zpos := Player.Zpos + 2;

  PaintBox1.Canvas.FillRect(Rect(PaintBox1.Left, PaintBox1.Top, PaintBox1.Width, PaintBox1.Height));

  StationarySound.Draw(PaintBox1.Canvas);
  MovingSound.Draw(PaintBox1.Canvas);
  Player.Draw(PaintBox1.Canvas);

  // set position of the listener
  Sound3D.ListenerPositionX := Player.Xpos / 50;
  Sound3D.ListenerPositionZ := Player.Zpos / 50;

  // set position of "hit.wav"
  Sound3D.AudioPathList.Items[0].SoundPositionX := MovingSound.Xpos / 50;
  Sound3D.AudioPathList.Items[0].SoundPositionZ := MovingSound.Zpos / 50;

  // set position of "test.mid"
  Sound3D.AudioPathList.Items[1].SoundPositionX := StationarySound.Xpos / 50;
  Sound3D.AudioPathList.Items[1].SoundPositionZ := StationarySound.Zpos / 50;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Player := TPlayer.Create;
  StationarySound := TStationarySound.Create;
  MovingSound := TMovingSound.Create;

  Player.Xpos := PaintBox1.Width / 2;
  Player.Zpos := PaintBox1.Height / 2;

  StationarySound.Xpos := 300;
  StationarySound.Zpos := 100;

  MovingSound.Xpos := 50;
  MovingSound.Zpos := 50;
  MovingSound.Dir := 1;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Sound3D.SoundList.Items[0].Stop;
  Sound3D.SoundList.Items[1].Stop;
  StationarySound.Free;
  MovingSound.Free;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  Keys[Key] := True;
  if Key = ord('1') then Sound3D.ModList.Items[0].Play;
  if Key = ord('2') then Sound3D.ModList.Items[0].Stop;
  if Key = 32 then begin
    Sound3D.SoundList.Items[1].Stop;
    Sound3D.SoundList.Items[1].Play;
  end;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  Keys[Key] := False;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
  // play sounds
  Sound3D.Init;
  Sound3D.SoundList.Items[0].Play;
  Sound3D.SoundList.Items[1].Play;
end;

end.
