(*
             Red Ant Productions 2003

   ModPlug Sound System Library for Delphi v1.0
   ============================================

     Package Dated: 20 May 2003

     This unit is based on the source used in ModPlug Player and is free for
  commercial and non-commercial use of your programs as stated by the author
  of the mppsdk dynamic library and the original C Header, Oliver Lapicque.

     No royalties or fees are required for the use of this source and the
  accompanying library files. I would however ask that any redistrubution of
  this package include all files and documents contained within un-altered and
  in it's original compression file.

    C header and DLL library programmed by Olivier Lapicque <olivierl@jps.net>
    Some code was based on original conversion to Delphi by Dean Ellis
    ModPlugSDK Unit written by: Jason McMillen <raproductions@hotmail.com>

*)
unit ModPlugSDK;

interface

uses
  Windows, Classes;

const
  MPPSDKLibrary = 'mppsdk.dll';

  MPPAPI_Version = $0147;

// Version-Specific Functions
  MPPVersion_Has_Navigation = $0139;
  MPPVersion_Has_SongTime = $0141;
  MPPVersion_Has_PreAmp = $0146;
  MPPVersion_Has_RepeatCount = $0147;

// Error Codes
  MPPError_NoError = 0;
  MPPError_Failed = 1;
  MPPError_InvalidParam = 2;

  MPPSONG_Invalid = 0;
  MPPSONG_MOD = 1;
  MPPSONG_S3M = 2;
  MPPSONG_XM = 3;
  MPPSONG_IT = 4;
  MPPSONG_MDL = 5;
  MPPSONG_Unknown = 100;

// Mixer Options MPPMIX_XXXX
  MPPMIX_NoResampling = $01; // Faster, but crappy quality
  MPPMIX_BassExpansion = $02; // Bass Expansion
  MPPMIX_Surround = $04; // Surround Encoding
  MPPMIX_Reverb = $08; // Reverb
  MPPMIX_Loop = $10; // Loop the song (backward jumps will be enabled)
  // v1.40+ flags
  MPPMIX_HighQuality = $20; // HQ mixing (better resampling, dithering enabled)
  MPPMIX_GainControl = $40; // Automatic Gain Control
  MPPMIX_NoiseReduction = $80; // Noise reduction (-6dB 22kHz lowpass filter)

type
  MPPError = Integer;

  PModMixer = ^IModMixer;
  IModMixer = class(TObject)
  public
    // Reference count: the initial reference count is 1, so you shouldn't have to call AddRef()
    function AddRef: Cardinal; virtual; stdcall; abstract;
    function Release: Cardinal; virtual; stdcall; abstract;
    // API Version: you should refuse to continue if the returned value is smaller than MPPAPI_VERSION
    function GetVersion: Cardinal; virtual; stdcall; abstract;
    // Basic I/O Functions
    function LoadSong(const MemFile: Pointer; Length: Integer): MPPError; virtual; stdcall; abstract;
     // Songs are always loaded from memory. The pointer pmemfile can be destroyed
     // after the call to LoadSong. You can use memory-mapped files or a pointer returned
     // by LockResource(), or whatever file in memory.

    // Free the memory used by the song
    function FreeSong: MPPError; virtual; stdcall; abstract;
    // Audio Rendering Functions: example: (44100, 2, 16) for 44.1kHz, stereo, 16-bit
    function SetWaveFormat(SamplesPerSecond, Channels, BitsPerSample: Integer): MPPError; virtual; stdcall; abstract;
     // return # of SAMPLES that have been written to the buffer, 0 if end has been reached
     // Note: protect calls to Render() and SetMixerOptions() by a critical section, if they
     //       are used in different threads.
    function Render(Buffer: Pointer; BufferSize: Cardinal): MPPError; virtual; stdcall; abstract;
    // Player Configuration: set of MPPMIX_XXXX
    function SetMixerOptions(dwOptions: Cardinal): MPPError; virtual; stdcall; abstract;
    function GetMixerOptions: Cardinal; virtual; stdcall; abstract;
    // Song Information
    function GetSongType: Cardinal; virtual; stdcall; abstract; // Return MPPSONG_XXXX
    // Buffer must be at least 32-bytes
    procedure GetSongName(Buffer: PChar); virtual; stdcall; abstract;
    // v1.39+: Navigation functions
     { The order is the position in the pattern sequence list: this allows you
       to jump to a specific part of a song. It can be useful in a game with a
       song that uses pattern position jump effects (or pattern loops).
       These function will not be available if GetVersion() returns a value
       smaller than $139 (MPPVersion_Has_Navigation)}
    function GetNumOrders: Cardinal; virtual; stdcall; abstract;
    function GetCurrentOrder: Cardinal; virtual; stdcall; abstract;
    function SetCurrentOrder(NewOrder: Cardinal): MppError; virtual; stdcall; abstract;
    //////////////////////////////////////////////////////////////////////////////////////
    // v1.41+: Returns the song length in seconds
    function GetSongLength: Cardinal; virtual; stdcall; abstract;

    //////////////////////////////////////////////////////////////////////////////////////
    // v1.46+: Set Pre-Amp level, in percent (25% - 400%)
    procedure SetPreAmpLevel(PreAmp: Cardinal); virtual; stdcall; abstract;

    //////////////////////////////////////////////////////////////////////////////////////
    // v1.47+: Set repeat count (-1 = Infinite)
    procedure SetRepeatCount(RepeatCount: Integer); virtual; stdcall; abstract;
  end;

var
  MPPSDKDLL_Loaded: Boolean;
//!  MODMixer: IModMixer;

function InitModPlugSDKLibrary(var MODMixer: IModMixer): Boolean;
procedure DeInitModPlugSDKLibrary(var MODMixer: IModMixer);

implementation

uses
  SysUtils;

var
  MPPSDKLibHandle: THandle;
  ModPlugTimesLoaded: Integer;

  GetModAPI: function(MODMixer: PModMixer): MPPError; cdecl;


{----------------------------------------------------------------------------}
{InitLibrary - Will try to load the mppsdk.dll and get the ModMixer object}
{----------------------------------------------------------------------------}

function InitModPlugSDKLibrary(var MODMixer: IModMixer): Boolean;
begin
  Result := False;

  if (MPPSDKLibHandle = 0) then
    MPPSDKLibHandle := LoadLibrary(MPPSDKLibrary);

  if (MPPSDKLibHandle = 0) then
    Exit;

  try
    GetModAPI := GetProcAddress(MPPSDKLibHandle, 'MPP_GetModAPI');
    GetModAPI(@MODMixer);
    Inc(ModPlugTimesLoaded);
    Result := True;
  except
    Result := False;
  end;
end;

{----------------------------------------------------------------------------}
{DeInitLibrary - Will unload the mppsdk.dll if it was loaded}
{----------------------------------------------------------------------------}

procedure DeInitModPlugSDKLibrary(var MODMixer: IModMixer);
begin
  Dec(ModPlugTimesLoaded);
  if ModPlugTimesLoaded = 0 then
    FreeLibrary(MPPSDKLibHandle);
end;

{----------------------------------------------------------------------------}
{Automatic initialization and finalization - comment out if you want to
 do this manually}
{----------------------------------------------------------------------------}
initialization
  ModPlugTimesLoaded := 0;
//!  MPPSDKDLL_Loaded := InitLibrary;
finalization
//!  DeInitLibrary;
end.
