unit OmegaBitmapFont;
{
//****************************************************************
// OmegaBitmapFont unit
// author : Rheenen (original version)
//          Tobias Fleischer (updates and extensions)
// contact: tobybear@web.de
// comments are welcome!
//****************************************************************

version history:
05/11/05: speed improvement idea Print4Col (Athena),
          new Print4ColFast and PrintFast overloaded methods (Bazi)
05/10/04: added line break/multiple line support
04/07/04: improved TextWidth and TextHeight functions
03/21/04: added better TextWidth function
02/10/04: added CenterX and CenterY properties
01/26/04: added TextWidth and TextHeight functions
01/14/04: added optional blendmode setting to every print method
09/19/03: more fixes
09/03/03: some fixes for initialisation
09/02/03: now using new renderer in OmegaScreen
08/31/03: added proper Init/UnInit functions and notification
08/24/03: some more bugs fixed!
08/19/03: fixed nasty bug: OmegaImageIndex initialized with -1!
08/18/03: some bugfixes to make it compatible with old version
08/13/03: first release

"todo":
- add proper documentation
}

interface
uses Classes, OmegaImageList, OmegaUtils,
 Windows, OmegaRenderer, OmegaScreen, D3DX8,
 {$I UseD3D8.inc};

const
  DEFCHARACTERS2 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
  DEFCHARACTERS = ' !"#$%&''()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ'+
  '[\]^_`abcdefghijklmnopqrstuvwxyz{|}~';

  // Font creation flags
  D3DFONT_BOLD         = 1;
  D3DFONT_ITALIC       = 2;
  D3DFONT_UNDERLINE    = 4;
  D3DFONT_STRIKEOUT    = 8;
  D3DFONT_NOANTIALIAS  = 16;
  D3DFONT_EQUALSPACED  = 32;

type
  TBitmapFontPositionMode = (pmTopLeft, pmCenter);
  TOmegaBitmapFont = class(TComponent)
  private
    FDevice: IDirect3DDevice8;
    FRenderer: TOmegaRenderer;
    TexWidth: DWORD;
    TexHeight: DWORD;
    TexCoords: array[0..255, 0..3] of Single;
    Coords: array[0..255] of TPoint;
    FCenterX, FCenterY, FTextScale: Single;
    FOmegaImageList: TOmegaImageList;
    FOmegaImageIndex: Integer;
    FOwnTexture: boolean;
    FTexture: IDirect3DTexture8;
    FCharacters: String;
    FMapping: array[0..255] of Integer;
    FCharWidth, FCharHeight: Integer;
    FRed, FGreen, FBlue, FAlpha: Integer;
    FScaleX, FScaleY: Single;
    FRotation, FCharRotation: Single;
    FFontName: string;
    FSpacing: integer;
    FCol1, FCol2, FCol3, FCol4: TOmegaColor;
    FPositionMode: TBitmapFontPositionMode;
    procedure SetCharacters(Value: String);
    procedure SetOmegaImageIndex(const Value: Integer);
    procedure SetOmegaImageList(const Value: TOmegaImageList);
    procedure SetOmegaScreen(const Value: TOmegaScreen);
    procedure SetAlpha(const Value: Integer);
    procedure SetBlue(const Value: Integer);
    procedure SetGreen(const Value: Integer);
    procedure SetRed(const Value: Integer);
  protected
    FOmegaScreen: TOmegaScreen;
    FLineSpace: integer;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    function TextHeight(s: string): integer;
    function TextWidth(s: string): integer;
    procedure LoadBMP(filename: string);
    procedure LoadFNT(filename: string);
    procedure LoadWID(filename: string);
    procedure SaveBMP(filename: string);
    procedure SaveFNT(filename: string);
    procedure SaveWID(filename: string);
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Print(X, Y: integer; txt: String; blendmode: integer = bmNormal); overload;
    procedure Print(X, Y: integer;
     Rotation, ScaleX, ScaleY: single; col: TOmegaColor; txt: String; blendmode: integer = bmNormal); overload;
    procedure Print4Col(X, Y: integer;
     Rotation, ScaleX, ScaleY: single; col1, col2, col3, col4: TOmegaColor;
     txt: String; blendmode: integer = bmNormal);
    procedure Print4ColFast(X, Y: integer; col1, col2, col3, col4: TOmegaColor;
     txt: String; blendmode: integer = bmNormal); // Bazi
    procedure PrintFast(X, Y: integer; col: TOmegaColor;
     txt: String; blendmode: integer = bmNormal); overload; // Bazi
    procedure PrintFast(X, Y: integer;
     txt: String; blendmode: integer = bmNormal); overload; // Bazi
    procedure MakeFont(font, characters: string; height, flags: DWORD);
    procedure Init;
    procedure UnInit;
    property CharWidth: Integer read FCharWidth;
    property CharHeight: Integer read FCharHeight;
  published
    property FontName: string read FFontName;
    property OmegaScreen: TOmegaScreen read FOmegaScreen write SetOmegaScreen;
    property OmegaImageList: TOmegaImageList read FOmegaImageList write SetOmegaImageList;
    property OmegaImageIndex: Integer read FOmegaImageIndex write SetOmegaImageIndex;
    property Characters: String read FCharacters write SetCharacters;
    property Red: Integer read FRed write SetRed;
    property Green: Integer read FGreen write SetGreen;
    property Blue: Integer read FBlue write SetBlue;
    property Alpha: Integer read FAlpha write SetAlpha;
    property LineSpace: Integer read FLineSpace write FLineSpace;
    property CenterX: Single read FCenterX write FCenterX;
    property CenterY: Single read FCenterY write FCenterY;
    property ScaleX: Single read FScaleX write FScaleX;
    property ScaleY: Single read FScaleY write FScaleY;
    property Rotation: Single read FRotation write FRotation;
    property CharRotation: Single read FCharRotation write FCharRotation;
    property Spacing: Integer read FSpacing write FSpacing;
    property PositionMode: TBitmapFontPositionMode read FPositionMode write FPositionMode;
  end;

implementation

{ TOmegaBitmapFont }

constructor TOmegaBitmapFont.Create(AOwner: TComponent);
begin
  FRenderer := TOmegaRenderer.Create;
  FLineSpace := 10;
  inherited;
  FFontName := '';
  FOmegaImageIndex := 0;
  FCol1 := OmegaColor($FFFFFFFF);
  FCol2 := OmegaColor($FFFFFFFF);
  FCol3 := OmegaColor($FFFFFFFF);
  FCol4 := OmegaColor($FFFFFFFF);
  FCenterX := 0;
  FCenterY := 0;
  FRed := 255;
  FGreen := 255;
  FBlue := 255;
  FAlpha := 255;
  FScaleX := 1;
  FScaleY := 1;
  FRotation := 0;
  FCharRotation := 0;
  FPositionMode := pmTopLeft;
  FSpacing := 0;
  FOwnTexture := false;
  FTexture := nil;
end;

destructor TOmegaBitmapFont.Destroy;
begin
 inherited;
 if FTexture <> nil then FTexture := nil;
 if FRenderer <> nil then FRenderer.free;
end;

function GetRect(index, width, height, awidth, aheight: integer): TRect;
var rowcount, colcount: integer;
    rect: TRect;
begin
  rect.Left := 0;
  rect.Right := 0;
  rect.Top := 0;
  rect.Bottom := 0;
  ColCount := aWidth div Width;
  RowCount := aHeight div Height;
  if Index > RowCount * ColCount then Index := RowCount * ColCount;
  if ColCount > 0 then
  begin
   rect.Left := (index mod colcount) * Width;
   rect.Top := (index div colcount) * Height;
  end;
  rect.Right := Rect.Left + Width;
  rect.Bottom := Rect.Top + Height;
  result := rect;
end;

procedure TOmegaBitmapFont.SetOmegaImageIndex(const Value: Integer);
var x, w, h, rw, rh: integer;
    t: TRect;
begin
  if (value < 0) then exit;
  //  if FOmegaImageIndex = Value then Exit;
  FOmegaImageIndex := Value;
  if (FOmegaImageList = nil) or (FOmegaImageIndex > FOmegaImageList.ImageList.Count - 1) then Exit;
  FOwnTexture := false;
  FFontName := FOmegaImageList.ImageList.Items[FOmegaImageIndex].Name;
  FTextScale := 1;
  if FTexture <> nil then FTexture := nil;
  FCharWidth := FOmegaImageList.ImageList.Items[FOmegaImageIndex].TileWidth;
  FCharHeight := FOmegaImageList.ImageList.Items[FOmegaImageIndex].TileHeight;
  rw := FOmegaImageList.ImageList.Items[FOmegaImageIndex].RealWidth;
  rh := FOmegaImageList.ImageList.Items[FOmegaImageIndex].RealHeight;
  w := FOmegaImageList.ImageList.Items[FOmegaImageIndex].ImageWidth;
  h := FOmegaImageList.ImageList.Items[FOmegaImageIndex].ImageHeight;
  for x := 0 to 255 do
  begin
   Coords[x].x := FCharWidth;
   Coords[x].y := FCharHeight;
   t := GetRect(x, FCharWidth, FCharHeight, w, h);
   TexCoords[x][0] := t.Left / rw;
   TexCoords[x][1] := t.Top / rh;
   TexCoords[x][2] := (t.Right) / rw;
   TexCoords[x][3] := (t.Bottom) / rh;
  end;
end;

procedure TOmegaBitmapFont.SetOmegaImageList(const Value: TOmegaImageList);
begin
  if Value = FOmegaImageList then Exit;
  FOmegaImageList := Value;
  if (FOmegaImageList = nil) then Exit;
  SetOmegaImageIndex(FOmegaImageIndex);
end;

procedure TOmegaBitmapFont.SetCharacters(Value: String);
var i: Integer;
begin
  if Value = '' then Value := DEFCHARACTERS;
  FCharacters := Value;
  for i := 0 to 255 do FMapping[i] := -1;
  for i := Length(FCharacters) downto 1 do
   FMapping[Ord(FCharacters[i])] := i-1;
  if (FMapping[Ord(' ')] = -1) then
   FMapping[Ord(' ')] := 0;
end;

procedure TOmegaBitmapFont.MakeFont(font, characters: string;
 height, flags: DWORD);
// Part of this is converted from some old Microsoft SDK,
// hence the strange code :-) But it works...
type pBit = array[0..0] of DWORD;
     PpBit = ^pBit;
var
  d3dCaps: TD3DCaps8;
  pBitmapBits: PpBit;
  bmi: TBitmapInfo;
  hDC_: HDC;
  hbmBitmap: HBITMAP;
  nHeight: Integer;
  Ch: DWORD;
  dwStrikeOut, dwUnderline,
  dwEqual, dwAlias, dwBold, dwItalic: DWORD;
  hFont_: HFONT;
  x, y: DWORD;
  str: array[0..1] of Char;
  size_: TSize;
  c: byte;
  d3dlr: TD3DLockedRect;
  pDstRow: PByte;
  pDst16: PWord;
  bAlpha: Byte;
begin
  if FOwnTexture then FTexture := nil;
  FFontName := font;
  FTextScale := 1.0;
  if ((Flags and D3DFONT_BOLD) = D3DFONT_BOLD)
   then dwBold := FW_BOLD else dwBold := FW_NORMAL;
  if ((Flags and D3DFONT_ITALIC) = D3DFONT_ITALIC)
   then dwItalic := 1 else dwItalic := 0;
  if ((Flags and D3DFONT_UNDERLINE) = D3DFONT_UNDERLINE)
   then dwUnderline := 1 else dwUnderline := 0;
  if ((Flags and D3DFONT_STRIKEOUT) = D3DFONT_STRIKEOUT)
   then dwStrikeOut := 1 else dwStrikeOut := 0;
  if ((Flags and D3DFONT_NOANTIALIAS) = D3DFONT_NOANTIALIAS)
   then dwAlias := 0 else dwAlias := ANTIALIASED_QUALITY;
  if ((Flags and D3DFONT_EQUALSPACED) = D3DFONT_EQUALSPACED)
   then dwEqual := 1 else dwEqual := 0;
  if (dwEqual = 1) then
  begin
    ch := 2 * length(characters) * (height + 1) * (height + 1);
    ch := round(sqrt(ch));
    x := 1; while (x < ch) do x := x shl 1;
    TexHeight := x;
  end else
  begin
   if (height > 40) then TexHeight := 1024
   else if (height > 20) then TexHeight := 512
   else TexHeight := 256;
  end;
  TexWidth := TexHeight;
  FDevice.GetDeviceCaps(d3dCaps);
  if (TexWidth > d3dCaps.MaxTextureWidth) then
  begin
    FTextScale := d3dCaps.MaxTextureWidth / TexWidth;
    TexHeight := d3dCaps.MaxTextureWidth;
    TexWidth := TexHeight;
  end;
  FDevice.CreateTexture(TexWidth, TexHeight, 1, 0, D3DFMT_A4R4G4B4, D3DPOOL_MANAGED, FTexture);
  ZeroMemory(@(bmi.bmiHeader), SizeOf(TBitmapInfoHeader));
  bmi.bmiHeader.biSize        := SizeOf(TBitmapInfoHeader);
  bmi.bmiHeader.biWidth       :=  TexWidth;
  bmi.bmiHeader.biHeight      := -TexHeight;
  bmi.bmiHeader.biPlanes      := 1;
  bmi.bmiHeader.biCompression := BI_RGB;
  bmi.bmiHeader.biBitCount    := 32;
  hDC_      := CreateCompatibleDC(0);
  hbmBitmap := CreateDIBSection(hDC_, bmi, DIB_RGB_COLORS, Pointer(pBitmapBits), 0, 0);
  SetMapMode(hDC_, MM_TEXT);
  nHeight := -MulDiv(Height, Trunc(GetDeviceCaps(hDC_, LOGPIXELSY) * FTextScale), 72);
  hFont_ := CreateFont(nHeight, 0, 0, 0, dwBold, dwItalic,
                       dwUnderline, dwStrikeOut, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS,
                       CLIP_DEFAULT_PRECIS, dwAlias,
                       VARIABLE_PITCH, pchar(font));
  if (0 = hFont_) then Exit;
  SelectObject(hDC_, hbmBitmap);
  SelectObject(hDC_, hFont_);
  SetTextColor(hDC_, RGB(255, 255, 255));
  SetBkColor  (hDC_, $00000000);
  SetTextAlign(hDC_, TA_TOP);
  x := 0;
  y := 0;
  if dwEqual = 1 then str := 'W'#0 else str := 'a'#0;
  GetTextExtentPoint32(hDC_, str, 1, size_);
  FCharWidth := size_.cx;
  FCharHeight := size_.cy;
  str := 'x'#0;
  for c:= 1 to length(characters) do
  begin
    str[0] := characters[c];
    if dwEqual = 0 then
    begin
     GetTextExtentPoint32(hDC_, str, 1, size_);
     if (dwItalic = 1) then
     begin
      size_.cx := size_.cx + 1;
      size_.cy := size_.cy + 2;
     end;
    end;
    if (x + DWord(size_.cx) + 2 > TexWidth) then
    begin
      x := 0;
      y := y + DWord(size_.cy) + 2;
    end;
    ExtTextOut(hDC_, x, y, ETO_OPAQUE, nil, str, 1, nil);
    ch := ord(characters[c]);
    Coords[ch].x := size_.cx;
    Coords[ch].y := size_.cy;
    TexCoords[ch][0] := (x)/TexWidth;
    TexCoords[ch][1] := (y)/TexHeight;
    TexCoords[ch][3] := ((y+1+DWord(size_.cy)))/TexHeight;
    if (dwItalic = 1) and (dwEqual = 0) then
     TexCoords[ch][2] := ((x-1+DWord(size_.cx)))/TexWidth
    else
     TexCoords[ch][2] := ((x+1+DWord(size_.cx)))/TexWidth;
    x := x + DWord(size_.cx) + 2;
  end;
  FTexture.LockRect(0, d3dlr, nil, 0);
  pDstRow:= d3dlr.pBits;
  for y:= 0 to (TexHeight - 1) do
  begin
    pDst16:= PWord(pDstRow);
    for x:= 0 to (TexWidth - 1) do
    begin
      bAlpha := (pBitmapBits^[TexWidth*y + x] and $ff) shr 4;
      if (bAlpha > 0) then
      begin
        pDst16^ := (bAlpha shl 12) or $0fff;
        Inc(pDst16);
      end
      else begin
        pDst16^ := $0000;
        Inc(pDst16);
      end;
    end;
    pDstRow:= PByte(Integer(pDstRow) + d3dlr.Pitch);
  end;
  FTexture.UnlockRect(0);
  DeleteObject(hbmBitmap);
  DeleteDC(hDC_);
  DeleteObject(hFont_);
  FOwnTexture := true;
  SetCharacters(characters);
  for x := 0 to 255 do FMapping[x] := x;
end;

procedure TOmegaBitmapFont.SetOmegaScreen(const Value: TOmegaScreen);
begin
 FOmegaScreen := Value;
end;

procedure TOmegaBitmapFont.Print(X, Y: integer; txt: String; blendmode:integer = bmNormal);
begin
 Print4Col(X, Y, FRotation, FScaleX, FScaleY, FCol1, FCol2, FCol3, FCol4, txt, blendmode);
end;

procedure TOmegaBitmapFont.Print(X, Y: integer;
 Rotation, ScaleX, ScaleY: single; col: TOmegaColor; txt: String; blendmode: integer = bmNormal);
begin
 Print4Col(X, Y, Rotation, ScaleX, ScaleY, col, col, col, col, txt, blendmode);
end;

procedure TOmegaBitmapFont.PrintFast(X, Y: integer; txt: String; blendmode:integer = bmNormal);
begin
 Print4ColFast(X, Y, FCol1, FCol2, FCol3, FCol4, txt, blendmode);
end;

procedure TOmegaBitmapFont.PrintFast(X, Y: integer; col: TOmegaColor; txt: String; blendmode: integer = bmNormal);
begin
 Print4ColFast(X, Y, col, col, col, col, txt, blendmode);
end;

procedure TOmegaBitmapFont.Print4Col(X, Y: integer; Rotation,
  ScaleX, ScaleY: single; col1, col2, col3, col4: TOmegaColor; txt: String; blendmode: integer = bmNormal);
var i, j: Integer;
    sx, sy: Integer;
begin
 if FOwnTexture then FRenderer.Texture := FTexture else
  FRenderer.Texture := FOmegaImageList.ImageList.Items[FOmegaImageIndex].image;
 if FPositionMode = pmTopLeft then
 begin
  sx := X;
  sy := Y;
 end else
 begin
  sx := X - Round(Length(txt) * 0.5 * FCharWidth);
  sy := Y - Round(FCharHeight * 0.25);
 end;
 if blendmode <> bmIgnore then FRenderer.OmegaScreen.SetBlendMode(blendmode);
 for i := 1 to length(txt) do
 begin
  j := FMapping[ord(txt[i])];
  if txt[i] = #13 then
  begin
   sy := sy + CharHeight + FLineSpace ;
   sx := x;
  end;
  if j > -1 then
  begin
   if txt[i] <> ' ' then
   begin
   FRenderer.SetPositions(sx, sy, sx + Coords[j].x, sy,
    sx + Coords[j].x, sy + Coords[j].y, sx, sy + Coords[j].y);
   FRenderer.SetColors(col1, col2, col3, col4);
   FRenderer.SetTexturePositions(
    TexCoords[j][0], TexCoords[j][1],
    TexCoords[j][2], TexCoords[j][1],
    TexCoords[j][2], TexCoords[j][3],
    TexCoords[j][0], TexCoords[j][3]);
   FRenderer.Rotate(sx + Coords[j].x*0.5, sy + Coords[j].y*0.5, FCharRotation);
   FRenderer.RotateAndScale(x + FCenterX, y + FCenterY,
    Rotation,
    ScaleX * FTextScale,
    ScaleY * FTextScale);
   FRenderer.Draw;
   end;
   sx := sx + Coords[j].x + FSpacing;
  end;
 end;
end;

// Optimization of Print4Col, by Athena and Bazi.
// The aim is to render all the letters at once, which translate to a 4x speed compared to
// the previous method. A needed update if you display more than a few words on your game :o)

procedure TOmegaBitmapFont.Print4ColFast(X, Y: integer; col1, col2, col3, col4: TOmegaColor; txt: String; blendmode: integer = bmNormal);
var i, j, sx, sy, NbChars: Integer;
    a : single;
    FontVertices : array[0..6 * 1024 -1] of TOmegaVertex;
begin
  if FOwnTexture then FRenderer.Texture := FTexture
                 else FRenderer.Texture := FOmegaImageList.ImageList.Items[FOmegaImageIndex].image;

  // Position test
  if FPositionMode = pmTopLeft then
  begin
    sx := X;
    sy := Y;
  end
      else
  begin
    sx := X - Round(Length(txt) * 0.5 * FCharWidth);
    sy := Y - Round(FCharHeight * 0.25);
  end;

  if blendmode <> bmIgnore then FRenderer.OmegaScreen.SetBlendMode(blendmode);

  NbChars := Length(txt);
  // I could have used Min, but it would have needed Math unit
  if NbChars > (High(FontVertices)+1) div 6 -1 then NbChars := (High(FontVertices)+1) div 6 -1;

  a:=0.5; // See FRenderer.SetPositions for this +a thing

  // Main loop, we build 2 triangles for each letter, BUT the rendering is done at the end, all at once
  for i := 1 to NbChars do
  begin
    j := FMapping[ord(txt[i])];
    // New row test
    if txt[i] = #13 then
    begin
      sy := sy + CharHeight + FLineSpace ;
      sx := x;
    end;
    if j > -1 then
    begin
      // Filling vertex array
      FontVertices[0 + (i-1)*6] := MakeOmegaVertex(sx, sy, Col1, TexCoords[j][0], TexCoords[j][1]);
      FontVertices[1 + (i-1)*6] := MakeOmegaVertex(sx + Coords[j].x +a, sy, Col2, TexCoords[j][2], TexCoords[j][1]);
      FontVertices[2 + (i-1)*6] := MakeOmegaVertex(sx + Coords[j].x +a, sy + Coords[j].y +a, Col3, TexCoords[j][2], TexCoords[j][3]);
      FontVertices[3 + (i-1)*6] := FontVertices[2 + (i-1)*6];
      FontVertices[4 + (i-1)*6] := MakeOmegaVertex(sx, sy + Coords[j].y +a, Col4, TexCoords[j][0], TexCoords[j][3]);
      FontVertices[5 + (i-1)*6] := FontVertices[0 + (i-1)*6];
      // Next letter
      sx := sx + Coords[j].x + FSpacing;
    end;// j > -1
  end;// for i := 1 to length(txt)

  // Rendering all the vertices as once as a list of triangles
  FOmegaScreen.CopyVertices(@FontVertices, NbChars*6);
  FOmegaScreen.FDevice.DrawPrimitive(D3DPT_TRIANGLELIST, 0, NbChars*2);
end;

procedure TOmegaBitmapFont.SetAlpha(const Value: Integer);
begin
  FAlpha := Value;
  FCol1 := OmegaColor(FRed, FGreen, FBlue, FAlpha);
  FCol2 := FCol1; FCol3 := FCol1; FCol4 := FCol1;
end;

procedure TOmegaBitmapFont.SetBlue(const Value: Integer);
begin
  FBlue := Value;
  FCol1 := OmegaColor(FRed, FGreen, FBlue, FAlpha);
  FCol2 := FCol1; FCol3 := FCol1; FCol4 := FCol1;
end;

procedure TOmegaBitmapFont.SetGreen(const Value: Integer);
begin
  FGreen := Value;
  FCol1 := OmegaColor(FRed, FGreen, FBlue, FAlpha);
  FCol2 := FCol1; FCol3 := FCol1; FCol4 := FCol1;
end;

procedure TOmegaBitmapFont.SetRed(const Value: Integer);
begin
  FRed := Value;
  FCol1 := OmegaColor(FRed, FGreen, FBlue, FAlpha);
  FCol2 := FCol1; FCol3 := FCol1; FCol4 := FCol1;
end;

procedure TOmegaBitmapFont.LoadBMP(filename: string);
begin
 if FTexture <> nil then FTexture := nil;
 FOwnTexture := true;
 D3DXCreateTextureFromFileEx(
  FDevice, pchar(filename), 0, 0, 0, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,
  D3DX_FILTER_NONE, D3DX_FILTER_NONE, $FF000000,
  nil, nil, FTexture);
end;

procedure TOmegaBitmapFont.LoadFNT(filename: string);
begin
 LoadBMP(filename);
 LoadWID(filename);
end;

procedure TOmegaBitmapFont.SaveBMP(filename: string);
var i: IDirect3DSurface8;
begin
 FTexture.GetSurfaceLevel(0, i);
 D3DXSaveSurfaceToFile(pchar(filename), D3DXIFF_BMP, i, nil, nil);
end;

procedure TOmegaBitmapFont.SaveFNT(filename: string);
begin
 SaveBMP(filename);
 SaveWID(filename);
end;

procedure TOmegaBitmapFont.LoadWID(filename: string);
var f: file of DWORD;
    x, a: integer;
    y: DWORD;
begin
 assignfile(f, filename);
 reset(f);
 a := filesize(f) - 7 * 256 - 1;
 seek(f, a);
 for x := 0 to 255 do
 begin
  read(f, y); FMapping[x] := y;
  read(f, y); TexCoords[x][0] := y / 100000;
  read(f, y); TexCoords[x][1] := y / 100000;
  read(f, y); TexCoords[x][2] := y / 100000;
  read(f, y); TexCoords[x][3] := y / 100000;
  read(f, y); Coords[x].x := y;
  read(f, y); Coords[x].y := y;
 end;
 read(f, y); FTextScale := y / 100000;
 closefile(f);
end;

procedure TOmegaBitmapFont.SaveWID(filename: string);
var f: file of DWORD;
    x: integer;
    a: DWORD;
begin
 assignfile(f, filename);
 reset(f);
 x := filesize(f);
 seek(f, x);
 for x := 0 to 255 do
 begin
  a := FMapping[x]; write(f, a);
  a := round(TexCoords[x][0] * 100000); write(f, a);
  a := round(TexCoords[x][1] * 100000); write(f, a);
  a := round(TexCoords[x][2] * 100000); write(f, a);
  a := round(TexCoords[x][3] * 100000); write(f, a);
  a := Coords[x].x; write(f, a);
  a := Coords[x].y; write(f, a);
 end;
 a := round(FTextScale * 100000); write(f, a);
 closefile(f);
end;

procedure TOmegaBitmapFont.Init;
begin
 if FOmegaScreen = nil then exit;
 if FRenderer = nil then exit;
 FRenderer.OmegaScreen := FOmegaScreen;
 FRenderer.Init;
 if FOmegaScreen.Device = nil then exit;
 FDevice := FOmegaScreen.Device;
 FOwnTexture := false;
 SetOmegaImageIndex(FOmegaImageIndex);
 SetCharacters(FCharacters);
 if FOmegaImageList = nil then MakeFont('Arial', DEFCHARACTERS, 12, D3DFONT_BOLD);
end;

procedure TOmegaBitmapFont.UnInit;
begin
 if FRenderer <> nil then FRenderer.UnInit;
end;

procedure TOmegaBitmapFont.Notification(AComponent: TComponent;
 Operation: TOperation);
begin
 inherited;
 case Operation of
  opInsert: if (AComponent is TOmegaScreen) and (not Assigned(OmegaScreen)) then
   SetOmegaScreen(TOmegaScreen(AComponent));
  opRemove: if (AComponent = OmegaScreen) then OmegaScreen:= nil;
 end;
end;

function TOmegaBitmapfont.TextWidth(s: string): integer;
var i,w:integer;
begin
 w := 0;
 for i := 1 to length(s) do
  w:=w+
   round(Coords[FMapping[Ord(s[i])]].x * ScaleX);
 Result := w;
//  Result:=Round((FCharWidth * Length(s)) * ScaleX);
 if Result = 0 then Result := 1;
end;

function TOmegaBitmapFont.TextHeight(s: string): integer;
begin
  Result := Round(FcharHeight * ScaleY);
  if Result = 0 then Result := 1;
end;

end.
