{
//****************************************************************
// OmegaFader unit
// author : Tobias Fleischer
// contact: tobybear@web.de
// comments are welcome!
//****************************************************************

version history:
01/27/04 added blendmode property
09/19/03 added MultiFade, LoopFade and Stop
09/02/03 now using new renderer in OmegaScreen
08/31/03 added notification
08/19/03 renamed component to OmegaFader
         added PrepareFadeOut and PrepareFadeIn functions
         fixed 'move' in second draw function
         took out some stuff that is obsolete because of OmegaPrimitives
08/13/03 first release

"todo":
- add proper documentation
}
unit OmegaFader;
interface
uses Windows, OmegaScreen, Classes, OmegaRenderer, {$I UseD3D8.inc};

type
  TOmegaFader = class(TComponent)
  private
   FMirrorCnt, FCurColCnt, FNumCols: integer;
   FCurCol, FMultiCols: ^TOmegaColor;
   FMulti: boolean;
   vertices: array[0..3] of TOmegaVertex;
   MoveCnt: single;
   FSpeed: single;
   FLooped, FMultiFade, FMirrorFade, FFadeEnded: boolean;
   FSrcR, FSrcG, FSrcB, FSrcA,
   FDestR, FDestG, FDestB, FDestA,
   FCurR, FCurG, FCurB, FCurA: byte;
   FBlendMode: integer;
   procedure SetOmegaScreen(const Value: TOmegaScreen);
   procedure SetFadeSpeed(sp: single);
  protected
    FOmegaScreen: TOmegaScreen;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure MultiFade(cols: array of TOmegaColor);
    procedure SetSourceColor(Red, Green, Blue, Alpha: byte); overload;
    procedure SetSourceColor(col: TOmegaColor); overload;
    procedure SetDestinationColor(Red, Green, Blue, Alpha: byte); overload;
    procedure SetDestinationColor(col: TOmegaColor); overload;
    procedure PrepareFadeIn(r, g, b: byte; speed: single);
    procedure PrepareFadeOut(r, g, b: byte; speed: single);
    procedure SwapColors;
    procedure Move;
    procedure Reset;
    procedure Stop;
    procedure Draw; overload;
    procedure Draw(rec: TRect); overload;
    property FadeEnded: boolean read FFadeEnded;
    property BlendMode: integer read FBlendMode write FBlendMode;
  published
    property OmegaScreen: TOmegaScreen read FOmegaScreen write SetOmegaScreen;
    property FadeSpeed: single read FSpeed write SetFadeSpeed;
    property MirrorFade: boolean read FMirrorFade write FMirrorFade;
    property LoopFade: boolean read FLooped write FLooped;
  end;

implementation

{ TOmegaFader }

procedure TOmegaFader.Draw;
begin
 vertices[0].color := OmegaColor(FCurR, FCurG, FCurB, FCurA);
 vertices[1].color := vertices[0].color;
 vertices[2].color := vertices[0].color;
 vertices[3].color := vertices[0].color;
 vertices[0].x := 0;
 vertices[0].y := 0;
 vertices[1].x := FOmegaScreen.Width;
 vertices[1].y := 0;
 vertices[2].x := FOmegaScreen.Width;
 vertices[2].y := FOmegaScreen.Height;
 vertices[3].x := 0;
 vertices[3].y := FOmegaScreen.Height;
 FOmegaScreen.Device.SetTexture(0, nil);
 FOmegaScreen.CopyVertices(@vertices, 4);
 FOmegaScreen.SetBlendMode(FBlendMode);
 FOmegaScreen.Device.DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);
 if FSpeed > 0 then Move;
end;

procedure TOmegaFader.Draw(rec: TRect);
begin
 vertices[0].color := OmegaColor(FCurR, FCurG, FCurB, FCurA);
 vertices[1].color := vertices[0].color;
 vertices[2].color := vertices[0].color;
 vertices[3].color := vertices[0].color;
 vertices[0].x := rec.left;
 vertices[0].y := rec.top;
 vertices[1].x := rec.right;
 vertices[1].y := rec.top;
 vertices[2].x := rec.right;
 vertices[2].y := rec.bottom;
 vertices[3].x := rec.left;
 vertices[3].y := rec.bottom;
 FOmegaScreen.Device.SetTexture(0, nil);
 FOmegaScreen.CopyVertices(@vertices, 4);
 FOmegaScreen.SetBlendMode(FBlendMode);
 FOmegaScreen.Device.DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);
 if FSpeed > 0 then Move;
end;

procedure TOmegaFader.SetOmegaScreen(const Value: TOmegaScreen);
begin
 FOmegaScreen := Value;
end;

procedure TOmegaFader.Move;
var a, b: single;
begin
 FFadeEnded := false;
 a := MoveCnt * 0.01; b := 1 - a;
 FCurR := round(FSrcR * b + a * FDestR);
 FCurG := round(FSrcG * b + a * FDestG);
 FCurB := round(FSrcB * b + a * FDestB);
 FCurA := round(FSrcA * b + a * FDestA);
 MoveCnt := MoveCnt + FSpeed;
 if MoveCnt >= 100 then
 begin
  if FMultiFade then
  begin
   inc(FCurColCnt);
   if FCurColCnt > FNumCols then
   begin
    if FLooped then
    begin
     MoveCnt := 0;
     FCurColCnt := 0;
     FCurCol := FMultiCols;
     SetSourceColor(FCurR, FCurG, FCurB, FCurA);
     SetDestinationColor(FCurCol^);
     exit;
    end else
    begin
     MoveCnt := 100;
     FFadeEnded := true;
     FMultiFade := false;
     FMulti := false;
     Freemem(FMultiCols);
    end;
    exit;
   end;
   inc(FCurCol);
   MoveCnt := 0;
   SetSourceColor(FCurR, FCurG, FCurB, FCurA);
   SetDestinationColor(FCurCol^);
  end else
  if FMirrorFade then
  begin
   inc(FMirrorCnt);
   if (FMirrorCnt > 1) and (FLooped = false) then
   begin
    MoveCnt := 100;
    FFadeEnded := true;
   end else
   begin
    MoveCnt := 0;
    SetDestinationColor(FSrcR, FSrcG, FSrcB, FSrcA);
    SetSourceColor(FCurR, FCurG, FCurB, FCurA);
   end;
  end else
  begin
   if (Flooped) then MoveCnt := 0 else
   begin
    MoveCnt := 100;
    FFadeEnded := true;
   end; 
  end;
 end;
end;

procedure TOmegaFader.Reset;
begin
 MoveCnt := 0;
 FMirrorCnt := 0;
 FFadeEnded := false;
end;

procedure TOmegaFader.SetSourceColor(Red, Green, Blue, Alpha: byte);
begin
 FSrcR := Red;
 FSrcG := Green;
 FSrcB := Blue;
 FSrcA := Alpha;
 FCurR := Red;
 FCurG := Green;
 FCurB := Blue;
 FCurA := Alpha;
end;

procedure TOmegaFader.SetDestinationColor(Red, Green, Blue, Alpha: byte);
begin
 FDestR := Red;
 FDestG := Green;
 FDestB := Blue;
 FDestA := Alpha;
end;

procedure TOmegaFader.SetDestinationColor(col: TOmegaColor);
begin
 SetDestinationColor(col.red, col.green, col.blue, col.alpha);
// (col shr 16) and $FF, (col shr 8) and $FF, col and $FF, (col shr 24) and $FF);
end;

procedure TOmegaFader.SetSourceColor(col: TOmegaColor);
begin
 SetSourceColor(col.red, col.green, col.blue, col.alpha);
// (col shr 16) and $FF, (col shr 8) and $FF, col and $FF, (col shr 24) and $FF);
end;

procedure TOmegaFader.SetFadeSpeed(sp: single);
begin
 if sp > 100 then sp := 100;
 if sp < 0 then sp := 0;
 FSpeed := sp;
end;

constructor TOmegaFader.Create(AOwner: TComponent);
begin
 inherited;
 FMultiFade := false;
 FLooped := false;
 FMulti := false;
 FBlendMode := bmNormal;
 vertices[0].x := 0;
 vertices[0].y := 0;
 vertices[0].rhw := 1;
 vertices[0].tu := 0;
 vertices[0].tv := 0;
 vertices[0].color := OmegaColor($FFFFFFFF);
 vertices[1].x := 50;
 vertices[1].y := 0;
 vertices[1].rhw := 1;
 vertices[1].tu := 1;
 vertices[1].tv := 0;
 vertices[1].color := OmegaColor($FFFFFFFF);
 vertices[2].x := 50;
 vertices[2].y := 50;
 vertices[2].rhw := 1;
 vertices[2].tu := 1;
 vertices[2].tv := 1;
 vertices[2].color := OmegaColor($FFFFFFFF);
 vertices[3].x := 0;
 vertices[3].y := 50;
 vertices[3].rhw := 1;
 vertices[3].tu := 0;
 vertices[3].tv := 1;
 vertices[3].color := OmegaColor($FFFFFFFF);

 SetFadeSpeed(1);
 SetSourceColor(0, 0, 0, 0);
 SetDestinationColor(0, 0, 0, 255);
 FMirrorFade := false;
 FMirrorCnt := 0;
 Reset;
end;

destructor TOmegaFader.Destroy;
begin
 if FMulti {FMultiCols<>nil{assigned(FMultiCols)} then Freemem(FMultiCols);
 inherited;
end;

procedure TOmegaFader.SwapColors;
begin
 FCurR := FDestR;
 FCurG := FDestG;
 FCurB := FDestB;
 FCurA := FDestA;
 FDestR := FSrcR;
 FDestG := FSrcG;
 FDestB := FSrcB;
 FDestA := FSrcA;
 FSrcR := FCurR;
 FSrcG := FCurG;
 FSrcB := FCurB;
 FSrcA := FCurA;
end;

procedure TOmegaFader.PrepareFadeIn(r, g, b: byte; speed: single);
begin
 SetSourceColor(r, g, b, 255);
 SetDestinationColor(r, g, b, 0);
 SetFadeSpeed(speed);
 Reset;
end;

procedure TOmegaFader.PrepareFadeOut(r, g, b: byte; speed: single);
begin
 SetSourceColor(r, g, b, 0);
 SetDestinationColor(r, g, b, 255);
 SetFadeSpeed(speed);
 Reset;
end;

procedure TOmegaFader.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
 inherited;
 case Operation of
  opInsert: if (AComponent is TOmegaScreen)and(not Assigned(OmegaScreen)) then
   SetOmegaScreen(TOmegaScreen(AComponent));
  opRemove: if (AComponent = OmegaScreen) then OmegaScreen:= nil;
 end;
end;

procedure TOmegaFader.MultiFade(cols: array of TOmegaColor);
begin
 GetMem(FMultiCols, sizeof(cols));
 FMulti := true;
 system.Move(cols, FMultiCols^, sizeof(cols));
 FNumCols := high(cols);
 if FNumCols < 0 then exit;
 SetSourceColor(cols[0]);
 if FNumCols > 0 then SetDestinationColor(cols[1]);
 FCurColCnt := 0;
 FCurCol := FMultiCols;
 inc(FCurCol);
 FMultiFade := true;
 Reset;
end;

procedure TOmegaFader.Stop;
begin
 FFadeEnded := true;
end;

end.


