unit OmegaGUI;
{
//****************************************************************
// OmegaGUI component 0.96c
// Author : David Senabre (Patroclus), Tobias Fleischer (Tobybear)
// Omega Components
//****************************************************************

standar component draw uses DrawInRect instead of Draw method

version history:
18/02/05 : Components animation improved
02/10/04 : OmegaBars can be clicked to set position ;)
08/09/04 : Improvements, fixes, bugs,...
           Many of ZTar improvements
22/06/04 : Lots of fixes ;) (Patroclus)
06/06/04: added support for new DrawRect/DrawInRect functions
04/21/04: fixed cursor display, added umlaut support (Tobybear)
08/03/04: XYFactor added, Bitmapfont.
older changes:
 OmegaPanels support Drag and Drop
 OmegaEdit now supports Blending font, and enhaced inputchar property
 OmegaEdit now only accepts valid characters in OmegaBitmapFont
 Added AutoScroll for Omegatext
 Fixed Edit rotation text (only supports rotation if scale=1)
 Mirror and BlendMode support for components (except bars)
 Fixed OmegaEdit cursor position when scaling
 Added Component mirroring
 OmegaEdits now work with either FontRotate or Rotate effect (not both!)
 Added OmegaEdits CursorShow for blinking control
}

interface
{$IFDEF VER130} {$DEFINE DXG_COMPAT} {$ENDIF}

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms,
  OmegaImageList, OmegaScreen, Mmsystem, OmegaBitmapFont, dialogs;

type
  TGUIStatus = record
    Mx, My : Integer;
    SelID : Integer;
    PressID: Integer;
    FocusID: Integer;
    B1,B2,B3 : Boolean;
  end;

  TOmegaGUIComponent = class;

  TOmegaGUI = class(TComponent)
  private
    FGUIList: TList;
    FIDList: TList;
//    FGUIStatus : TGUIStatus;
    FGUIStatusOld : TGUIStatus;
    FInitializated: Boolean;
    FIDCounter: Cardinal;
    FScreenHandle: Cardinal;
    FOmegaScreen: TOmegaScreen;
    FOmegaBitmapFont: TOmegaBitmapFont;
    FBackGround: TImageListItem;
    procedure ProcessPos;
    procedure ProcessBtn(Btn: TMouseButton; Click, Dbl: Boolean);
    procedure ProcessKey(CharKey: Boolean; WParam, LParam: Word);
    procedure WindowsMessages(var Msg: TMsg; var Handled: Boolean);
    procedure WM(var Message:TMessage);
    function GetSurface(h: HWND): TWinControl;
    function CheckClickPanel: Integer;
  public
    FGUIStatus : TGUIStatus;
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Init;
    procedure Stop;
    procedure Draw;
    procedure CursorPos(x,y: Integer);
    function FindID(i: Integer): TOmegaGUIComponent;
    property SelID: Integer read FGUIStatus.SelID write FGUIStatus.SelID;
    property FocusID: Integer read FGUIStatus.FocusID write FGUIStatus.FocusID;
    property Initializated: Boolean read FInitializated;
    property IDCounter: Cardinal read FIDCounter;
    property BackGround: TImageListItem read FBackGround write FBackGround;
  published
    property OmegaScreen: TOmegaScreen read FOmegaScreen write FOmegaScreen;
    property OmegaBitmapFont: TOmegaBitmapFont read FOmegaBitmapFont write FOmegaBitmapFont;
  end;

  TOmegaGUIImageRecord = record
      Image: TImageListItem;
      ImageIndex: Integer;
      DoAnimate: Boolean;
      OrigIndex: Integer;       //first animation frame
      AnimCount: integer;       //number of consecutive frames
      AnimSpeed: Integer;       //higher number, slower
      Animloop: Boolean;
      AnimBack: Boolean;    //play animation backwards?
      AnimOver: Boolean;        //is non-loop animation finished?
      Color: TOmegaColor;
      AlphaBlend: Boolean;
      BlendMode: Integer;
  end;
  TOmegaGUIImage = array [0..3] of TOmegaGUIImageRecord;

  TOmegaGUIList = class(TObject)
  private
    FList: TList;
    FName: String;
    FParent: TOmegaGUI;
    FEnabled: Boolean;
    FVisible: Boolean;
    procedure SetID(Value: Integer);
    function GetID: Integer;
    function GetOldID: Integer;
    procedure SetFocus(Value: Integer);
    function AssignID: Cardinal;
  public
    constructor Create(AOwner : TOmegaGUI); virtual;
    destructor Destroy; override;
    procedure Clear;
    procedure ProcessPos; virtual;
    procedure ProcessBtn;
    procedure Add(Sender: TOmegaGUIComponent); virtual;
    procedure Delete(Value: Integer);
    procedure Draw; dynamic;
    procedure ZTop;
    procedure ZBottom;
    property Name: String read FName write FName;
    property Enabled: Boolean read FEnabled write FEnabled;
    property Visible: Boolean read FVisible write FVisible;
    property List:TList read FList write FList;
  end;

  TOmegaGUIPanel = class(TOmegaGUIList)
  private
    FImage: TOmegaGUIImageRecord;
    FCaption: String;
    FX, FY : Integer;
    FWidth, FHeight: Integer;
    FMirrorH: Boolean;
    FMirrorV: Boolean;
    FBlendMode: Integer;
    FCanBedragged: Boolean;
    FdraggedX: Integer;                  //offset from which panel has been dragged
    FdraggedY: Integer;
    FOldX:Integer;                      //used to calculate panel drag/drop
    FOldY:Integer;
    FDragHeight: Integer;
    FDragWidth: Integer;
    procedure SetX(Value: Integer);
    procedure SetY(Value: Integer);
    procedure SetWidth(Value: Integer);
    procedure SetHeight(Value: Integer);
    procedure Drag;
    procedure UnDrag;
  public
    procedure ChildRelative;
    procedure CheckPos;
    procedure UpdatePos(DeltaX,DeltaY:Integer);
    constructor Create(AOwner : TOmegaGUI); override;
    procedure Add(Sender: TOmegaGUIComponent); override;
    procedure Draw; override;
    property Image: TImageListItem read FImage.Image write FImage.Image;
    property Color: TOmegaColor read FImage.Color write FImage.Color;
    property ImageIndex: Integer read FImage.ImageIndex write FImage.ImageIndex;
    property Anim: Boolean read FImage.DoAnimate write FImage.DoAnimate;
    property AnimCount: Integer read FImage.AnimCount write FImage.AnimCount;
    property AnimSpeed: Integer read FImage.AnimSpeed write FImage.AnimSpeed;
    property X: Integer         read FX         write SetX;
    property Y: Integer         read FY         write SetY;
    property Width: Integer     read FWidth     write SetWidth;
    property Height: Integer    read FHeight    write SetHeight;
    property Caption: String    read FCaption   write FCaption;
    property MirrorH:Boolean    read FMirrorH   write FMirrorH;
    property MirrorV:Boolean    read FMirrorV   write FMirrorV;
    property BlendMode: Integer   read FBlendMode    write FBlendMode;
    property DragHeight: Integer  read FDragHeight   write FDragHeight;
    property DragWidth: Integer   read FDragWidth    write FDragWidth;
    property CanBedragged:Boolean read FCanBedragged write FCanBedragged;
    procedure ProcessPos; override;
  end;

  TOmegaGUIComponent = class(TObject)
  private
    FID: Integer;
    FParent: TOmegaGUIList;
    FName: String;
    FVisible: Boolean;
    FEnabled: Boolean;
    FX, FY: Integer;
    FWidth, FHeight : Integer;
    FImage: TOmegaGUIImage;
    FMirrorH: Boolean;
    FMirrorV: Boolean;
    FOmegaBitmapFont: TOmegaBitmapFont;
    FFontColor: TOmegaColor;
    FFontRotate: Single;
    FFontScaleX: Single;
    FFontScaleY: Single;
    FRotate: Single;
    Factive: Boolean;
    FAnimCounter: Integer;
    FOnClick: TNotifyEvent;
    FOnDblClick: TNotifyEvent;
    FOnEnter: TNotifyEvent;
    FOnExit: TNotifyEvent;
    FOnMouseDown: TMouseEvent;
    FOnMouseUp: TMouseEvent;
    FOnKeyPress: TKeyEvent;
    procedure Animate(var Pattern:TOmegaGUIImageRecord);
    procedure ResetAnimation(var Pattern:TOmegaGUIImageRecord);
    procedure DoClick; dynamic;
    procedure DoDblClick; virtual;
    procedure DoKeyDown(CharKey: Boolean; var Key: Word; Shift: TShiftState); dynamic;
    procedure DoMouseOver; dynamic;
    procedure DoEnter; virtual;
    procedure DoExit; dynamic;
    procedure SetX(Value: Integer);
    procedure SetY(Value: Integer);
    procedure SetWidth(Value: Integer);
    procedure SetHeight(Value: Integer);
    procedure SetImage(index: integer; Value: TImageListItem);
    function GetImage(index: integer): TImageListItem;
    procedure SetImageIndex(index: integer; Value: Integer);
    function GetImageIndex(index: integer): Integer;
    procedure SetAnim(index: integer; Value: Boolean);
    function GetAnim(index: integer): Boolean;
    procedure SetAnimC(index: integer; Value: Integer);
    function GetAnimC(index: integer): Integer;
    procedure SetAnimS(index: integer; Value: Integer);
    function GetAnimS(index: integer): Integer;
    procedure SetAnimLoop(index: integer; Value: boolean);
    function GetAnimLoop(index: integer): boolean;
    procedure SetAnimBack(index: integer; Value: boolean);
    function GetAnimBack(index: integer): boolean;
    procedure SetOrig(index: integer; Value: Integer);
    function GetOrig(index: integer): Integer;
    procedure SetBlend(index: integer; Value: Boolean);
    function GetBlend(index: integer): Boolean;
    procedure SetBlendMode(index: integer; Value: Integer);
    function GetBlendMode(index: integer): Integer;
    procedure SetColor(index: integer; Value: TOmegaColor);
    function GetColor(index: integer): TOmegaColor;
    procedure SetAlpha(index: integer; Value: Byte);
    function GetAlpha(index: integer): Byte;
  public
    constructor Create(AOwner : TOmegaGUIList); virtual;
    destructor Destroy; override;
    procedure ChangeList(Dest: TOmegaGUIList);
    procedure Draw; dynamic;
    procedure ZTop;
    procedure ZBottom;
    property Name: String read FName write FName;
    property Image[Index:Integer]: TImageListItem read GetImage write SetImage;
    property ImageIndex[Index:Integer]: Integer read GetImageIndex write SetImageIndex;
    property Anim[Index:Integer]: Boolean read GetAnim write SetAnim;
    property AnimCount[Index:Integer]: Integer read GetAnimC write SetAnimC;
    property AnimSpeed[Index:Integer]: Integer read GetAnimS write SetAnimS;
    property AnimLoop[Index:Integer]: boolean read GetAnimLoop write SetAnimLoop;
    property AnimBack[Index:Integer]: boolean read GetAnimBack write SetAnimBack;
    property OrigIndex[Index:Integer]: Integer read GetOrig  write SetOrig;
    property BlendMode[Index:Integer]: Integer read GetBlendMode write SetBlendMode;
    property AlphaBlend[Index:Integer]: Boolean read GetBlend write SetBlend;
    property Color[Index:Integer]: TOmegaColor read GetColor write SetColor;
    property Alpha[Index:Integer]: Byte         read GetAlpha   write SetAlpha;
    property MirrorH: Boolean read FMirrorH write FMirrorH;
    property MirrorV: Boolean read FMirrorV write FMirrorV;
    property FontColor: TOmegaColor read FFontColor write FFontColor;
    property FontAlpha: Byte read FFontColor.Alpha write FFontColor.Alpha;
    property Rotation: Single read FRotate write FRotate;
    property OmegaBitmapFont: TOmegaBitmapFont read FOmegaBitmapFont write FOmegaBitmapFont;
    property FontRotation: Single read FFontRotate write FFontRotate;
    property FontScaleX: Single read FFontScaleX write FFontScaleX;
    property FontScaleY: Single read FFontScaleY write FFontScaleY;
    property ID: Integer        read FID;
    property X: Integer         read FX         write SetX;
    property Y: Integer         read FY         write SetY;
    property Width: Integer     read FWidth     write SetWidth;
    property Height: Integer    read FHeight    write SetHeight;
    property Enabled: Boolean   read FEnabled   write FEnabled;
    property Visible: Boolean   read FVisible   write FVisible;
    property Active: Boolean    read FActive    write FActive default False;
    property OnClick: TNotifyEvent     read FOnClick     write FOnClick;
    property OnDblClick: TNotifyEvent  read FOnDblClick  write FOnDblClick;
    property OnEnter: TNotifyEvent     read FOnEnter     write FOnEnter;
    property OnExit: TNotifyEvent      read FOnExit      write FOnExit;
    property OnMouseDown: TMouseEvent  read FOnMouseDown write FOnMouseDown;
    property OnMouseUp: TMouseEvent    read FOnMouseUp   write FOnMouseUp;
    property OnKeyPress: TKeyEvent     read FOnKeyPress  write FOnKeyPress;
  end;

  TOmegaButton = class(TOmegaGUIComponent)
  private
    FCaption: String;
    FCentered: Boolean;
    FRepeating: Boolean;
    FCheckeable: Boolean;
    FTexttop,FTextleft: Integer;
    FGroup: Integer;
    procedure DoClick; override;
    procedure DoKeyDown(CharKey: Boolean; var Key: Word; Shift: TShiftState);override;
  public
    constructor Create(AOwner : TOmegaGUIList); override;
    procedure Draw; override;
    property Caption: String read FCaption write FCaption;
    property Centered: Boolean read FCentered write FCentered;
    property Repeating: Boolean read FRepeating write FRepeating;
    property Checkeable: Boolean read FCheckeable write FCheckeable;
    property textleft: integer read Ftextleft write Ftextleft;
    property texttop: integer read Ftexttop write Ftexttop;
    property Group: Integer read FGroup write FGroup;
  end;

  TEditInputChar = (eicNormal,eicUpperCase,eicLowerCase,
   eicNumeric,eicForceUpperCase, eicForceLowerCase);

  TOmegaEdit = class(TOmegaGUIComponent)
  private
    FMaxLength: Integer;
    FReadOnly: Boolean;
    FInputChar: TEditInputChar;
    FText: String;
    FTextPoint: Integer;    //possible property
    FTextBlend: Integer;
    FLinePattern: String;
    FBorder: Integer;
    FCursorChar: String;
    FCursorShow: Boolean;
    procedure DoClick; override;
    procedure DoKeyDown(CharKey: Boolean; var Key: Word; Shift: TShiftState);override;
    procedure SetLinePattern(Value: String);
    function GetTextLen: Integer;
    function CheckCharValid(c: Char):Boolean;
  public
    constructor Create(AOwner: TOmegaGUIList); override;
    procedure Draw; override;
    procedure Clear;
    property Border: Integer            read FBorder     write FBorder;
    property ReadOnly: Boolean          read FReadOnly   write FReadOnly;
    property LinePattern: String        read FLinePattern write SetLinePattern;
    property Text: String               read FText       write FText;
    property TextLen: Integer           read GetTextLen;
    property TextBlend: Integer         read FTextBlend  write FTextBlend;
    property InputChar: TEditInputChar  read FInputChar  write FInputChar;
    property MaxLength:Integer          read FMaxLength  write FMaxLength;
    property CursorChar: String         read FCursorChar write FCursorChar;
    property CursorShow: Boolean        read FCursorShow write FCursorShow;
    property CursorPos: Integer         read FTextPoint  write FTextPoint;
  end;

  TOmegaText = class(TOmegaGUIComponent)
  private
    FLines:TStringList;
//    FLinePattern: String;
//    FLineNumber: Integer;
    FLineIndex: Integer;
    FBorder: Integer;
    FBorderY: Integer;
    FNoCarry: Boolean;
    FAutoScroll: Boolean;
    function MaxLineIndex: Integer;
    procedure SetLinePattern(Value: String);
    procedure SetLineNumber(Value: Integer);
    procedure SetLineIndex(Value: Integer);
    function GetLineCount: Integer;
    procedure DoClick; override;
    procedure DoKeyDown(CharKey: Boolean; var Key: Word; Shift: TShiftState);override;
    function GetLine(Index:Integer): String;
    procedure SetLine(Index:Integer;Value: String);
    Procedure DrawText; virtual;
  public
    constructor Create(AOwner: TOmegaGUIList); override;
    destructor Destroy; override;
    procedure LoadFromFile(const FileName: string);
    procedure LoadFromStream(Stream: TStream);
    procedure Justify;
    procedure Draw; override;
    function Add(Value: String): Integer; virtual;
    procedure Insert(Index: Integer; Value: String); virtual;
    procedure Delete(Value: Integer); virtual;
    function IndexOf(Value: String): Integer;
    procedure Clear;
    function TextWidth(s: string):Integer;
    property Items[Index: Integer]: String read GetLine write SetLine;
    property NoCarry: Boolean read FNoCarry write FNoCarry;
    property Border: Integer read FBorder write FBorder;
    property BorderY: Integer read FBorderY write FBorderY;
    property LineIndex: Integer read FLineIndex write SetLineIndex;
    property LinePattern: String write SetLinePattern; //length according to a pattern
    property LineNumber: Integer write SetLineNumber;  //sets height according to a number of lines
    property LineCount: Integer read GetLineCount;
    property AutoScroll: Boolean read FAutoScroll write FAutoScroll;
  end;

  TOmegaList = class(TOmegaText)
  private
    FPointedItem: Integer;
    FCanSelect: Boolean;
    FSelected: Integer;
    procedure DoClick; override;
    procedure DoExit; override;
    procedure DoKeyDown(CharKey: Boolean; var Key: Word; Shift: TShiftState);override;
    procedure DoMouseOver; override;
    function GetSelected(Index: Integer): Boolean;
    procedure SetSelected(Index: Integer; Value: Boolean);
    Procedure DrawText; override;
  public
    constructor Create(AOwner: TOmegaGUIList); override;
    destructor Destroy; override;
    procedure Draw; override;
    procedure Sort;
    property CanSelect: Boolean read FCanSelect write FCanSelect;
    property Selected[index: integer]: Boolean read GetSelected write SetSelected;
    property ItemIndex: Integer read FSelected write FSelected;
    property PointedIndex: Integer read FPointedItem write FPointedItem;
  end;

  const MaxMultiListItems=25;
  type
    TMultiListIndex = -1 .. MaxMultiListItems;

  TOmegaMultiList = class(TOmegaList)
  private
  public
  end;

  TBarKind = (bHorizontal, bVertical);
  TOmegaBar = class(TOmegaGUIComponent)
  private
    FKind: TBarKind;
    FPosition: Integer;
    FMin: Integer;
    FMax: Integer;
    FShowPos: Boolean;
    FCanBeClicked: Boolean;
  public
    constructor Create(AOwner: TOmegaGUIList); override;
    procedure Draw; override;
    procedure DoClick; override;
    property Position: Integer read FPosition write FPosition;
    property Min: Integer read FMin write FMin;
    property Max: Integer read FMax write FMax;
    property Kind: TBarKind read FKind write FKind;
    property ShowPos: Boolean read FShowPos write FShowPos;
    property CanBeClicked: Boolean read         FCanBeClicked   write FCanBeClicked;
  end;

  TOmegaScrollBar = class(TOmegaBar)
  private
    procedure DoClick; override;
  public
    procedure Draw; override;
  end;

var
  Reloj: TTimeCaps;
  Timer1, Timer2, TimeDelta: Cardinal;
  OldWndMethod: TWndMethod;

  XFactor, YFactor: Single;

implementation

{
function float(i: Integer): Single;
begin
  Result:=StrToFloat(IntToStr(i));
end;
}

function TOmegaGUI.GetSurface(h: HWND): TWinControl;
var aux: TControl;
    i: Integer;
begin
  Result:=nil;
  if self.Owner=nil then exit;
  If TWinControl(Self.Owner).Handle = h then Result:=TWinControl(Self.Owner)
  else
    for i:=0 to TWinControl(Self.Owner).ControlCount-1 do
    begin
      If TWincontrol(Self.Owner).Controls[i] is TWinControl then
      begin
       aux:=TWinControl(Self.Owner).Controls[i];
       If TWinControl(aux).Handle = h then Result:=TWinControl(aux);
      end; 
    end;
end;

////////////////////////////////////////////////////////////////////////
//                TOmegaGUI component                                 //
////////////////////////////////////////////////////////////////////////
constructor TOmegaGUI.Create(AOwner : TComponent);
begin
  inherited;
  FGUIList:=TList.Create;
  FIDList:=TList.Create;
  FIDList.Add(Self);
  FGUIStatus.Mx:=0;
  FGUIStatus.My:=0;
  FGUIStatus.SelID:=-1;
  FGUIstatus.FocusID:=0;
  FIDCounter:=1;
end;

Destructor TOmegaGUI.Destroy;
begin
   While FGUIList.Count > 0 do
     TOmegaGUIList(FGUIList.Items[FGUIList.Count - 1]).Free;
   FGUIList.Free;
   If FInitializated then Stop;
   inherited;
end;

function TOmegaGUI.CheckClickPanel: Integer;
var i:Integer;
    p: TOmegaGUIPanel;
begin
  Result:=-1;
  for i:=0 to FGUIList.Count -1 do
  begin
    If not (TOmegaGUIList(FGUIList.Items[i]) is TOmegaGUIPanel) then Continue;
    p:=TOmegaGUIPanel(FGUIList.Items[i]);
    If not p.CanBedragged then Continue;
    If (FGUIStatus.Mx> p.FX) and (FGUIStatus.Mx< p.FX + p.FWidth) and
       (FGUIStatus.My> p.FY) and (FGUIStatus.My< p.FY + p.FHeight -
        (p.Height - p.DragHeight)) then
         Result:=i;
  end;
end;

procedure TOmegaGUI.Init;
begin
  if (OmegaScreen = nil) then Exit;
  if not FInitializated then
  begin
   TimeGetDevCaps(@Reloj,sizeof(Reloj));
   TimeBeginPeriod(Reloj.wPeriodMin);
   Application.OnMessage:=WindowsMessages;
   OldWndMethod:=TWinControl(Owner).WindowProc;
   TWinControl(Owner).WindowProc:=WM;
   FInitializated:=True;
  end;
  FScreenHandle:=OmegaScreen.ThehWnd;
  SetFocus(FScreenHandle);

//  XFactor:=Float(FOmegaScreen.Width) /
  XFactor:=FOmegaScreen.Width /
   TWinControl(Getsurface(FOmegaScreen.ThehWnd)).ClientWidth;
//  YFactor:=Float(FOmegaScreen.Height) /
  YFactor:=FOmegaScreen.Height /
   TWinControl(Getsurface(FOmegaScreen.ThehWnd)).ClientHeight;
  If FOmegaScreen.Width = 0 then XFactor:=1;
  If FOmegaScreen.Height = 0 then YFactor:=1;
end;

procedure TOmegaGUI.Stop;
begin
  Application.OnMessage:=nil;
  TimeEndPeriod(Reloj.wPeriodMin);
  if owner<>nil then TWinControl(Owner).WindowProc:=OldWndMethod;
  FInitializated:=False;
end;

procedure TOmegaGUI.ProcessPos;
var i:integer;
begin
// if X/Y-factor changed
//  XFactor:=Float(FOmegaScreen.Width) /
  XFactor:=FOmegaScreen.Width /
   TWinControl(Getsurface(FOmegaScreen.ThehWnd)).ClientWidth;
//  YFactor:=Float(FOmegaScreen.Height) /
  YFactor:=FOmegaScreen.Height /
   TWinControl(Getsurface(FOmegaScreen.ThehWnd)).ClientHeight;
  If FOmegaScreen.Width = 0 then XFactor:=1;
  If FOmegaScreen.Height = 0 then YFactor:=1;

  If (FGUIList=nil) then Exit;
  If (FGUIList.Count=0) then exit;
  For i:=0 To FGUIList.Count-1 do
  begin
    If TOmegaGUIList(FGUIList.Items[i]) is TOmegaGUIPanel then
      TOmegaGUIPanel(FGUIList.Items[i]).ProcessPos
    else
   // if panel was under mouse last frame, maybe it was dragged.
   // Avoid procesing all other components outside the panel
   // until it isn't under mouse
//    If FGUIStatusOld.SelID<>-2 then
      TOmegaGUIList(FGUIList.Items[i]).ProcessPos;
  end;
end;

procedure TOmegaGUI.ProcessBtn(Btn: TMouseButton; Click, Dbl: Boolean);
var Temp: TOmegaGUIComponent;
    Temp2: TGUIStatus;
    Btns: TShiftState;
    p: Integer;
begin
  If (FGUIList=nil) then Exit;
  If (FGUIList.Count=0) then exit;
  If (SelID=0) then Exit;
    for p:=0 to FGUIList.Count-1 do
      If TOmegaGUIList(FGUIList.Items[p]) is TOmegaGUIPanel then
        TOmegaGUIPanel(FGUIList.Items[p]).FdraggedX:=-1;

  Btns:=[];
  Temp2:=FGUIStatus;
  If Temp2.B1 then Btns:=Btns + [ssLeft];
  If Temp2.B2 then Btns:=Btns + [ssRight];
  If Temp2.B3 then Btns:=Btns + [ssMiddle];

  If SelID>0 then
  begin
    Temp:=FIDList.Items[SelID];
    If Dbl then Temp.DoDblClick else
    begin
      If Click then begin
        If Assigned(Temp.OnMouseDown) then
         Temp.OnMouseDown(Self,Btn,Btns,Temp2.Mx,Temp2.My);
      end else begin
        If (Btn=mbLeft) and (Temp2.PressID=Temp2.SelID) then
          Temp.DoClick;
        If (Btn<>mbLeft) or (Temp2.PressID<>Temp2.SelID) then
          If Assigned(Temp.OnMouseUp) then 
           Temp.OnMouseUp(Self,Btn,Btns,Temp2.Mx,Temp2.My);
      end;
    end;
  end;

  If SelID=-2 then
  begin
    p:=CheckClickPanel;
    If p<0 then Exit;  //if panel is dragged out of DragHeight area, exit
    If not Dbl then
    If Click then begin
      If TOmegaGUIPanel(FGUIList.Items[p]).FdraggedX=-1 then
      TOmegaGUIPanel(FGUIList.Items[p]).Drag;
    end else begin
      If (Btn<>mbLeft) then// or (Temp2.PressID<>Temp2.SelID) then
        TOmegaGUIPanel(FGUIList.Items[p]).UnDrag;
    end;
  end;

end;

procedure TOmegaGUI.ProcessKey(CharKey: Boolean; WParam, LParam: Word);
var
  Temp: TOmegaGUIComponent;
begin
    //if not char key, check if it is Return or Space and
    //if a button is selected, so we call the CLick method
{  If (not CharKey) and (TOmegaGUIComponent(FIDList[SelID]) is TOmegaButton) then
    If (WParam = VK_RETURN) or (WParam = VK_SPACE) then
      TOmegaButton(FIDList[SelID]).DoClick;}
  If FocusID<1 then Exit;
  Temp:=FIDList.Items[FocusID];
  If Temp.FParent.Enabled then
    Temp.DoKeyDown(CharKey,WParam,[]);
end;

procedure TOmegaGUI.Draw;
var i:Integer;
begin
  If FBackGround<> nil then
    FBackGround.Draw(0,0,-1);
  If (FGUIList=nil) then Exit;
  For i:=0 To FGUIList.Count-1 do
   TOmegaGUIList(FGUIList.Items[i]).Draw; //draw each list
end;

procedure TOmegaGUI.WM(var Message:TMessage);
begin
{  if message.Msg = WM_KEYDOWN then begin
 //if handled, it doesn't reach
    Application.MainForm.Caption:='teclado new';
  end;
  if message.Msg = WM_CHAR then begin
    Application.MainForm.Caption:='CHAR new';
  end;
  if message.Msg = WM_MOUSEWHEEL then begin
    Application.MainForm.Caption:='Wheel is working!';
  end;}
  OldWndMethod(Message);
end;

//Trap Windows Messages
procedure TOmegaGUI.WindowsMessages(var Msg: TMsg; var Handled: Boolean);
begin
  If Msg.hwnd<>FScreenHandle then Exit;         //If the Msg is not for us Exit
  If GetFocus<>(FScreenHandle) then SetFocus(FScreenHandle);
  if Msg.Message = WM_MOUSEMOVE then
  begin
    Timer1:=TimeGetTime;                          //If last Msg handle was less
    If (Timer1-Timer2)<10 then Exit;              //than 50 ms ago, Exit
    Timer2:=Timer1;

    FGUIStatusOld:=FGUIStatus;
    FGUIStatus.Mx:=Msg.lParam and $FFFF;
    FGUIStatus.My:=Msg.lParam shr 16;

    FGUIStatus.Mx:=Round(FGUIStatus.Mx * XFactor);
    FGUIStatus.My:=Round(FGUIStatus.My * YFactor);
//    FGUIStatus.Mx:=Round(Float(FGUIStatus.Mx) * XFactor);
//    FGUIStatus.My:=Round(Float(FGUIStatus.My) * YFactor);
    FGUIStatus.SelID:=0;
    ProcessPos;
    Handled := True;
  end;
  if Msg.Message = WM_LBUTTONDOWN then begin
    FGUIStatusOld:=FGUIStatus;
    FGUIStatus.B1:=True;
    FGUIStatus.PressID:=FGUIStatus.SelID;
    ProcessBtn(mbLeft, True, False);
    Handled := True;
  end;
  if Msg.Message = WM_LBUTTONUP then begin
    FGUIStatusOld:=FGUIStatus;
    FGUIStatus.B1:=False;
    ProcessBtn(mbLeft,False, False);
    FGUIStatus.PressID:=-1;
    Handled := True;
  end;
  if Msg.Message = WM_RBUTTONDOWN then begin
    FGUIStatusOld:=FGUIStatus;
    FGUIStatus.B2:=True;
    ProcessBtn(mbRight,true, False);
    Handled := True;
  end;
  if Msg.Message = WM_RBUTTONUP then begin
    FGUIStatusOld:=FGUIStatus;
    FGUIStatus.B2:=False;
    ProcessBtn(mbRight,False, False);
    Handled := True;
  end;
  if Msg.Message = WM_MBUTTONDOWN then begin
    FGUIStatusOld:=FGUIStatus;
    FGUIStatus.B3:=True;
    ProcessBtn(mbMiddle,true, False);
    Handled := True;
  end;
  if Msg.Message = WM_MBUTTONUP then begin
    FGUIStatusOld:=FGUIStatus;
    FGUIStatus.B3:=False;
    ProcessBtn(mbMiddle,False, False);
    Handled := True;
  end;
  if Msg.Message = WM_LBUTTONDBLCLK then begin
    FGUIStatusOld:=FGUIStatus;
    FGUIStatus.B1:=True;
    ProcessBtn(mbLeft,True, True);
    Handled := True;
  end;
{  if Msg.Message = $020B then begin //boton X1 y X2 DOWN, 20C UP, 20D Dbl
    FGUIStatusOld:=FGUIStatus;
    FGUIStatus.B3:=True;
    ProcessBtn(mbMiddle,true, False);
    Handled := True;
  end;}
  if Msg.Message = WM_CHAR then begin
    ProcessKey(True, Msg.wParam,Msg.lParam);
    Handled := True;
  end;
  if Msg.Message = WM_KEYDOWN then begin
    ProcessKey(False, Msg.wParam,Msg.lParam);
    Handled := False;
  end;
end;

procedure TOmegaGUI.CursorPos(x,y: Integer);
var x2,y2: Integer;
begin
{  If TWinControl(Owner).Handle=FScreenHandle then
  begin}
    x2:=TWinControl(Owner).Left;
    y2:=TWinControl(Owner).Top;
{  end else
    for i:=0 to (Owner.ComponentCount-1) do
    begin
      If not (Owner.Components[i] is TWinControl) then Continue;
      If TWinControl(Owner.Components[i]).Handle=FScreenHandle then
      begin
        x2:=TWinControl(Owner.Components[i]).Left;
        y2:=TWinControl(Owner.Components[i]).Top;
      end;
    end;}
{  X2:=FHandledPos.X;
  Y2:=FHandledPos.Y;}

  //  SetCursorPos(x+x2 ,y+y2);
 SetCursorPos(Trunc((x+x2)*XFactor), Trunc((y+y2)*YFactor));
end;

function TOmegaGUI.FindID(i: Integer): TOmegaGUIComponent;
begin
  Result:=FIDList.Items[i];
end;

////////////////////////////////////////////////////////////////////////
//                TOmegaGUIList class                                 //
////////////////////////////////////////////////////////////////////////

constructor TOmegaGUIList.Create(AOwner : TOmegaGUI);
begin
  inherited Create;
  FList:=TList.Create;
  FParent:=AOwner;
  FParent.FGUIList.Add(Self);
  FEnabled:=True;
  FVisible:=True;
end;

//destroy the list and all child components 
destructor TOmegaGUIList.Destroy;
begin
  Clear;
  FList.Destroy;
  FParent.FGUIList.Remove(Self);
  inherited;
end;

procedure TOmegaGUIList.Clear;
var i:Integer;
begin
  for i:=0 to (FList.Count-1) do
    TOmegaGUIComponent(FList.Items[0]).Destroy;
end;

procedure TOmegaGUIList.Add(Sender: TOmegaGUIComponent);
begin
  FList.Add(Sender);
end;

//deletes a certain component inside the list
procedure TOmegaGUIList.Delete(Value: Integer);
var i: Integer;
    Index: Integer;
begin
  Index:=-1;
  For i:=0 to (FList.Count-1) do
    If TOmegaGUIComponent(FList.Items[i]).FID=Value then
      Index:=i;
  If Index>-1 then FList.Delete(Index);
end;

function TOmegaGUIList.AssignID: Cardinal;
begin
  Result:=FParent.FIDCounter;
  FParent.FIDCounter:=FParent.FIDCounter+1;
end;

procedure TOmegaGUIList.SetID(Value: Integer);
begin
  FParent.FGUIStatus.SelID:=Value;
end;

procedure TOmegaGUIList.SetFocus(Value: Integer);
begin
  FParent.FGUIStatus.FocusID:=Value;
end;

function TOmegaGUIList.GetID: Integer;
begin
  Result:=FParent.FGUIStatus.SelID;
end;

function TOmegaGUIList.GetOldID: Integer;
begin
  Result:=FParent.FGUIStatusOld.SelID;
end;

//see what component in the list is under mouse pointer, if any
procedure TOmegaGUIList.ProcessPos;

  function PtInRect2(R:TRect; P:TPoint):Boolean;
  begin
    If (P.X>=R.Left) and (P.X<=R.Right) and (P.Y>=R.Top) and (P.Y<=R.Bottom) 
then
      Result:=True
    else
      Result:=False;
  end;

var
  i:integer;
  Temp: TOmegaGUIComponent;
  Status: TGUIStatus;
begin
  If not(FEnabled) or not(FVisible) then Exit;
  Status:=FParent.FGUIStatus;                           //Get the actual status record
  For i:=0 To (FList.Count)-1 do                        //check all components in list
  begin
    Temp:=(TOmegaGUIComponent(FList.Items[i]));
    If (Temp.FEnabled) and (Temp.FVisible) then         //if component is enabled/visible
    begin
      with Temp do                                      //test is mouse is over component
        If
PtInRect2(Rect(FX,FY,FX+FWidth,FY+FHeight),Point(Status.Mx,Status.My)) then
        begin
          SetID(FID);                                   //the component is under mouse
          DoMouseOver;
          If GetOldID<>FID then                         //if is just entered
            DoEnter;                                    //call the event
        end else                                        //else
          If GetOldID=FID then                          //If it was under mouse
            DoExit;                                     //call the onexit event
    end;
  end;
end;

procedure TOmegaGUIList.ProcessBtn;
begin

end;

// General component draw procedure
procedure TOmegaGUIList.Draw;
var
  i:integer;
  Temp: TOmegaGUIComponent;
  index:integer;
  sx, sy: Double;
//  r:TRect;
begin
  If not(FVisible) then Exit;
  //

  For i:=0 To (FList.Count)-1 do
  begin
    Temp:=(TOmegaGUIComponent(FList.Items[i]));
    If Temp.FVisible then
      Temp.Draw;
  end;
end;

procedure TOmegaGUIList.ZTop;
begin
  If FParent.FGUIList.Remove(Self)>-1 then
    FParent.FGUIList.Add(Self);
end;

procedure TOmegaGUIList.ZBottom;
begin
  if FParent.FGUIList.Remove(Self)>-1 then
    FParent.FGUIList.Insert(0,Self);
end;
////////////////////////////////////////////////////////////////////////
//                TOmegaGUIPanel class                                //
////////////////////////////////////////////////////////////////////////

constructor TOmegaGUIPanel.Create(AOwner: TOmegaGUI);
begin
  inherited;
  FX:=1; FY:=1;
  FWidth:=10; FHeight:= 10;
  FCaption:='';
  FdraggedX:=-1;
  FdraggedY:=-1;         //only draggedX is enough, but this is done for clarity
  FCanBedragged:=False;
  FDragWidth:=FWidth;
  FDragHeight:=FHeight;
end;

procedure TOmegaGUIPanel.ProcessPos;
begin
  If FCanBedragged then
  If (FParent.FGUIStatus.Mx> FX) and (FParent.FGUIStatus.Mx< FX + FWidth) and
     (FParent.FGUIStatus.My> FY) and (FParent.FGUIStatus.My< FY + FHeight) then
       FParent.FGUIStatus.SelID:=-2;
  inherited;
end;

procedure TOmegaGUIPanel.Add(Sender: TOmegaGUIComponent);
begin
  inherited;
  Sender.FX:=Sender.FX+FX;                      //add relative position of panel
  Sender.FY:=Sender.FY+FY;

  If Sender.FX+Sender.FWidth> FX+FWidth then    //and do not let it go out
    Sender.FX:=(FX+Width)-Sender.FWidth;
  If Sender.FX< FX then
    Sender.FX:=FX;
  If Sender.FY+Sender.FHeight> FY+FHeight then
    Sender.FY:=(FY+Height)-Sender.FHeight;
  If Sender.FY< FY then
    Sender.FY:=FY;
end;

//if you move the panel, or change its Width or Heigth, be aware of checking that
//no component owned by panel get out of its borders :)
procedure TOmegaGUIPanel.CheckPos;
var i:Integer;
    Temp:TOmegaGUIComponent;
begin
  For i:=0 to FList.Count-1 do
  begin
    Temp:=FList[i];
    If Temp.FX+Temp.FWidth> FX+FWidth then
      Temp.FX:=(FX+Width)-Temp.FWidth;
    If Temp.FX < FX then
      Temp.FX:=FX;
    If Temp.FY+Temp.FHeight> FY+FHeight then
      Temp.FY:=(FY+Height)-Temp.FHeight;
    If Temp.FY < FY then
      Temp.FY:=FY;
  end;
end;

//update position of all child components
procedure TOmegaGUIPanel.UpdatePos(DeltaX,DeltaY:Integer);
var i:Integer;
    Temp:TOmegaGUIComponent;
begin
  For i:=0 to FList.Count-1 do
  begin
    Temp:=FList[i];
    Temp.FX:=Temp.FX+DeltaX;
    Temp.FY:=Temp.FY+DeltaY;
  end;
end;

//make all X and Y position of ALL childs relative to panel, not to whole screen
procedure TOmegaGUIPanel.ChildRelative;
var i:Integer;
    Temp:TOmegaGUIComponent;
begin
  For i:=0 to FList.Count-1 do
  begin
    Temp:=FList[i];
    Temp.FX:=Temp.FX+FX;
    Temp.FY:=Temp.FY+FY;
  end;
end;

procedure TOmegaGUIPanel.SetX(Value: Integer);
begin
  UpdatePos(Value-FX,0);
  FX:=Value;
end;

procedure TOmegaGUIPanel.SetY(Value: Integer);
begin
  UpdatePos(0,Value-FY);
  FY:=Value;
end;

procedure TOmegaGUIPanel.SetWidth(Value: Integer);
begin
  FWidth:= Value;
  CheckPos;
end;

procedure TOmegaGUIPanel.SetHeight(Value: Integer);
begin
  FHeight:= Value;
  CheckPos;
end;

//Store mouse position - panel position (relative position inside panel)
procedure TOmegaGUIPanel.Drag;
begin
  FdraggedX:=FParent.FGUIStatus.Mx-FX;
  FdraggedY:=FParent.FGUIStatus.My-FY;
end;

// -1 means no dragging
procedure TOmegaGUIPanel.UnDrag;
begin
  FdraggedX:=-1;
  FdraggedY:=-1;
end;

procedure TOmegaGUIPanel.Draw;
var
//  i:integer;
//  Temp: TOmegaGUIComponent;
  TempO: TOmegaBitmapFont;
//  index:integer;
  sx, sy: Double;
//  oldx, oldy: Integer;
begin
  If FdraggedX>-1 then
  begin
    FoldX:=Fx;
    FoldY:=Fy;
    FX:=FX - (FOldX - FParent.FGUIStatus.Mx)-FdraggedX;
    FY:=FY - (FOldY - FParent.FGUIStatus.My)-FdraggedY;

    UpdatePos(FX-Foldx, FY-Foldy);
  end;

  IF not(FVisible) then Exit;
  TempO:=FParent.FOmegaBitmapFont;
  sx:=1; sy:=1;
  If Image<> nil then
{    Image.Draw(FX,FY,0,0.5,0.5,sx,sy,OmegaColor(255,255,255,255),
                      MirrorH,MirrorV,ImageIndex,BlendMode);}

//    Image.DrawRect(Rect(0,0,Width,Height),FX,FY,0,0.5,0.5,sx,sy,255,255,255,195, False);
//    Image.DrawInRect(Rect(0,0,Width,Height),FX,FY,0,0.5,0.5,sx,sy,255,255,255,195, False);
{    Image.DrawInRect(Rect(0,0,Width,Height),FX,FY,0,0.5,0.5,sx,sy,
                    FImage.Color.Red,FImage.Color.Green,FImage.Color.Blue,FImage.Color.alpha, False);}


    Image.DrawInRect(Rect(0,0,Width,Height),FX,FY,0,0.5,0.5,sx,sy,
                     FImage.Color.Red,FImage.Color.Green,FImage.Color.Blue,Fimage.Color.Alpha,MirrorH,MirrorV,0,BlendMode);


  TempO.Print(X+((Width shr 1)-TempO.TextWidth(FCaption) shr 1),
              Y+((Height shr 1)-TempO.TextHeight(FCaption) shr 1),FCaption);

// original draw method, recalculating FX, FY.
  inherited;
end;

////////////////////////////////////////////////////////////////////////
//                TOmegaGUIComponent class                            //
////////////////////////////////////////////////////////////////////////
//cuidar de no instanciar una variable objeto mas de una vez

constructor TOmegaGUIComponent.Create(AOwner : TOmegaGUIList);
var i:integer;
begin
  inherited Create;
  FParent:=AOwner;
  FName:= '';
  FVisible:= True;
  FEnabled:= True;
  FX:=0; FY:=0;
  FWidth:=20; FHeight:=15;
  FRotate:=0;
  For i:=0 to 3 do
  begin
    FImage[i].Image:=nil;
    FImage[i].ImageIndex:=-1;
    FImage[i].OrigIndex:=-1;
    FImage[i].DoAnimate:=False;
    FImage[i].AnimCount:=0;
    FImage[i].AnimSpeed:=0;
    FImage[i].Animloop:=true;
    FImage[i].AnimOver:=False;
    FImage[i].AnimBack:=False;
//    FImage[i].AlphaBlend:=False;
    FImage[i].BlendMode:=bmNormal;
    FImage[i].Color:=OmegaColor(255,255,255,255);
  end;
  FParent.Add(Self);
  FID:=FParent.AssignID;
  FParent.FParent.FIDList.Add(Self);
  FOmegaBitmapFont:= FParent.FParent.FOmegaBitmapFont;  //by default, use Bitmap
                                                        //font assigned to GUI
  FFontColor.Red:=FOmegaBitmapFont.Red;
  FFontColor.Green:=FOmegaBitmapFont.Green;
  FFontColor.Blue:=FOmegaBitmapFont.Blue;
  FFontColor.Alpha:=FOmegaBitmapFont.Alpha;
  FFontRotate:=0;
  FFontScaleX:=FOmegaBitmapFont.ScaleX;
  FFontScaleY:=FOmegaBitmapFont.ScaleY;
end;

destructor TOmegaGUIComponent.Destroy;
begin
  FParent.Delete(FID);
  inherited;
end;

procedure TOmegaGUIComponent.ChangeList(Dest: TOmegaGUIList);
begin
  FParent.Delete(FID);
  Dest.Add(Self);
end;

procedure TOmegaGUIComponent.Animate(var Pattern:TOmegaGUIImageRecord);
begin
  If Pattern.AnimOver then Exit;
        //advance animation counter
  Self.FAnimCounter := Self.FAnimCounter + 1;
        //check if need to replace current frame
  If (Pattern.AnimSpeed div 10 <= Self.FAnimCounter) then
  begin
  If not pattern.AnimBack then
  begin
    Self.FAnimCounter:=0;                        //reset counter
    Pattern.ImageIndex:= Pattern.ImageIndex + 1; //advance frame
        //now check if animation is a loop animation
      If Pattern.ImageIndex > (Pattern.OrigIndex + Pattern.AnimCount) then
      begin
        if pattern.animloop then
          Pattern.ImageIndex:= Pattern.OrigIndex;
        if not pattern.animloop then
        begin
          Pattern.AnimOver := True;
          Pattern.ImageIndex:= Pattern.OrigIndex + Pattern.AnimCount;
        end;
      end;
  end;

  If pattern.AnimBack then
  begin
    Self.FAnimCounter:= 0;                                     //reset counter
    Pattern.ImageIndex:= Pattern.ImageIndex - 1;                //advance frame
        //now check if animation is a loop animation
      If Pattern.ImageIndex < (Pattern.OrigIndex + 1) then
      begin
        if pattern.animloop then
          Pattern.ImageIndex:= Pattern.OrigIndex + Pattern.AnimCount;
        if not pattern.animloop then
        begin
          Pattern.AnimOver := True;
          Pattern.ImageIndex := Pattern.OrigIndex;
        end;
      end;
  end;

  end;
end;

procedure TOmegaGUIComponent.ResetAnimation(var Pattern:TOmegaGUIImageRecord);
begin
  If not pattern.AnimBack then
    Pattern.ImageIndex := Pattern.OrigIndex ;
  If pattern.AnimBack then
    Pattern.ImageIndex := Pattern.OrigIndex + Pattern.AnimCount;
  Pattern.AnimOver:= False;
end;

procedure TOmegaGUIComponent.DoClick;
begin
//  FParent.SetFocus(-1);
  If Assigned(FOnClick) then FOnClick(Self);
end;

procedure TOmegaGUIComponent.DoDblClick;
begin
//  FParent.SetFocus(-1);
  If Assigned(FOnDblClick) then FOnDblClick(Self);
end;

procedure TOmegaGUIComponent.DoKeyDown(charKey:Boolean; var Key: Word; Shift: 
TShiftState);
begin
  If Assigned(FOnKeyPress) then FOnKeyPress(Self, Key, Shift);
end;

procedure TOmegaGUIComponent.DoMouseOver;
begin
//
end;

{procedure(Sender: TObject; var Key: Word;
    Shift: TShiftState);}
{procedure TOmegaGUIComponent.MouseDown;
begin
end;

procedure TOmegaGUIComponent.MouseUp;
begin
end;}

  //when enter a component, reactivate animations
  //this makes posible to animate always that pointer enters component area, ...
procedure TOmegaGUIComponent.DoEnter;
var i: Integer;
begin
  if Assigned(OnEnter) then OnEnter(Self);
  for i:=0 to 3 do
    Self.ResetAnimation( Self.FImage[i] );

//    Self.FImage[i].AnimOver := False;
end;

  //when leave a component, reactivate animations
  //this makes posible to animate always that pointer enters component area, ...
procedure TOmegaGUIComponent.DoExit;
var i: Integer;
begin
  If Assigned(OnExit) then OnExit(Self);
  for i:=0 to 3 do
    Self.ResetAnimation( Self.FImage[i] );
//    Self.FImage[i].AnimOver := False;
end;

procedure TOmegaGUIComponent.SetImage(index: integer;Value: TImageListItem);
begin
  FImage[index].Image:=Value;
end;

function TOmegaGUIComponent.GetImage(index: integer): TImageListItem;
begin
  Result:=FImage[index].Image;
end;

procedure TOmegaGUIComponent.SetImageIndex(index: integer; Value: Integer);
begin
  FImage[index].ImageIndex:=Value;
  FImage[index].OrigIndex:=Value;
end;

function TOmegaGUIComponent.GetImageIndex(index: integer): Integer;
begin
  Result:=FImage[index].ImageIndex;
end;

procedure TOmegaGUIComponent.SetAnim(index: integer; Value: Boolean);
begin
  FImage[index].DoAnimate:=Value;
end;

function TOmegaGUIComponent.GetAnim(index: integer): Boolean;
begin
  Result:=FImage[index].DoAnimate;
end;

procedure TOmegaGUIComponent.SetAnimC(index: integer; Value: Integer);
begin
  FImage[index].AnimCount:=Value;
end;

function TOmegaGUIComponent.GetAnimC(index: integer): Integer;
begin
  Result:=FImage[index].AnimCount;
end;

procedure TOmegaGUIComponent.SetAnimS(index: integer; Value: Integer);
begin
  FImage[index].AnimSpeed:=Value;
end;

function TOmegaGUIComponent.GetAnimLoop(index: integer): boolean;
begin
  Result:=FImage[index].AnimLoop;
end;

procedure TOmegaGUIComponent.SetAnimLoop(index: integer; Value: boolean);
begin
  FImage[index].AnimLoop:=Value;
end;

function TOmegaGUIComponent.GetAnimBack(index: integer): boolean;
begin
  Result:=FImage[index].AnimBack;
end;

procedure TOmegaGUIComponent.SetAnimBack(index: integer; Value: boolean);
begin
  FImage[index].AnimBack:=Value;
end;

function TOmegaGUIComponent.GetAnimS(index: integer): Integer;
begin
  Result:=FImage[index].AnimSpeed;
end;

procedure TOmegaGUIComponent.SetOrig(index: integer; Value: Integer);
begin
  FImage[index].OrigIndex:=Value;
end;

function TOmegaGUIComponent.GetOrig(index: integer): Integer;
begin
  Result:=FImage[index].OrigIndex;
end;

procedure TOmegaGUIComponent.SetBlend(index: integer; Value: Boolean);
begin
  FImage[index].AlphaBlend:=Value;
end;

function TOmegaGUIComponent.GetBlend(index: integer): Boolean;
begin
  Result:=Fimage[index].AlphaBlend;
end;

procedure TOmegaGUIComponent.SetBlendMode(index: integer; Value: Integer);
begin
  FImage[index].BlendMode:=Value;
end;

function TOmegaGUIComponent.GetBlendMode(index: integer): Integer;
begin
  Result:=Fimage[index].BlendMode;
end;

procedure TOmegaGUIComponent.SetColor(index: integer; Value: TOmegaColor);
begin
  FImage[index].Color:=Value;
end;

function TOmegaGUIComponent.GetColor(index: integer): TOmegaColor;
begin
  Result:=FImage[index].Color;
end;

procedure TOmegaGUIComponent.SetAlpha(index: integer; Value: Byte);
begin
  FImage[index].Color.Alpha:=Value;
end;

function TOmegaGUIComponent.GetAlpha(index: integer): Byte;
begin
  Result:=FImage[index].Color.Alpha;
end;

procedure TOmegaGUIComponent.SetX(Value: Integer);
begin
  FX:=Value;
end;

procedure TOmegaGUIComponent.SetY(Value: Integer);
begin
  FY:=Value;
end;

procedure TOmegaGUIComponent.SetWidth(Value: Integer);
begin
  FWidth:=Value;
end;

procedure TOmegaGUIComponent.SetHeight(Value: Integer);
begin
  FHeight:=Value;
end;

procedure TOmegaGUIComponent.ZTop;
begin
  If FParent.FList.Remove(Self)>-1 then
    FParent.FList.Add(Self);
end;

procedure TOmegaGUIComponent.ZBottom;
begin
  If FParent.FList.Remove(Self)>-1 then
    FParent.FList.Insert(0,Self);
end;

procedure TOmegaGUIComponent.Draw;
var index: Integer;
begin
      index:=0;
      If FParent.GetID = FID then                    //if mouse over component
        If (FParent.FParent.FGUIStatus.B1) then index:=2        //and button clicked
          else                                  //if no button click
            index:=1;
      If Image[index]=nil then index:=0;   //if that image is not assigned, then
                                                //turn back to default image.

      Image[index].DrawInRect(Rect(0, 0, Width, Height), FX, FY,
                             Rotation, 0.5, 0.5, 1, 1,
                             Color[index].Red,
                             Color[index].Green,
                             Color[index].Blue,
                             Color[index].Alpha,
                             MirrorH, MirrorV,
                             ImageIndex[index],
                             BlendMode[index]);
                                                // draw and active image (if aplicable)
                                                //or if it has the focus
                                                //check weather is animated
      If Anim[index] then Animate( FImage[index] );
                                                //and call an optional component draw
    //*check weather do active draw here
end;

////////////////////////////////////////////////////////////////////////
//                TOmegaButton                                        //
////////////////////////////////////////////////////////////////////////

constructor TOmegaButton.Create(AOwner : TOmegaGUIList);
begin
  inherited;
  FCentered:=True;
  FCaption:=IntToStr(FID);
  FGroup:=-1;
  FTextleft:=0;
  FTexttop:=0;
  FRepeating:=False;
  FCheckeable:=False;
  FFontColor:=OmegaColor(255,255,255,255);
end;

procedure TOmegaButton.Draw;
begin
inherited;
  // Draw highlited image if focussed
  //---------------------------------
{  If (FParent.FParent.FGUIStatus.FocusID=FID) then
    If (Image[1]<>nil) then

Image[1].Draw(FX,FY,0,0.5,0.5,1,1,OmegaColor(Color[1].Red,Color[1].Green,Color[1
].Blue,Color[1].Alpha),
                    MirrorH, MirrorV,ImageIndex[1], BlendMode[1]);
  If Anim[1] then Animate(FImage[1]);       //check weather is animated}

  // Draw focussed image if active
  //-------------------------------
  If active then
    If (Image[3]<>nil) then

      Image[3].Draw(FX,FY,0,0.5,0.5,1,1,OmegaColor(Color[3].Red,Color[3].Green,Color[3].Blue,Color[3].Alpha),
                    MirrorH, MirrorV, ImageIndex[3],BlendMode[3]);
  If Anim[3] then Animate(FImage[3]);

  // Draw caption over images
  //-------------------------
  // I use "shr 1" instead of "div 2" for performance. This line centers the caption
  If FCentered then
    with FOmegaBitmapFont do
    FOmegaBitmapFont.Print(X+ftextleft+((Width shr 1)- Round(TextWidth(FCaption)*FontScaleX) shr 1),
     ftexttop+Y+((Height shr 1)- Round(TextHeight(' ')*FontScaleY) shr 1),
     0, FontScaleX, FontScaleY,
     FFontColor,FCaption)
  else
    with FOmegaBitmapFont do
    FOmegaBitmapFont.Print(X+ftextleft+((Width shr 2)),
     ftexttop+Y+((Height shr 1)- Round(TextHeight(' ')*FontScaleY) shr 1),
     0, FontScaleX, FontScaleY,
     FFontColor,FCaption);
end;

procedure TOmegaButton.DoClick;
var i:Integer;
    Temp: TOmegaGUIComponent;
begin
  inherited;
  If FCheckeable then                           //if is CheckButton
    Factive:=not(Factive);                      //change state
    If (Factive) and (FGroup>-1) then           //if we just activated it
    begin                                       //and it belongs to a group
      with FParent do                           //look all component in its list
        For i:=0 to FList.Count-1 do            //test if they are CheckButtons too
        begin                                   //and if they are in the same group
           Temp:=FList.Items[i];
           If (Temp is TOmegaButton) and (Temp<>Self) then
             If TOmegaButton(FList.Items[i]).FGroup=Self.FGroup then
               Temp.active:=False;             //if so, deactivate it
        end;

  end;
end;

procedure TOmegaButton.DoKeyDown(CharKey: Boolean; var Key: Word; Shift: 
TShiftState);
begin
  inherited;
  If CharKey then Exit;
  If (Key=VK_RETURN) or (Key=VK_SPACE) then DoClick;
end;

////////////////////////////////////////////////////////////////////////
//                TOmegaEdit                                          //
////////////////////////////////////////////////////////////////////////
//remember that you cannot use FOmegaBitmapFont.TextWidth or TextHeight, as
//OmegaEdit has its own FontscaleX and Y properties

constructor TOmegaEdit.Create(AOwner: TOmegaGUIList);
begin
  inherited;
  Ftext:='';
  FTextPoint:=0;
  FTextBlend:=0;
  FMaxLength:=-1;
  FBorder:=5;
  FReadOnly:=False;
  FInputChar:=eicNormal;
  FCursorChar:='_';
  FCursorShow:=True;
  FfontScalex:=FOmegaBitmapFont.ScaleX;
  FfontScaley:=FOmegaBitmapFont.ScaleY;
end;

//There can be whole font rotation (rotation) OR
//individual character rotation (fontrotation)
procedure TOmegaEdit.Draw;
var Temp: Integer;
    i: Integer;
//    s: String;
begin
  inherited;
  
  If FFontRotate=0 then
  begin
    IF FRotate<>0 then          //if there is rotation, change center of rotation
    begin
     FOmegaBitmapFont.CenterX:=(Trunc(Width*FontScaleX) shr 1);
     FOmegaBitmapFont.CenterY:=(Trunc(Height*FontScaleY) shr 1);
    end;
    FOmegaBitmapFont.Print(FX+FBorder,FY+(FHeight shr 1)-
     (Trunc(FOmegaBitmapFont.CharHeight*FontScaleY) shr 1),
     FRotate,FFontScaleX,FFontScaleY,FFontColor,FText,FTextBlend)
  end else
  begin                          //if font rotation
    If Length(FText)>0 then
     for i:=0 to Length(FText) do
     begin
      Temp:=Round((i-1)*(FOmegaBitmapFont.TextWidth(FText[i])//CharWidth
       * FFontScaleX));
       FOmegaBitmapFont.Print(FX+FBorder+Temp,FY+(FHeight shr 1)-
       (Trunc(FOmegaBitmapFont.CharHeight * FontScaleY) shr 1),
        FFontRotate,FFontScaleX,FFontScaleY,FFontColor,FText[i], FTextBlend);
     end;
  end;

  //print cursor
  If FCursorShow then
  If (FParent.FParent.FGUIStatus.FocusID=FID) then
  begin
    If FRotate=0 then
    begin
      Temp:=Round(FOmegaBitmapFont.TextWidth(copy(FText,1,FTextPoint))*FontScaleX);
      FOmegaBitmapFont.Print(FX+FBorder+Temp,
       FY+((FHeight shr 1)-(Trunc(FOmegaBitmapFont.CharHeight*FontScaleY) shr 1)),
       FFontRotate,FFontScaleX,FFontScaleY,FFontColor,FCursorChar,FTextBlend)
    end
    else begin
      Temp:=Round(FOmegaBitmapFont.TextWidth(copy(FText,1,FTextPoint)));
      FOmegaBitmapFont.Print(
       round(FX+FBorder+temp*cos(FRotate*pi/180)),
       round(temp*sin(FRotate*pi/180))+
       FY+((FHeight shr 1)-(Trunc(FOmegaBitmapFont.CharHeight*FontScaleY) shr 1)),
       FRotate,FFontScaleX,FFontScaleY,FFontColor,FCursorChar,FTextBlend)
    end;
  end;
  FOmegaBitmapFont.CenterX:=0;
  FOmegaBitmapFont.CenterY:=0;
                                       //if it has the focus
  If (Image[3]<>nil) and               //as far as there is an image assigned
   ((Factive) or (FParent.FParent.FGUIStatus.FocusID=FID)) then
  begin
   Image[3].Draw(FX,FY,FRotate,0.5,0.5,1,1,OmegaColor(Color[3].Red,Color[3].Green,
    Color[3].Blue,Color[3].Alpha),MirrorH, MirrorV, ImageIndex[3],BlendMode[3]);
  end;
  If Anim[3] then Animate(FImage[3]);       //check if it animated
end;

procedure TOmegaEdit.DoClick;
begin
  inherited;
  FParent.SetFocus(FID);
end;

//keyboard control
procedure TOmegaEdit.DoKeyDown(CharKey: Boolean; var Key: Word;
 Shift: TShiftState);

  function UpCase(C: Word) : Word;
  begin
    Result := C;
    Case Result of
      Ord('a')..Ord('z'): Dec(Result, Ord('a') - Ord('A'));
      Ord(''),Ord(''),Ord(''),Ord(''): Dec(Result, Ord('a') - Ord('A'));
    end;
  end;
  function LoCase(C: Word) : Word;
  begin
    Result := C;
    Case Result of
      Ord('A')..Ord('Z'): Inc(Result, Ord('a') - Ord('A'));
      Ord(''),Ord(''),Ord(''),Ord(''): Inc(Result, Ord('a') - Ord('A'));
    end;
  end;
  function NumKey(C: Char): Boolean;
  begin
    Result:=False;
    Case C of
      '0'..'9' : Result:=True;
    end;
  end;

  function IsUpCase(C: Word) : boolean;
  begin
    Case C of
      Ord('A')..Ord('Z'): Result:=True;
      Ord(''),Ord(''),Ord(''),Ord(''): result:=True;
      else Result:=False;
    end;
  end;

  function IsLoCase(C: Word) : boolean;
  begin
    Case C of
      Ord('a')..Ord('z'): Result:=True;
      Ord(''),Ord(''),Ord(''),Ord(''): result:=True;
      else Result:=False;
    end;
  end;

begin
  inherited;
  If FReadOnly then Exit;
  If CharKey then                               //If we recieve a Char
  begin
    If Key=8 then                               //but it stands for backspace
    begin
        If FTextPoint<1 then Exit;              //if we are over Position 1,
        Delete(FText,FTextPoint,1);             //cursor in the Text
        FTextPoint:=FTextPoint-1;               //we delete the char before the
    end else
    if Key=13 then Exit
    else
    begin                                       //if it is any other char
      If (FMaxLength<0) or
         (Length(FText)<=FMaxLength-1) then
         begin
           If InputChar=eicForceUpperCase then Key:=UpCase((Key));
           If InputChar=eicForceLowerCase then Key:=LoCase((Key));
           If (InputChar=eicUpperCase) and not IsUpCase(Key) then exit;
           If (InputChar=eicLowerCase) and not IsLoCase(Key) then Exit;
           If (InputChar=eicNumeric) and (not NumKey(Chr(Key))) then Exit;
           If not CheckCharValid(Chr(Key)) then Exit;
           Insert(Chr(Key),FText,FTextPoint+1);      //we insert it into the Text
           FTextPoint:=FTextPoint+1;                 //and increment the cursor
         end;
    end;
  end else
  begin                                //If what we recieve isn't a Char
    Case Key of
      VK_DELETE :                               //If it is DEL key
      begin
        If (Length(FText)<1) then Exit;         //as far as there's at least one char
        If (FTextPoint=Length(FText)) then Exit;//or if we are at the end of the line
        Delete(FText,FTextPoint+1,1);           //we delete the one over the cursor
      end;
      VK_LEFT :                                 //If it is LEFT key, move cursor left
      begin
        If FTextPoint<1 then FTextPoint:=1;
        FTextPoint:=FTextPoint-1;
      end;
      VK_RIGHT:                                 //same for right
      begin
        FTextPoint:=FTextPoint+1;
        If FTextPoint>Length(FText) then FTextPoint:=Length(FText);
      end;
      VK_END:                                   //If it is END key, move cursor to end
      begin
        FTextPoint:=Length(FText);
      end;
      VK_HOME:                                  //Same, but now to begining
      begin
        FTextPoint:=0;
      end;
    end;
  end;
//  Application.MainForm.Caption:=IntToStr(Length(FText)) + ' ' + IntToStr(FTextPoint);
end;

function TOmegaEdit.CheckCharValid(c: Char):Boolean;
var i:Integer;
begin
  Result:=False;
  For i:=0 to Length(FOmegaBitmapFont.Characters)-1 do
    If FOmegaBitmapFont.Characters[i]=c then result:=True;
end;

procedure TOmegaEdit.SetLinePattern(Value: String);
begin
  Width:=FParent.FParent.FOmegaBitmapFont.TextWidth(Value)+FBorder;
end;

function TOmegaEdit.GetTextLen:Integer;
begin
  Result:=Length(FText);
end;

procedure TOmegaEdit.Clear;
begin
  FText:='';
  FTextPoint:=0;
end;

////////////////////////////////////////////////////////////////////////
//                TOmegaText                                          //
////////////////////////////////////////////////////////////////////////
constructor TOmegaText.Create(AOwner: TOmegaGUIList);
begin
  inherited;
  FLines:=TStringList.Create;
  FLineIndex:=0;
  FNoCarry:=True;
  FBorder:=2;
  FAutoScroll:=False;
  FFontScaleX:=1;   FFontScaleY:=1;
end;

destructor TOmegaText.Destroy;
begin
  FLines.Free;
  inherited;
end;

procedure TOmegaText.DoClick;
begin
  inherited;
  FParent.SetFocus(FID);
end;

procedure TOmegaText.DoKeyDown(CharKey: Boolean; var Key: Word; Shift: 
TShiftState);
begin
  inherited;
  If CharKey then Exit;
  Case Key of
    VK_DOWN: If FLineIndex<MaxLineIndex then FLineIndex:=FLineIndex+1;
    VK_UP: If FLineIndex>0 then FLineIndex:=FLineIndex-1;
    VK_HOME: FLineIndex:=0;
    VK_END: FLineIndex:=MaxLineIndex;
  end;
end;

procedure TOmegaText.LoadFromFile(const FileName: string);
var
  Stream: TStream;
begin
  Stream:=TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
  try
    LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TOmegaText.LoadFromStream(Stream: TStream);
var
  Size: Integer;                                //Size of Stream
  S: string;                                    //the string we want to load
  P, Start: PChar;                              //pointer to char
  LineText, Sub : String;                       //auxiliar strings
begin
  Size := Stream.Size - Stream.Position;        //size of Stream
  SetString(S, nil, Size);                      //set the size of the string
  Stream.Read(Pointer(S)^, Size);               //and set its content reading from Stream
  FLines.Clear;                                 //clear all text before

  LineText:=''; Sub:='';                        //initialize auxiliar strings
  P:=Pointer(S);                                //Point to begining of string
  If P<>nil then                                //If it isn't empty
    While P^<>#0 do                             //do while not reached the end
    begin
      Start:=P;                                 //set Start to begining of string
                                                //while char is not space,EOL,carry
                                                //or null, get next char
      while not (P^ in [#0, #10, #13, #32]) do Inc(P);
                                                //now we have a whole text word
      SetString(Sub, Start, P - Start);         //so we set Sub string to it
      If P^ in [#13,#32] then Sub:=Sub+' ';     //If now we have space or carry,
                                                //add aditional space at end
                                                //check wether it fits in actual line
//      If (FOmegaBitmapFont.TextWidth(Sub+LineText)+FBorder>FWidth) then
      If (TextWidth(Sub+LineText)+FBorder>FWidth) then
      begin                                     //if it does not, write all except Sub
        If LineText<>'' then                    //as far as we had some text already
        begin
          FLines.Add(LineText);                 //add the text we already had for line
          LineText:='';                         //and set the variable empty
          //but if we finished, or if we want to consider EOL and carry from orig file
          If (P^=#0) or ((P^ in [#10,#13]) and not FNoCarry) then
            FLines.Add(Sub)                     //add another line with the Sub string
          else                                  //else
            LineText:=Sub;                      //Set the next line with the Sub string
        end else                                //in case there was no text
        begin                                   //means that the word is larger than line
          FLines.Add(Sub);                      //so we'll have to write it
        end                                     //but it won't fit
      end else                                  //in case Sub+Linetext fit
      begin
        LineText:=LineText+Sub;                 //acumulate both in LineText
        //but if we finished, or if we want to consider EOL and carry from orig file
        If (P^=#0) or ((P^ in [#10,#13]) and not FNoCarry) then
        begin
          FLines.Add(LineText);                 //add the line
          LineText:='';                         //and set it empty
        end;
      end;
      while (P^ in [#10,#13,#32]) do Inc(P);    //skip spaces,EOL and carry chars
    end;
  If FAutoScroll then
    FLineIndex:=MaxLineIndex;
end;

procedure TOmegaText.Justify;
var
  Stream: TStringStream;
  i: Integer;
begin
  Stream:=TStringStream.Create('');
  For i:=0 to FLines.Count-1 do
  begin
    TStringStream(Stream).WriteString(FLines.Strings[i]);
  end;
  Stream.Position:=0;
  try
    LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

// keep in mind the possible Fontscaling
function TOmegaText.TextWidth(s: string):Integer;
begin
  Result:=Round(OmegaBitMapFont.TextWidth(s) * FFontScaleX);
end;

function TOmegaText.MaxLineIndex: Integer;
begin
  Result:= FLines.Count - (Height div Trunc(FomegaBitmapFont.CharHeight *
FOmegaBitmapFont.ScaleY));
end;

function TOmegaText.Add(Value: String): Integer;
begin
  Result:=FLines.Add(Value);
  If FAutoScroll then
    FLineIndex:=FLines.Count;
end;

procedure TOmegatext.Insert(Index:Integer; Value:String);
begin
  FLines.Insert(Index,Value);
end;

procedure TOmegaText.Delete(Value: Integer);
begin
  FLines.Delete(Value);
end;

function TOmegatext.IndexOf(Value:STring): Integer;
begin
  Result:=FLines.IndexOf(Value);
end;

function TOmegaText.GetLine(Index:Integer):String;
begin
  Result:=FLines.Strings[Index];
end;

procedure TOmegaText.SetLine(Index:Integer;Value:String);
begin
  FLines.Strings[Index]:=Value;
end;

Procedure TOmegaText.DrawText;
var i,j:Integer;
    TempX,TempY:Integer;
begin
  inherited;

  j := 0;
  For i := FLineIndex to FLines.Count - 1 do
  begin
    TempY := FY + Trunc(j * FOmegaBitmapFont.TextHeight(' ') * FFontScaleY);        //calculate actual line FY
        //exit if it's out
    If TempY + (Trunc(FOmegaBitmapFont.TextHeight(' ') * FFontScaleY) - 2) >
       Y + Height then Break;

    TempX := FX + FBorder;                          //add border to FX
//    FOmegaBitmapFont.Print(TempX,TempY,FLines[i]);         //and print the line
    FOmegaBitmapfont.Print(TempX, TempY, 0, FFontscaleX, FFontScaleY, FFontColor, FLines[i]);
    j := j + 1;
  end;
end;

procedure TOmegaText.Draw;
begin
  inherited;
  DrawText;
                                       //if it has the focus
  If (Image[3]<>nil) and               //as far as there is an image assigned
     ((Factive) or (FParent.FParent.FGUIStatus.FocusID=FID)) then
  begin
   Image[3].Draw(FX,FY,0,0.5,0.5,1,1,OmegaColor(Color[3].Red,
    Color[3].Green,Color[3].Blue,Color[3].Alpha),
    MirrorH, MirrorV,ImageIndex[3], BlendMode[3]);
  end;
  If Anim[3] then Animate(FImage[3]);       //check weather is animated
end;

procedure TOmegaText.SetLinePattern(Value: String);
begin
  Width:=FParent.FParent.FOmegaBitmapFont.TextWidth(Value)+FBorder;
end;

procedure TOmegaText.SetLineNumber(Value: Integer);
begin
  Height:=FParent.FParent.FOmegaBitmapFont.TextHeight(' ') * Value;
end;

procedure TOmegaText.SetLineIndex(Value: Integer);
begin
  If Value>FLines.Count then Exit;
  FLineIndex:=Value;
end;

function TOmegatext.GetLineCount:Integer;
begin
  Result:=FLines.Count;
end;

procedure TOmegaText.Clear;
begin
  FLines.Clear;
end;

////////////////////////////////////////////////////////////////////////
//                TOmegaList                                          //
////////////////////////////////////////////////////////////////////////

constructor TOmegaList.Create(AOwner: TOmegaGUIList);
begin
  inherited;
  FSelected:=-1;
  FPointedItem:=-1;
  FCanSelect:=True;
end;

destructor TOmegaList.Destroy;
begin
  inherited;
end;

procedure TOmegaList.DoClick;
var Temp: Integer;
begin
  If not FCanSelect then Exit;
  Temp:=FPointedItem+FLineIndex;
  If FPointedItem>-1 then
    If FSelected=Temp then
      FSelected:=-1
    else
      FSelected:=Temp;

  inherited;
  FParent.SetFocus(FID);

end;

procedure TOmegaList.DoExit;
begin
  inherited;
  FPointedItem:=-1;
end;

procedure TOmegaList.DoKeyDown(CharKey: Boolean; var Key: Word; Shift: 
TShiftState);
begin
  inherited;
  DoMouseOver;
end;

procedure TOmegaList.DoMouseOver;
begin
  inherited;
  If FParent.FParent.FGUIStatus.SelID=-2 then Exit;
//find pointed element
  FPointedItem:=((FParent.FParent.FGUIStatus.My-FY - FBorderY) div
                 FParent.FParent.FOmegaBitmapFont.TextHeight(' ')) + FLineIndex;
  If FLines.Count-1-FLineIndex< FPointedItem then FPointedItem:=-1;
end;

function TOmegaList.GetSelected(Index: Integer): Boolean;
begin
  Result:=False;
  If Index = FSelected then Result:=True;
end;

procedure TOmegaList.SetSelected(Index: Integer; Value: Boolean);
begin
  If Value then
    FSelected:=Index
  else
    If FSelected=Index then FSelected:=-1;
end;

procedure TOmegaList.Sort;
begin
  FLines.Sort;
end;

//We only have to override DrawText since Draw is the same as in OmegaText
Procedure TOmegaList.DrawText;
var i,j: Integer;
    TempX,TempY: Integer;
begin
  j := 0;
  For i := FLineIndex to FLines.Count - 1 do
  begin
    //calculate actual line FY
    TempY := FY + BorderY + Trunc(j * FOmegaBitmapFont.TextHeight(' ') *
                                  FFontScaleY);
    If TempY + (FOmegaBitmapFont.TextHeight(' ') * FFontScaleY - 2) > Y + Height then
      Break;      //exit if it's out
    //center the line in X axis
    TempX := FX  + FBorder + ((Width -
             Trunc(FOmegaBitmapFont.TextWidth(FLines[i]) * FFontScaleX)) shr 1);
    if ((i = FPointedItem) or (i + FLineIndex = FSelected)) and (Image[3]<> nil) then
    begin
      Image[3].Draw(FX + FBorder, TempY, 0, 0.5, 0.5, 1, 1, Color[3],
                    MirrorH, MirrorV, ImageIndex[3], BlendMode[3]);
    end;
    FOmegaBitmapFont.Print(TempX, TempY, 0, FFontScaleX, FFontscaleY,
                           OmegaColor(255, 255, 255, 255), FLines[i]);
    j := j + 1;
  end;
end;

//here we'll not use inherited procedure, cause when focused, only the selected
//list item will be highlited, not whole component
procedure TOmegaList.Draw;
var i,j: Integer;
    TempX,TempY: Integer;
begin
  Image[0].DrawInRect(Rect(0, 0, Width, Height), FX, FY,
                             Rotation, 0.5, 0.5, 1, 1,
                             Color[0].Red, Color[0].Green, Color[0].Blue, Color[0].Alpha,
                             MirrorH, MirrorV, ImageIndex[0], BlendMode[0]);
  DrawText;

{  //print all elements in the list
  j:=0;
  For i:=FLineIndex to FLines.Count-1 do
  begin
    TempY:=FY+BorderY+Trunc(j*FOmegaBitmapFont.TextHeight(' ')*FFontScaleY);        //calculate actual line FY
    If TempY+(FOmegaBitmapFont.TextHeight(' ')*FFontScaleY-2)>Y+Height then Break;
//exit if it's out
                                                //center the line in X axis
    TempX:=FX+((Width-Trunc(FOmegaBitmapFont.TextWidth(FLines[i])*FFontScaleX)) shr 1);
    FOmegaBitmapFont.Print(TempX,TempY,0,FFontScaleX, FFontscaleY,OmegaColor(255, 255, 255, 255),FLines[i]);
    j:=j+1;
  end;

  If (Image[3]<>nil) then
  //as far as there is an image assigned
  begin
    For i:=0 to FLines.Count-1 do
    begin
      If (i<>FPointedItem) and not(i+FLineIndex = FSelected) then Continue;
      TempX:=FX;
      TempY:=(FY+BorderY+(i*Trunc(FOmegaBitmapFont.TextHeight(' ')*FFontScaleY)))-(Image[3].TileHeight shr 2);
      Image[3].Draw(TempX,TempY,0,0.5,0.5,1,1,OmegaColor(Color[3].Red,
       Color[3].Green,Color[3].Blue,Color[3].Alpha),
       MirrorH, MirrorV,ImageIndex[3], BlendMode[3]);
    end;
  end;}
  If Anim[3] then Animate(FImage[3]);       //check weather is animated
end;

////////////////////////////////////////////////////////////////////////
//                TOmegaBar                                           //
////////////////////////////////////////////////////////////////////////
constructor TOmegaBar.Create(AOwner: TOmegaGUIList);
begin
  inherited;
  FKind:=bHorizontal;
  FPosition:=0;
  FMin:=0;
  FMax:=100;
  FShowPos:=True;
end;

procedure TOmegaBar.DoClick;
var r: Single;
    c: Single;
begin
  If not FCanBeClicked then Exit;
  If FKind=bHorizontal then
  begin
    r := FWidth / (FMax - FMin + 1);
    c := FParent.FParent.FGUIStatus.Mx-FX;
    FPosition := Round(FMin + c / r);
  end else
  begin
    r := FHeight / (FMax - FMin + 1);
    c := FParent.FParent.FGUIStatus.My-FY;
    FPosition := Trunc(FMax - c / r);
  end;
  If FPosition<FMin then FPosition:=FMin;
  If FPosition>FMax then FPosition:=FMax;
end;

procedure TOmegaBar.Draw;
var XLong, YLong: Integer;
begin
  inherited;

  If FPosition=0 then Exit;
  If FKind=bHorizontal then
  begin
    XLong:=(FWidth *(FPosition - FMin)) div (FMax-FMin);
    If Image[3]<>nil then
      Image[3].DrawRect(Rect(0,0,XLong,FHeight),FX,FY,0,0.5,0.5,1,1,
      Color[3].Red,Color[3].Green,Color[3].Blue,Color[3].Alpha, //ImageIndex[3]);
      // the values below are: mirrorv,mirrorh,imageindex,blendmode
      false, false, ImageIndex[3], bmNormal);

  end else                                      //if bVertical
  begin
    YLong:=FHeight-(FHeight *(FPosition - FMin)) div (FMax-FMin);
    If Image[3]<>nil then
     Image[3].DrawRect(Rect(0,YLong,FWidth,FHeight),FX,FY+YLong,0,0.5,0.5,1,1,
     Color[3].Red,Color[3].Green,Color[3].Blue,Color[3].Alpha,  //ImageIndex[3]);
      // the values below are: mirrorv,mirrorh,imageindex,blendmode
      false, false, ImageIndex[3], bmNormal);
  end;

  If FShowPos then
    FOmegaBitmapFont.Print(X+((Width-FOmegaBitmapFont.TextWidth('100')) shr 1),
    Y+((Height shr 1)-FOmegaBitmapFont.TextHeight(' ') shr 1),
    IntToStr(FPosition));
end;

////////////////////////////////////////////////////////////////////////
//                TOmegaScrollBar                                     //
////////////////////////////////////////////////////////////////////////
procedure TOmegaScrollBar.DoClick;
begin
  If FKind=bHorizontal then
  begin
    FPosition:=FMin+(FParent.FParent.FGUIStatus.Mx-FX) div
     (FWidth div (FMax-FMin+1));
  end else
  begin
    FPosition:=FMax-(FParent.FParent.FGUIStatus.My-FY) div
     (FHeight div (FMax-FMin+1));
  end;
  If FPosition<FMin then FPosition:=FMin;
  If FPosition>FMax then FPosition:=FMax;
//  inherited;
end;

procedure TOmegaScrollBar.Draw;
var
    Temp:Integer;
    TempB:Integer;
begin
  Image[0].DrawInRect(Rect(0, 0, Width, Height), FX, FY,
                             Rotation, 0.5, 0.5, 1, 1,
                             Color[0].Red, Color[0].Green, Color[0].Blue, Color[0].Alpha,
                             MirrorH, MirrorV, ImageIndex[0], BlendMode[0]);
 
  If FKind=bHorizontal then
  begin
    TempB:=FWidth div (1+FMax-FMin);
    Temp:=(FPosition-FMin)*TempB;
    If Image[3]<>nil then
      Image[3].DrawRect(Rect(0,0,TempB,FHeight),FX+Temp,FY,0,0.5,0.5,1,1,
        Color[3].Red,Color[3].Green,Color[3].Blue,Color[3].Alpha,
        // the values below are: mirrorv,mirrorh,imageindex,blendmode
        false, false, -1, bmNormal);
  end else                                      //if bVertical
  begin
    TempB:=FHeight div (1+FMax-FMin);
    Temp:=FHeight-(FPosition-FMin+1)*TempB;
    If Image[3]<>nil then
      Image[3].DrawRect(Rect(0,0,FWidth,TempB),FX,FY+Temp,0,0.5,0.5,1,1,
        Color[3].Red,Color[3].Green,Color[3].Blue,Color[3].Alpha,
        // the values below are: mirrorv,mirrorh,imageindex,blendmode
        false, false, -1, bmNormal);
  end;
end;

end.

