{
//****************************************************************
// OmegaPrimitives unit
// author : Tobias Fleischer
// contact: tobybear@web.de
// comments are welcome!
//****************************************************************

version history:

05/17/05: added new LineTo2Col method (Bazi)
05/11/05: added new Triangle3Col, Rectangle4Col, PolygonCol method
          and Ellipse overload method (Bazi)
03/16/04: added user texture support
12/03/03: added width and DrawStartEnd property to Arc function
11/06/03: extended Points() procedure for usage with TPoint lists
09/19/03: added Points() procedure
09/02/03: now uses the new renderer in OmegaScreen
08/31/03: added proper Init and UnInit functions and notification
08/20/03: many improvements and additions for various primitives
08/19/03: first release

"todo":
- add proper documentation
}
unit OmegaPrimitives;
interface
uses Windows, OmegaScreen, Classes, {$I UseD3D8.inc};

type
  TOmegaPrimitives = class(TComponent)
  private
   mx, my: single;
   FFilled: boolean;
   FTexture: IDirect3DTexture8;
   procedure SetOmegaScreen(const Value: TOmegaScreen);
  protected
    FOmegaScreen: TOmegaScreen;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    vertices: array[0..1000] of TOmegaVertex;
    property Texture: IDirect3DTexture8 read FTexture write FTexture;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Custom(v: array of TOmegaVertex; numofpoints: integer); overload;
    procedure Custom(v: array of TOmegaVertex); overload;
    procedure Polygon(v: array of TPoint; numofpoints: integer; col: TOmegaColor); overload;
    procedure Polygon(v: array of TPoint; col: TOmegaColor); overload;
    procedure PolygonCol(v: array of TPoint; numofpoints: integer; col: array of TOmegaColor); overload; // Bazi
    procedure PolygonCol(v: array of TPoint; col: array of TOmegaColor); overload; // Bazi
    procedure Point(x, y: single; col: TOmegaColor);
    procedure Points(v: array of TPoint; numofpoints: integer; col: TOmegaColor); overload;
    procedure Points(v: array of TOmegaVertex; numofpoints: integer); overload;
    procedure Points(v: array of TOmegaVertex); overload;
    procedure Circle(x, y, r: single; col: TOmegaColor); overload;
    procedure Circle(x, y, r: single; wid: integer; col: TOmegaColor); overload;
    procedure Ellipse(x, y, r1, r2: single; col: TOmegaColor); overload;
    procedure Ellipse(x, y, r1, r2: single; wid: integer; col: TOmegaColor); overload; // Bazi
    procedure Arc(x, y, r: single; start_rad, end_rad: single; col: TOmegaColor; DrawStartEnd: boolean=true);overload;
    procedure Arc(x, y, r: single; start_rad, end_rad: single; wid:integer; col: TOmegaColor; DrawStartEnd: boolean=true);overload;
    procedure Triangle(x1, y1, x2, y2, x3, y3: single; col: TOmegaColor);
    procedure Triangle3Col(x1, y1, x2, y2, x3, y3: single; col1, col2, col3: TOmegaColor); // Bazi
    procedure Rectangle(x, y, width, height: single; col: TOmegaColor);
    procedure Rectangle4Col(x, y, width, height: single; col1, col2, col3, col4: TOmegaColor); // Bazi
    procedure Quadrangle(x1, y1, x2, y2, x3, y3, x4, y4: single; col: TOmegaColor); overload;
    procedure Quadrangle4Col(x1, y1, x2, y2, x3, y3, x4, y4: single; col1, col2, col3, col4: TOmegaColor);
    procedure Line(x1, y1, x2, y2: single; col: TOmegaColor);
    procedure Line2Col(x1, y1, x2, y2: single; col1, col2: TOmegaColor);
    procedure LineTo(x, y: single; col: TOmegaColor);
    procedure LineTo2Col(x, y: single; col1, col2: TOmegaColor); // Bazi
    procedure MoveTo(x, y: single);
  published
    property Filled: boolean read FFilled write FFilled;
    property OmegaScreen: TOmegaScreen read FOmegaScreen write SetOmegaScreen;
  end;

implementation
uses dialogs, sysutils;

{ TOmegaPrimitives }
const irad = 1 / 360;
      twopi = 2 * 3.14159265358;

procedure TOmegaPrimitives.SetOmegaScreen(const Value: TOmegaScreen);
begin
 FOmegaScreen := Value;
end;

constructor TOmegaPrimitives.Create(AOwner: TComponent);
var i: integer;
begin
 inherited;
 for i := 0 to 500 do vertices[i].rhw := 1;
 vertices[0].tu := 0;
 vertices[0].tv := 0;
 vertices[1].tu := 1;
 vertices[1].tv := 0;
 vertices[2].tu := 1;
 vertices[2].tv := 1;
 vertices[3].tu := 0;
 vertices[3].tv := 1;
 FFilled := true;
 FTexture := nil;
end;

destructor TOmegaPrimitives.Destroy;
begin
 inherited;
end;

procedure TOmegaPrimitives.Line(x1, y1, x2, y2: single; col: TOmegaColor);
begin
 vertices[0].x := x1;
 vertices[0].y := y1;
 vertices[0].color := col;
 vertices[1].x := x2;
 vertices[1].y := y2;
 vertices[1].color := col;
 FOmegaScreen.Device.SetTexture(0, FTexture);
 FOmegaScreen.CopyVertices(@vertices, 2);  
 FOmegaScreen.Device.DrawPrimitive(D3DPT_LINELIST, 0, 1);
end;

procedure TOmegaPrimitives.LineTo(x, y: single; col: TOmegaColor);
begin
 Line(mx, my, x, y, col);
 mx := x;
 my := y;
end;

procedure TOmegaPrimitives.LineTo2Col(x, y: single; col1, col2: TOmegaColor);
begin
 Line2Col(mx, my, x, y, col1, col2);
 mx := x;
 my := y;
end;

procedure TOmegaPrimitives.MoveTo(x, y: single);
begin
 mx := x;
 my := y;
end;

procedure TOmegaPrimitives.Circle(x, y, r: single; col: TOmegaColor);
var max, i: integer;
    ic, iinc: single;
begin
 if r > 1000 then r := 1000;
 max := round(r);
 iinc := 1 / max;
 ic := 0;
 vertices[0].x := x;
 vertices[0].y := y;
 vertices[0].color := col;
 for i := 1 to max + 1 do
 begin
  vertices[i].x := x + r * cos(ic * twopi);
  vertices[i].y := y + r * sin(ic * twopi);
  vertices[i].color := col;
  ic := ic + iinc;
 end;
 FOmegaScreen.Device.SetTexture(0, FTexture);

 if not FFilled then
 begin
  vertices[0].x := vertices[max + 1].x;
  vertices[0].y := vertices[max + 1].y;

  FOmegaScreen.CopyVertices(@vertices, max + 2);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_LINESTRIP, 0, max + 1);
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_LINESTRIP, max + 1, @vertices, sizeof(TOmegaVertex));
 end else
 begin
  FOmegaScreen.CopyVertices(@vertices, max + 2);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_TRIANGLEFAN, 0, max);
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_TRIANGLEFAN, max, @vertices, sizeof(TOmegaVertex));
 end;
end;

procedure TOmegaPrimitives.Arc(x, y, r, start_rad, end_rad: single;
  col: TOmegaColor; DrawStartEnd: boolean=true);
var max, i: integer;
    ic, iinc: single;
begin
 if r > 1000 then r := 1000;
 max := round(r);
 iinc := 1 / max;
 iinc := iinc * (end_rad - start_rad) * irad;
 ic := start_rad * irad;

 vertices[0].x := x;
 vertices[0].y := y;
 vertices[0].color := col;
 for i := 1 to max + 1 do
 begin
  vertices[i].x := x + r * cos(ic * twopi);
  vertices[i].y := y + r * sin(ic * twopi);
  vertices[i].color := col;
  ic := ic + iinc;
 end;
 FOmegaScreen.Device.SetTexture(0, FTexture);
 if DrawStartEnd then i := 0 else i := 1;
 if not FFilled then
 begin
  vertices[0].x := vertices[max + 1].x;
  vertices[0].y := vertices[max + 1].y;

  FOmegaScreen.CopyVertices(@vertices, max + 2);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_LINESTRIP, i, max + (1 - i));
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_LINESTRIP, max + 1, @vertices, sizeof(TOmegaVertex));
 end else
 begin
  FOmegaScreen.CopyVertices(@vertices, max + 2);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_TRIANGLEFAN, 0, max);
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_TRIANGLEFAN, max, @vertices, sizeof(TOmegaVertex));
 end;
end;

procedure TOmegaPrimitives.Arc(x, y, r, start_rad, end_rad: single;
  wid: integer; col: TOmegaColor; DrawStartEnd: boolean=true);
var i: integer;
begin
 if FFilled then Arc(x, y, r + wid, start_rad, end_rad, col, DrawStartEnd) else
  for i := 0 to 4 * (wid - 1) do
   Arc(x, y, r + i * 0.15, start_rad, end_rad, col, DrawStartEnd);
end;

procedure TOmegaPrimitives.Ellipse(x, y, r1, r2: single; col: TOmegaColor);
var max, i: integer;
    ic, iinc: single;
begin
 if r1 > 1000 then r1 := 1000;
 max := round(r1);
 iinc := 1 / max;
 ic := 0;
 vertices[0].x := x;
 vertices[0].y := y;
 vertices[0].color := col;
 for i := 1 to max + 1 do
 begin
  vertices[i].x := x + r1 * cos(ic * twopi);
  vertices[i].y := y + r2 * sin(ic * twopi);
  vertices[i].color := col;
  ic := ic + iinc;
 end;
 FOmegaScreen.Device.SetTexture(0, FTexture);

 if not FFilled then
 begin
  vertices[0].x := vertices[max + 1].x;
  vertices[0].y := vertices[max + 1].y;
  FOmegaScreen.CopyVertices(@vertices, max + 2);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_LINESTRIP, 0, max + 1);
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_LINESTRIP, max + 1, @vertices, sizeof(TOmegaVertex));
 end else
 begin
  FOmegaScreen.CopyVertices(@vertices, max + 2);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_TRIANGLEFAN, 0, max);
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_TRIANGLEFAN, max, @vertices, sizeof(TOmegaVertex));
 end;
end;

procedure TOmegaPrimitives.Triangle3Col(x1, y1, x2, y2, x3, y3: single;
  col1, col2, col3: TOmegaColor);
begin
 vertices[0].x := x1;
 vertices[0].y := y1;
 vertices[0].color := col1;
 vertices[1].x := x2;
 vertices[1].y := y2;
 vertices[1].color := col2;
 vertices[2].x := x3;
 vertices[2].y := y3;
 vertices[2].color := col3;
 FOmegaScreen.Device.SetTexture(0, FTexture);
 if FFilled then
 begin
  FOmegaScreen.CopyVertices(@vertices, 3);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_TRIANGLELIST, 0, 1);
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_TRIANGLELIST, 1, @vertices, sizeof(TOmegaVertex));
 end else
 begin
  vertices[3].x := x1;
  vertices[3].y := y1;
  vertices[3].color := col1;
  FOmegaScreen.CopyVertices(@vertices, 4);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_LINESTRIP, 0, 3);
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_LINESTRIP, 3, @vertices, sizeof(TOmegaVertex));
 end;
end;

procedure TOmegaPrimitives.Triangle(x1, y1, x2, y2, x3, y3: single;
  col: TOmegaColor);
begin
  Triangle3Col(x1,y1,x2,y2,x3,y3,col,col,col);
end;

procedure TOmegaPrimitives.Point(x, y: single; col: TOmegaColor);
begin
 vertices[0].x := x;
 vertices[0].y := y;
 vertices[0].color := col;
 FOmegaScreen.Device.SetTexture(0, FTexture);
 FOmegaScreen.CopyVertices(@vertices, 1);
 FOmegaScreen.Device.DrawPrimitive(D3DPT_POINTLIST, 0, 1);
end;

procedure TOmegaPrimitives.Quadrangle4Col(x1, y1, x2, y2, x3, y3, x4,
  y4: single; col1, col2, col3, col4: TOmegaColor);
begin
 vertices[0].x := x1;
 vertices[0].y := y1;
 vertices[0].color := col1;
 vertices[1].x := x2;
 vertices[1].y := y2;
 vertices[1].color := col2;
 vertices[2].x := x3;
 vertices[2].y := y3;
 vertices[2].color := col3;
 vertices[3].x := x4;
 vertices[3].y := y4;
 vertices[3].color := col4;

 FOmegaScreen.Device.SetTexture(0, FTexture);
 if FFilled then
 begin
  FOmegaScreen.CopyVertices(@vertices, 4);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 2, @vertices, sizeof(TOmegaVertex));
 end else
 begin
  vertices[4].x := x1;
  vertices[4].y := y1;
  vertices[4].color := col1;
  FOmegaScreen.CopyVertices(@vertices, 5);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_LINESTRIP, 0, 4);
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_LINESTRIP, 4, @vertices, sizeof(TOmegaVertex));
 end;
end;

procedure TOmegaPrimitives.Quadrangle(x1, y1, x2, y2, x3, y3, x4,
  y4: single; col: TOmegaColor);
begin
 Quadrangle4Col(x1, y1, x2, y2, x3, y3, x4, y4, col, col, col, col);
end;

procedure TOmegaPrimitives.Polygon(v: array of TPoint;
  numofpoints: integer; col: TOmegaColor);
var i: integer;
begin
 for i := 0 to numofpoints - 1 do
 begin
  vertices[i].x := v[i].x;
  vertices[i].y := v[i].y;
  vertices[i].color := col;
 end;
 FOmegaScreen.Device.SetTexture(0, FTexture);
 if FFilled then
 begin
  FOmegaScreen.CopyVertices(@vertices, numofpoints);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_TRIANGLEFAN, 0, numofpoints - 2);
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_TRIANGLEFAN, numofpoints - 2, @vertices, sizeof(TOmegaVertex));
 end else
 begin
  vertices[numofpoints].x := v[0].x;
  vertices[numofpoints].y := v[0].y;
  vertices[numofpoints].color := col;
  FOmegaScreen.CopyVertices(@vertices, numofpoints + 1);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_LINESTRIP, 0, numofpoints);
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_LINESTRIP, numofpoints, @vertices, sizeof(TOmegaVertex));
 end;
end;

procedure TOmegaPrimitives.Polygon(v: array of TPoint; col: TOmegaColor);
begin
 Polygon(v, high(v) + 1, col);
end;

procedure TOmegaPrimitives.PolygonCol(v: array of TPoint;
  numofpoints: integer; col: array of TOmegaColor);
var i: integer;
begin
 for i := 0 to numofpoints - 1 do
 begin
  vertices[i].x := v[i].x;
  vertices[i].y := v[i].y;
  vertices[i].color := col[i];
 end;
 FOmegaScreen.Device.SetTexture(0, FTexture);
 if FFilled then
 begin
  FOmegaScreen.CopyVertices(@vertices, numofpoints);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_TRIANGLEFAN, 0, numofpoints - 2);
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_TRIANGLEFAN, numofpoints - 2, @vertices, sizeof(TOmegaVertex));
 end else
 begin
  vertices[numofpoints].x := v[0].x;
  vertices[numofpoints].y := v[0].y;
  vertices[numofpoints].color := col[0];
  FOmegaScreen.CopyVertices(@vertices, numofpoints + 1);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_LINESTRIP, 0, numofpoints);
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_LINESTRIP, numofpoints, @vertices, sizeof(TOmegaVertex));
 end;
end;

procedure TOmegaPrimitives.PolygonCol(v: array of TPoint; col: array of TOmegaColor);
begin
 PolygonCol(v, high(v) + 1, col);
end;

procedure TOmegaPrimitives.Custom(v: array of TOmegaVertex;
  numofpoints: integer);
begin
 FOmegaScreen.Device.SetTexture(0, FTexture);
 if FFilled then
 begin
  FOmegaScreen.CopyVertices(@vertices, numofpoints);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_TRIANGLEFAN, 0, numofpoints - 2);
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_TRIANGLEFAN, numofpoints - 2, @v, sizeof(TOmegaVertex));
 end else
 begin
  FOmegaScreen.CopyVertices(@vertices, numofpoints);
  FOmegaScreen.Device.DrawPrimitive(D3DPT_LINESTRIP, 0, numofpoints - 1);
//  FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_LINESTRIP, numofpoints - 1, @v, sizeof(TOmegaVertex));
 end;
end;

procedure TOmegaPrimitives.Line2Col(x1, y1, x2, y2: single; col1,
  col2: TOmegaColor);
begin
 vertices[0].x := x1;
 vertices[0].y := y1;
 vertices[0].color := col1;
 vertices[1].x := x2;
 vertices[1].y := y2;
 vertices[1].color := col2;
 FOmegaScreen.Device.SetTexture(0, FTexture);

 FOmegaScreen.CopyVertices(@vertices, 2);
 FOmegaScreen.Device.DrawPrimitive(D3DPT_LINELIST, 0, 1);
// FOmegaScreen.Device.DrawPrimitiveUP(D3DPT_LINELIST, 1, @vertices, sizeof(TOmegaVertex));
end;

procedure TOmegaPrimitives.Circle(x, y, r: single; wid: integer;
  col: TOmegaColor);
var i: integer;
begin
 if FFilled then Circle(x, y, r + wid, col) else
  for i := 0 to 3 * (wid - 1) do Circle(x, y, r + i * 0.3, col);
end;

procedure TOmegaPrimitives.Ellipse(x, y, r1, r2: single; wid: integer; col: TOmegaColor);
var i: integer;
begin
 if FFilled then Ellipse(x, y, r1 + wid, r2 + wid, col) else
  for i := 0 to 3 * (wid - 1) do Ellipse(x, y, r1 + i * 0.3, r2 + i * 0.3, col);
end;

procedure TOmegaPrimitives.Rectangle4Col(x, y, width, height: single;
  col1, col2, col3, col4: TOmegaColor);
begin
 Quadrangle4Col(x, y, x + width, y, x + width, y + height,
  x, y + height, col1, col2, col3, col4);
end;

procedure TOmegaPrimitives.Rectangle(x, y, width, height: single;
  col: TOmegaColor);
begin
 Quadrangle4Col(x, y, x + width, y, x + width, y + height,
  x, y + height, col, col, col, col);
end;

procedure TOmegaPrimitives.Custom(v: array of TOmegaVertex);
begin
 Custom(v, high(v) + 1);
end;

procedure TOmegaPrimitives.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
 inherited;
 case Operation of
  opInsert: if (AComponent is TOmegaScreen)and(not Assigned(OmegaScreen)) then
   SetOmegaScreen(TOmegaScreen(AComponent));
  opRemove: if (AComponent = OmegaScreen) then OmegaScreen:= nil;
 end;
end;

procedure TOmegaPrimitives.Points(v: array of TOmegaVertex;
  numofpoints: integer);
begin
 FOmegaScreen.Device.SetTexture(0, FTexture);
 FOmegaScreen.CopyVertices(@v, numofpoints);
 FOmegaScreen.Device.DrawPrimitive(D3DPT_POINTLIST, 0, numofpoints - 1);
end;

procedure TOmegaPrimitives.Points(v: array of TOmegaVertex);
begin
 Points(v, high(v) + 1);
end;

procedure TOmegaPrimitives.Points(v: array of TPoint; numofpoints: integer;
  col: TOmegaColor);
var i: integer;
begin
 for i := 0 to numofpoints - 1 do
 begin
  vertices[i].x := v[i].x;
  vertices[i].y := v[i].y;
  vertices[i].Color := col;
 end;
 Points(vertices, numofpoints);
end;

end.

