unit OmegaRegister;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
{$IFDEF VER130}DsgnIntf, {$ELSE}DesignIntf, DesignEditors, {$ENDIF}
  OmegaScreen, OmegaSprite, OmegaImageList, OmegaFont, OmegaBitmapFont,
  OmegaSound3d, OmegaMusic, OmegaSurface, OmegaTimer, OmegaGUI, OmegaInput,
  OmegaMap, OmegaParticleEngine, OmegaPrimitives, OmegaFader, OmegaMultimedia,
  OmegaAnimationList, OmegaFlexiGrid, OmegaVideo;

type
  TImageListItem_PictureProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TOmegaImageListEditor = class(TComponentEditor)
  public
    procedure ExecuteVerb(index: integer); override;
    function GetVerb(index: integer): string; override;
    function GetVerbCount: integer; override;
  end;

  TOmegaInputEditor = class(TComponentEditor)
  public
    Procedure Edit; override;
  end;

procedure Register;

implementation

uses PictureEdit, OmegaInputEdit;

{$R Icons.res}

procedure Register;
begin
  RegisterPropertyEditor(TypeInfo(TPicture), TImageListItem, 'Picture', TImageListItem_PictureProperty);
  RegisterComponentEditor(TOmegaImageList, TOmegaImageListEditor);
  RegisterComponentEditor(TOmegaInput, TOmegaInputEditor);
  RegisterComponents('Omega', [TOmegaScreen, TOmegaSurface, TOmegaSprite, TOmegaImageList, TOmegaFont, 
  TOmegaBitmapFont, TOmegaSound3d, TOmegaMusic, TOmegaTimer, TOmegaGUI, TOmegaInput, TOmegaMap, 
  TOmegaParticleEngine, TOmegaPrimitives, TOmegaFader, TOmegaMultimedia, TOmegaAnimationList,
  TOmegaFlexiGrid, TOmegaVideo]); //register Omega stuff.
end;

procedure TImageListItem_PictureProperty.Edit;
var
  Form: TPictureEditForm;
begin
  Form := TPictureEditForm.Create(nil);
  try
    Form.ImagePreview.Picture := TPicture(GetOrdValue);
    if Form.ShowModal = mrOK then
    begin
      SetOrdValue(Integer(Form.ImagePreview.Picture));
      Designer.Modified;
    end;
  finally
    Form.Free;
  end;
end;

function TImageListItem_PictureProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TImageListItem_PictureProperty.GetValue: string;
begin
  if (TPicture(GetOrdValue).Graphic = nil) or (TPicture(GetOrdValue).Graphic.Empty) then
    Result := '(None)'
  else
    Result := Format('(%s)', [TPicture(GetOrdValue).Graphic.ClassName]);
end;

procedure TOmegaImageListEditor.ExecuteVerb(index: integer);
var
  OpenDialog: TOpenDialog;
  SaveDialog: TSaveDialog;
begin
  case index of
    0: begin
        OpenDialog := TOpenDialog.Create(nil);
        OpenDialog.DefaultExt := 'oil';
        OpenDialog.Filter := 'Omega ImageList Files (*.oil)|*.oil';
        OpenDialog.Options := [ofPathMustExist, ofFileMustExist];
        if OpenDialog.Execute then
        begin
          TOmegaImageList(Component).ImageList.LoadFromFile(OpenDialog.FileName);
          Designer.Modified;
        end;
        OpenDialog.Free;
      end;
    1: begin
        SaveDialog := TSaveDialog.Create(nil);
        SaveDialog.Filter := 'Omega ImageList Files (*.oil)|*.oil';
        SaveDialog.DefaultExt := 'oil';
        SaveDialog.Options := [ofOverwritePrompt, ofPathMustExist];
        try
          if SaveDialog.Execute then
            TOmegaImageList(Component).ImageList.SaveToFile(SaveDialog.FileName);
        finally
          SaveDialog.Free;
        end;
      end;
  end;
end;

function TOmegaImageListEditor.GetVerb(Index: integer): string;
begin
  case Index of
    0: Result := 'Open';
    1: Result := 'Save';
  end;
end;

function TOmegaImageListEditor.GetVerbCount: integer;
begin
  Result := 2;
end;

Procedure TOmegaInputEditor.Edit;
var
  Form: TOmegaInputEditForm;
begin
  Form := TOmegaInputEditForm.Create(nil);
  try
    Form.OmegaInput := TOmegaInput(Component);
    Form.ShowModal;
    if Form.Tag > 0 then
      Designer.Modified;
  finally
    Form.Free;
  end;
end;

end.
