{
//****************************************************************
// OmegaScreen unit
// author: Tobias Fleischer
// contact: tobybear@web.de
// comments are welcome!
//****************************************************************

version history:
02/09/05: added Frequency published property (Bazi)
07/25/04: added BlendModeString function
          added DecimalSeparator default setting
07/21/04: added new LineH, LineV functions
          fixed DrawRectangle function
03/10/04: adapted whole unit to latest DX8/DX9 headers
03/09/04: added InvColor() function
01/27/04: blendmode switching is now faster
          added Grayscale property
01/26/04: increased vertex buffer size, improved viewports
01/25/04: brandnew features for OmegaScreen!
          new properties: Red, Green, Blue, Alpha, Color,
                          ScaleX, ScaleY, CenterX, CenterY,
                          Rotation, OffsetX, OffsetY
          they allow some excellent new graphic effects
01/24/04: fixed bug concerning hWnd
01/17/04: DrawRectangle now has optional alpha parameter
01/14/04: added SetPixel function
11/25/03: several overall improvements and fixes
11/06/03: added fix for screen resize,
          BlendMode, Wireframe, Dithering and AntiAliasing are
          now read/write properties
11/04/03: added fix for visibility bug for two forms
10/27/03: added changes to compensate for some ATI RADEON issues
          re-wrote vertex buffer parts
10/10/03: experimental re-write, might still cause problems!
          added support for OmegaFlexiGrid
09/23/03: added bmInvert
09/19/03: some fixes here and there, added wireframe mode and gamma
09/02/03: added new renderer, new notification
08/31/03: total rewrite

"todo":
- finally find a cure for those ATI issues!!!
- improve vertex buffering
- add proper documentation
}

unit OmegaScreen;

interface

{$DEFINE DIRECTX8}
uses Windows, Classes, Messages, Forms, Graphics, {$I UseD3D8.inc};

const OmegaVertexDef = D3DFVF_XYZRHW or D3DFVF_DIFFUSE or D3DFVF_TEX1;
      maxVertices = 32768;

type

 TOmegaColor = packed record
  Blue: Byte;
  Green: Byte;
  Red: Byte;
  Alpha: Byte;
 end;

 TOmegaVertex = packed record
  x, y, z: single;     // Position
  rhw: single;         // Reciprocal of homogeneous w
  Color: TOmegaColor;  // Vertex Color
  tu, tv: single;      // Texture coordinates
 end;
 POmegaVertex = ^TOmegaVertex;

type
 TRenderMode = (rmHardware, rmReference, rmSoftware);
 TPoolType = (plManaged, plDefault);

// the constants for the blend modes:
const   bmIgnore = -1;
        bmNormal = 0;
        bmAdd = 1;
        bmSrcAdd = 2;
        bmSrcCol = 3;
        bmSrcColAdd = 4;
        bmMultiply = 5;
        bmMultiplyAlpha = 6;
        bmInvMultiply = 7;
        bmInvMultiplyAlpha = 8;
        bmInvert = 9;
        bmSrcBright = 10;
        bmDestBright = 11;
        bmInvSrcBright = 12;
        bmInvDestBright = 13;

  MSG_NONE = 0;
  MSGERR_APPMUSTEXIT = 1;
  MSGWARN_SWITCHEDTOREF = 2;

  D3DAPPERR_NODIRECT3D = $82000001;
  D3DAPPERR_NOWINDOW = $82000002;
  D3DAPPERR_NOCOMPATIBLEDEVICES = $8200003;
  D3DAPPERR_NOWINDOWABLEDEVICES = $8200004;
  D3DAPPERR_NOHARDWAREDEVICE = $82000005;
  D3DAPPERR_HALNOTCOMPATIBLE = $82000006;
  D3DAPPERR_NOWINDOWEDHAL = $82000007;
  D3DAPPERR_NODESKTOPHAL = $82000008;
  D3DAPPERR_NOHALTHISMODE = $82000009;
  D3DAPPERR_NONZEROREFCOUNT = $8200000A;
  D3DAPPERR_RESIZEFAILED = $8200000C;

type qsortcb = function(const arg1: pointer; const arg2: pointer): integer; cdecl;
     size_t = Cardinal;

  // Structure for holding information about a display mode
  PD3DModeInfo = ^TD3DModeInfo;
  TD3DModeInfo = packed record
    Width: DWORD; // Screen width in this mode
    Height: DWORD; // Screen height in this mode
    Format: TD3DFORMAT; // Pixel format in this mode
    Behavior: DWORD; // Hardware / Software / Mixed vertex processing
    DepthStencilFormat: TD3DFORMAT; // Which depth/stencil format to use with this mode
  end;

  // Structure for holding information about a Direct3D device, including
  // a list of modes compatible with this device
  FDeviceInfo = ^TD3DDeviceInfo;
  TD3DDeviceInfo = packed record
    // Device data
    DeviceType: TD3DDEVTYPE; // Reference, HAL, etc.
    Caps: TD3DCaps8; // Capabilities of this device
    strDesc: string; // Name of this device
    canDoWindowed: Boolean; // Whether this device can work in windowed mode
    // Modes for this device
    NumModes: DWORD;
    Modes: array[0..149] of TD3DModeInfo;
    // Current state
    CurrentMode: DWORD;
    isWindowed: BOOL;
    MultiSampleType: TD3DMULTISAMPLE_TYPE;
  end;

  // Structure for holding information about an adapter, including a list
  // of devices available on this adapter
  PD3DAdapterInfo = ^TD3DAdapterInfo;
  TD3DAdapterInfo = packed record
    // Adapter data
    AdapterIdentifier: TD3DADAPTER_IDENTIFIER8;
    Desktop: TD3DDISPLAYMODE; // Desktop display mode for this adapter
    // Devices for this adapter
    NumDevices: DWORD;
    Devices: array[0..4] of TD3DDeviceInfo;
    // Current state
    CurrentDevice: DWORD;
  end;

  TOmegaScreen = class(TComponent)
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  private
    oldVP: TRect;
    FWireframe, FGrayscale,
    FAntiAliasing,
    FDithering: boolean;
    FOnCreate: TNotifyEvent;
    FOnDestroy: TNotifyEvent;
    FOnInit: TNotifyEvent;
    FOnUnInit: TNotifyEvent;
    VB: IDirect3DVertexBuffer8;
    FGammaRamp: TD3DGammaRamp;
    FRenderMode: TRenderMode;
    FRenderStates: array[0..20] of cardinal;
    FBlendMode: integer;
    FWidth: LongInt;
    FHeight: LongInt;
    FBPP: LongInt;
    FOwner: TComponent;
    FTakeScreenShot: boolean;
    FScreenShotFile: string;
    FBBAccess: boolean;
    FFullScreen: boolean;
    FFrequency: Cardinal;       // Bazi
    FShowMouseCursor: boolean;
    FBBCount: Longint;
    FZBuffer: boolean; // Whether to autocreate depthbuffer
    FVSYNC: boolean;
    FWindowReady: boolean;
    FDXReady: boolean;
    FPool: TPoolType;
    FhWnd, FhWndSave: HWND; // The main app window
    FhWndFocus: HWND; // The D3D focus window (usually same as hWnd)
    FDeviceListBuilt: boolean;
    FAlpha: byte;
    FRed: byte;
    FGreen: byte;
    FBlue: byte;
    FScaleX: single;
    FScaleY: single;
    FRotation: single;
    FOffsetX: integer;
    FOffsetY: integer;
    FCenterX: integer;
    FCenterY: integer;
    FColor: TOmegaColor;

    function DisplayErrorMsg(hr: int64; dwType: dword):Hresult;
    procedure TakeScreenShot(FileName: string);
    function GetBackBuffer(n: integer): IDirect3DSurface8;
    procedure SetBPP(const Value: LongInt);
    procedure SubInit;
    procedure SubUnInit;
    procedure SetAlpha(const Value: byte);
    procedure SetBlue(const Value: byte);
    procedure SetColor(const Value: TOmegaColor);
    procedure SetGreen(const Value: byte);
    procedure SetRed(const Value: byte);
    procedure SetFrequency(const Value: Cardinal);       // Bazi
  public
    // Internal variables
    Rendering: boolean;
    Adapters: array[0..9] of TD3DAdapterInfo;
    CurrentAdapter: integer;
    NumAdapters: DWORD;
    // Internal error handling function

    // Main objects used for creating and rendering the 3D scene
    FD3DDEVCAPS8: TD3DCAPS8; // Caps for the device
    FDevice: IDIRECT3DDEVICE8; // The D3D rendering device
    FD3D8: IDIRECT3D8; // The main D3D object
    FD3DDEVPPAR8: TD3DPRESENT_PARAMETERS; // Parameters for CreateDevice/Reset
    BackBufferDesc: TD3DSURFACE_DESC; // Surface desc of the backbuffer
    dwCreateFlags: DWORD; // Indicate sw or hw vertex processing
    dwWindowStyle: DWORD; // Saved window style for mode switches
    rcWindowBounds: TRECT; // Saved window bounds for mode switches
    strDeviceStats: string;
    dwMinDepthBits: DWORD; // Minimum number of bits needed in depth buffer
    dwMinStencilBits: DWORD; // Minimum number of bits needed in stencil buffer

    // Internal functions to manage and render the 3D scene
    procedure CaptureRenderStates;
    procedure RestoreRenderStates;
    procedure ResetRenderStates;
    function BuildDeviceList: HRESULT;
    function Initialize: HRESULT;
    function ReInit: HRESULT;
    function ToggleFullscreen: HRESULT;
    procedure Cleanup;
    procedure SetHWnd(value: hWnd);
    function FindDepthStencilFormat(iAdapter: UINT;
      DeviceType: TD3DDEVTYPE; TargetFormat: TD3DFORMAT;
      var pDepthStencilFormat: TD3DFORMAT): BOOL;
    procedure SetTarget;
    function Init: hresult;
    procedure ScreenShot(FileName: string);
    property Direct3D8: IDirect3D8 read FD3D8 write FD3D8;
    property ThehWnd: hWnd read FhWnd write SethWnd;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DrawRectangle(x1, y1, x2, y2: integer; Solid: boolean;
     BorderWidth: integer; Red, Green, Blue: integer);
    procedure LineV(x, y1, y2: integer; Red, Green, Blue: byte; width: integer = 1);
    procedure LineH(x1, x2, y: integer; Red, Green, Blue: byte; width: integer = 1);
    procedure SetPixel(x, y: integer; Red, Green, Blue: integer; Alpha: integer = 255);
    procedure ClearScreen(r, g, b: Byte; a: Byte=255);
    procedure BeginRender;
    procedure SetAntiAliasing(activate: boolean);
    procedure SetAlphaBlending(activate: boolean);
    procedure SetBlendMode(mode: integer);
    procedure SetDithering(activate: boolean);
    procedure SetWireframe(activate: boolean);
    procedure SetGrayscale(activate: boolean);
    function FindMode(w, h, bpp: DWORD; rm: TRenderMode): boolean;
    function CanDraw: boolean;
    property WindowReady: boolean read FWindowReady write FWindowReady;
    property DXReady: boolean read FDXReady write FDXReady;
    procedure EndRender;
    procedure ResetViewPort;
    procedure SetPreviousViewPort;
    procedure SetGamma(Red, Green, Blue, Brightness, Contrast: byte);
    function GetViewPort: TRect;
    procedure SetViewPort(Rect: TRect); overload;
    procedure SetViewPort(x1, y1, x2, y2: Cardinal); overload;
    property BackBuffer[n: integer]: IDirect3DSurface8 read GetBackBuffer;
    procedure CopyVertices(pVertices: PByte; numVertices: integer);
    property BlendMode: integer read FBlendMode write SetBlendMode;
    property Color: TOmegaColor read FColor write SetColor;
    property Dithering: boolean read FDithering write SetDithering;
    property Wireframe: boolean read FWireframe write SetWireframe;
    property Grayscale: boolean read FGrayscale write SetGrayscale;
    property AntiAliasing: boolean read FAntiAliasing write SetAntiAliasing;
   published
    property Pool: TPoolType read FPool write FPool default plDefault;
    property ScaleX: single read FScaleX write FScaleX;
    property ScaleY: single read FScaleY write FScaleY;
    property CenterX: integer read FCenterX write FCenterX;
    property CenterY: integer read FCenterY write FCenterY;
    property OffsetX: integer read FOffsetX write FOffsetX;
    property OffsetY: integer read FOffsetY write FOffsetY;
    property Rotation: single read FRotation write FRotation;
    property Red: byte read FRed write SetRed;
    property Green: byte read FGreen write SetGreen;
    property Blue: byte read FBlue write SetBlue;
    property Alpha: byte read FAlpha write SetAlpha;
    property ShowMouseCursor: boolean read FShowMouseCursor write FShowMouseCursor default true;
    property RenderMode: TRenderMode read FRenderMode write FRenderMode default rmHardware;
    property Width: LongInt read FWidth write FWidth;
    property Height: LongInt read FHeight write FHeight;
    property bpp: LongInt read FBPP write SetBPP default 16;
    property ZBuffer: boolean read FZBuffer write FZBuffer default true;
    property FullScreen: boolean read FFullScreen write FFullScreen default false;
    property Frequency: Cardinal read FFrequency write SetFrequency default D3DPRESENT_RATE_DEFAULT; // Bazi
    property VSYNC: boolean read FVSYNC write FVSYNC default false;
    property BackBufferAccess: boolean read FBBAccess write FBBAccess default true;
    property BBCount: Longint read FBBCount write FBBCount default 1;
    property Device: IDirect3DDevice8 read FDevice write FDevice;
    property OnCreate: TNotifyEvent read FOnCreate write FOnCreate;
    property OnDestroy: TNotifyEvent read FOnDestroy write FOnDestroy;
    property OnInit: TNotifyEvent read FOnInit write FOnInit;
    property OnUnInit: TNotifyEvent read FOnUnInit write FOnUnInit;
  end;

{ Quick functions for creating a TOmegaColor record }
function InvColor(c: TOmegaColor): TOmegaColor;
function OmegaColor(Red, Green, Blue: Byte; Alpha: Byte = 255): TOmegaColor; overload;
function OmegaColor(Color: TColor; Alpha: Byte): TOmegaColor; overload;
function OmegaColor(Color: longword): TOmegaColor; overload;

procedure qsort(base: pointer; num: size_t; width: size_t; compare: qsortcb); cdecl; external 'MSVCRT.DLL';

function OmegaRed: TOmegaColor;
function OmegaGreen: TOmegaColor;
function OmegaBlue: TOmegaColor;
function OmegaBlack: TOmegaColor;
function OmegaWhite: TOmegaColor;
function OmegaYellow: TOmegaColor;
function OmegaMagenta: TOmegaColor;
function OmegaCyan: TOmegaColor;
function OmegaGray: TOmegaColor;

function BlendModeString(bm: integer): string;

implementation
uses dialogs, sysutils, d3dx8, OmegaImageList, OmegaBitmapFont, OmegaVideo,
 OmegaFader, OmegaPrimitives, OmegaSurface, math, OmegaInput, OmegaGUI,
 OmegaFont, OmegaFlexiGrid;

procedure TOmegaScreen.SethWnd(value: hWnd);
begin
 if self.FhWnd <> 0 then FhWndSave := FhWnd;
 FhWnd := value;
 FhWndFocus := 0;
end;

function TOmegaScreen.FindMode(w, h, bpp: DWORD; rm: TRenderMode): boolean;
var t2, i, j, k: DWORD;
begin
 if FDeviceListBuilt = false then BuildDeviceList;
 Result := false;
 for i := 0 to NumAdapters - 1 do
 for j := 0 to Adapters[i].NumDevices - 1 do
 begin
 for k := 0 to Adapters[i].Devices[j].NumModes - 1 do
 with Adapters[i].Devices[j].Modes[k] do
 begin
  t2 := 16;
  if (Format = D3DFMT_X8R8G8B8) or
     (Format = D3DFMT_A8R8G8B8) or
     (Format = D3DFMT_R8G8B8) then t2 := 32;
  if (Width = w) and (Height = h) and (t2 = bpp) then
   begin
    CurrentAdapter := i;
    Adapters[i].CurrentDevice := j;
    Adapters[i].Devices[j].CurrentMode := k;
    Result := true;
    exit;
   end;
 end;
 end;
end;

constructor TOmegaScreen.Create(AOwner: TComponent);
//var hr: HRESULT;
var i: integer;
begin
 decimalseparator := '.';
 inherited Create(AOwner);
 FScaleX:=1;
 FScaleY:=1;
 FOffsetX:=0;
 FOffsetY:=0;
 FRotation:=0;
 SetColor(OmegaColor($FFFFFFFF));
 FRenderMode := rmHardware;
 FWireframe := false;
 FGrayscale := false;
 FAntiAliasing := false;
 FDithering := false;
 FDeviceListBuilt := false;
 NumAdapters := 0;
 FOwner := AOwner;
 CurrentAdapter := 0;
 FD3D8 := nil;
 FDevice := nil;
 VB := nil;
 if FhWnd = 0 then
  FhWnd := (AOwner as TForm).Handle;
 FhWndSave := FhWnd;
 FhWndFocus := 0;
 FWindowReady := FALSE;
 FDXReady := FALSE;
 dwCreateFlags := 0;
 strDeviceStats := '';
 FZBuffer := TRUE;
 FFullScreen := FALSE;
 FFrequency := D3DPRESENT_RATE_DEFAULT;
 FWidth := 800;
 FHeight := 600;
 FCenterX := 0; //Width div 2;
 FCenterY := 0; //Height div 2;
 FBPP := 16;
 dwMinDepthBits := 16;
 dwMinStencilBits := 0;
 FShowMouseCursor := TRUE;
 // Create the Direct3D object
 FD3D8 := Direct3DCreate8(D3D_SDK_VERSION);
 if (FD3D8 = nil) then
 begin
  DisplayErrorMsg(D3DAPPERR_NODIRECT3D, MSGERR_APPMUSTEXIT);
  exit;
 end;
 for i := 0 to 255 do
 begin
  FGammaRamp.red[i] := i * 256;
  FGammaRamp.green[i] := i * 256;
  FGammaRamp.blue[i] := i * 256;
 end;
 oldVP := Bounds(0, 0, FWidth, FHeight);
 if assigned(FOnCreate) then FOnCreate(self);
end;

function TOmegaScreen.Init: hresult;
var hr: HRESULT;
begin
 if (FD3D8 = nil) then
 begin
  result := DisplayErrorMsg(D3DAPPERR_NODIRECT3D, MSGERR_APPMUSTEXIT);
  exit;
 end;
 // The focus window can be a specified to be a different window than the
 // device window.  If not, use the device window as the focus window.
 if (FhWndFocus = 0) then FhWndFocus := FhWnd;
 // Save window properties
 dwWindowStyle := GetWindowLong(FhWnd, GWL_STYLE);
 GetWindowRect(FhWnd, rcWindowBounds);
 // Initialize
 hr := Initialize;
 if (FAILED(hr)) then
 begin
  FD3D8 := nil;
  result := DisplayErrorMsg(hr, MSGERR_APPMUSTEXIT);
  exit;
 end;
 ShowCursor(FShowMouseCursor);
 // The app is ready to go
 FDXReady := TRUE;
 result := S_OK;
end;

function TOmegaScreen.Initialize: HRESULT;
var
  pBackBuffer: IDIRECT3DSURFACE8;
  d3ddm: TD3DDisplayMode;
  dtype: TD3DDevType;
  fsok: boolean;
  pool: TD3DPOOL;
  behave: cardinal;
begin
 if FDevice <> nil then FDevice := nil;
 result := S_OK;
 fsok := false;
 if ((FWidth=800)and(FHeight=600)) or
 ((FWidth=640)and(FHeight=400)) or
 ((FWidth=640)and(FHeight=480)) or
 ((FWidth=1024)and(FHeight=768)) or
 ((FWidth=320)and(FHeight=200)) or
 ((FWidth=320)and(FHeight=240)) or
 ((FWidth=1280)and(FHeight=1024)) then fsok:=true;

  Fillchar(FD3DDEVPPAR8, sizeof(FD3DDEVPPAR8), 0);
  with FD3DDEVPPAR8 do
  begin
   if FBBAccess then Flags := D3DPRESENTFLAG_LOCKABLE_BACKBUFFER else Flags := 0;
   SwapEffect := D3DSWAPEFFECT_DISCARD;
   hDeviceWindow := FhWnd;
   if FFullScreen and not fsok then
   begin
    beep;
    BackBufferWidth := 800;
    BackBufferHeight := 600;
   end else
   begin
    BackBufferWidth := FWidth;
    BackBufferHeight := FHeight;
   end;
   Windowed := not FFullScreen;
   MultiSampleType := D3DMULTISAMPLE_NONE;
   EnableAutoDepthStencil := FZBuffer;
   AutoDepthStencilFormat := D3DFMT_D16; // 16-bit z-Buffer

   if (not Windowed) then
   begin // FullScreen
    if Fbpp=16 then BackBufferFormat := D3DFMT_R5G6B5
    else BackBufferFormat := D3DFMT_A8R8G8B8;
    if FBBCount<1 then FBBCount := 1;
    BackBufferCount := FBBCount;
    if FVSync then
     FullScreen_PresentationInterval := D3DPRESENT_INTERVAL_ONE
    else
     FullScreen_PresentationInterval := D3DPRESENT_INTERVAL_IMMEDIATE;
    FullScreen_RefreshRateInHz := FFrequency;
   end else
   begin // Windowed mode
    FD3D8.GetAdapterDisplayMode(D3DADAPTER_DEFAULT, d3ddm);
    BackBufferFormat := d3ddm.Format;
    FullScreen_PresentationInterval := 0;
    FullScreen_RefreshRateInHz := 0;
   end;
  end;

  case FRenderMode of
  rmReference: // Reference
  begin
   dtype := D3DDEVTYPE_REF;
   behave := D3DCREATE_SOFTWARE_VERTEXPROCESSING;
  end;
  rmSoftware: // Software
  begin
   dtype := D3DDEVTYPE_HAL;
   behave := D3DCREATE_SOFTWARE_VERTEXPROCESSING;
  end;
  else
  begin // Hardware
   dtype := D3DDEVTYPE_HAL;
   behave := D3DCREATE_SOFTWARE_VERTEXPROCESSING;
//   behave := D3DCREATE_HARDWARE_VERTEXPROCESSING;
  end;
  end;

  if failed(FD3D8.CreateDevice(D3DADAPTER_DEFAULT, dtype,
   FhWnd, behave, FD3DDEVPPAR8, FDevice)) then
  begin
   if FRenderMode = rmHardware then
   begin
    FRenderMode := rmSoftware;
    dtype := D3DDEVTYPE_HAL;
    behave := D3DCREATE_SOFTWARE_VERTEXPROCESSING;
    if failed(FD3D8.CreateDevice(D3DADAPTER_DEFAULT, dtype, FhWnd,
     behave, FD3DDEVPPAR8, FDevice)) then
    begin
     FD3D8 := nil;
     Result := S_FALSE;
     DisplayErrorMsg(D3DAPPERR_NOCOMPATIBLEDEVICES, MSGERR_APPMUSTEXIT);
    end
   end else
   begin
    FD3D8 := nil;
    Result := S_FALSE;
    DisplayErrorMsg(D3DAPPERR_NOCOMPATIBLEDEVICES, MSGERR_APPMUSTEXIT);
   end;
  end;

  // Get Device Caps.
  FillChar(FD3DDEVCAPS8, SizeOf(FD3DDEVCAPS8), 0);
  if failed(FDevice.GetDeviceCaps(FD3DDEVCAPS8)) then
  begin FDevice := nil; FD3D8 := nil; Result := S_FALSE; end;

  {//xxx
  if not FFullScreen then
   SetWindowPos(FhWnd, HWND_NOTOPMOST,
    rcWindowBounds.left, rcWindowBounds.top,
    (rcWindowBounds.right - rcWindowBounds.left),
    (rcWindowBounds.bottom - rcWindowBounds.top),
    SWP_SHOWWINDOW);
}

  if Result = S_FALSE then exit;

  // Store render target surface desc
  FDevice.GetBackBuffer(0, D3DBACKBUFFER_TYPE_MONO, pBackBuffer);
  pBackBuffer.GetDesc(BackBufferDesc);
  pBackBuffer := nil;

  // All right.
  DXReady := true; // Our DirectX is all initialized and ready to go!
  FWindowReady := TRUE;
  ResetRenderStates;

  if FPool = plDefault then
  begin
   behave := D3DUSAGE_WRITEONLY or D3DUSAGE_DYNAMIC;
   pool := D3DPOOL_DEFAULT;
  end else
  begin
   behave := D3DUSAGE_WRITEONLY;
   pool := D3DPOOL_MANAGED;
  end;
  FDevice.CreateVertexBuffer(sizeof(TOmegaVertex) * maxVertices,
   behave, OmegaVertexDef, pool, VB);

  FDevice.SetVertexShader(OmegaVertexDef);
  FDevice.SetStreamSource(0, VB, sizeof(TOmegaVertex));
  SubInit;
  if assigned(FOnInit) then FOnInit(self);
end;

// Callback function for sorting display modes (used by BuildDeviceList)
function SortModesCallback(const arg1: pointer; const arg2: pointer): integer; cdecl;
var p1: PD3DDISPLAYMODE;
    p2: PD3DDISPLAYMODE;
begin
 p1 := arg1;
 p2 := arg2;
 if (p1.Format > p2.Format) then
 begin
  result := -1;
  exit;
 end;
 if (p1.Format < p2.Format) then
 begin
  result := +1;
  exit;
 end;
 if (p1.Width < p2.Width) then
 begin
  result := -1;
  exit;
 end;
 if (p1.Width > p2.Width) then
 begin
  result := +1;
  exit;
 end;
 if (p1.Height < p2.Height) then
 begin
  result := -1;
  exit;
 end;
 if (p1.Height > p2.Height) then
 begin
  result := +1;
  exit;
 end;
 result := 0;
end;

// Displays error messages in a message box
function TOmegaScreen.DisplayErrorMsg(hr: int64; dwType: dword):Hresult;
begin
 ShowCursor(true);
 if hr = D3DAPPERR_NODIRECT3D then
  messagebox(0, 'Could not initialize Direct3D. You may'#10 +
        'want to check that the latest version of'#10 +
        'DirectX is correctly installed on your'#10 +
        'system.  Also make sure that this program'#10 +
        'was compiled with header files that match'#10 +
        'the installed DirectX DLLs.', 'Error', 1) else
 if hr = D3DAPPERR_NODIRECT3D then
  messagebox(0, 'Could not initialize Direct3D. You may'#10 +
        'want to check that the latest version of'#10 +
        'DirectX is correctly installed on your'#10 +
        'system.  Also make sure that this program'#10 +
        'was compiled with header files that match'#10 +
        'the installed DirectX DLLs.', 'Error', 1) else
 if hr = D3DAPPERR_NOCOMPATIBLEDEVICES then
  messagebox(0, 'Could not find any compatible Direct3D'#10'devices.',
  'Error', 1) else
 if hr = D3DAPPERR_NOWINDOWABLEDEVICES then
  messagebox(0, 'This application cannot run in a desktop'#10 +
        'window with the current display settings.'#10 +
        'Please change your desktop settings to a'#10 +
        '16- or 32-bit display mode and re-run this'#10 +
        'application.', 'Error', 1) else
 if hr = D3DAPPERR_NOHARDWAREDEVICE then
  messagebox(0, 'No hardware-accelerated Direct3D devices'#10 +
        'were found.', 'Error', 1) else
 if hr = D3DAPPERR_HALNOTCOMPATIBLE then
  messagebox(0, 'This application requires functionality that is'#10 +
        'not available on your Direct3D hardware'#10 +
        'accelerator.', 'Error', 1) else
 if hr = D3DAPPERR_NOWINDOWEDHAL then
   messagebox(0, 'Your Direct3D hardware accelerator cannot'#10 +
        'render into a window.', 'Error', 1) else
 if hr = D3DAPPERR_NODESKTOPHAL then
  messagebox(0, 'Your Direct3D hardware accelerator cannot'#10 +
        'render into a window with the current'#10 +
        'desktop display settings.', 'Error', 1) else
 if hr = D3DAPPERR_NOHALTHISMODE then
  messagebox(0, 'This application requires functionality that is'#10 +
        'not available on your Direct3D hardware'#10 +
        'accelerator with the current desktop display'#10 +
        'settings.', 'Error', 1) else
 if hr = D3DAPPERR_RESIZEFAILED then
  messagebox(0, 'Could not reset the Direct3D device.', 'Error', 1) else
 if hr = D3DAPPERR_NONZEROREFCOUNT then
  messagebox(0, 'A D3D object has a non-zero reference'#10 +
        'count (meaning things were not properly'#10 +
        'cleaned up).', 'Error', 1) else
 if hr = E_OUTOFMEMORY then
   messagebox(0, 'Not enough memory.', 'Error', 1) else
 if hr = D3DERR_OUTOFVIDEOMEMORY then
  messagebox(0, 'Not enough video memory.', 'Error', 1)
 else
  messagebox(0, 'Generic application error.', 'Error', 1);

  if (MSGERR_APPMUSTEXIT = dwType) then
  begin
   messagebox(0, 'This application will now exit.', 'Error', 1);
   halt(1);
  end
  else
  begin
    if (MSGWARN_SWITCHEDTOREF = dwType) then
     messagebox(0, 'Switching to the reference rasterizer,'#10 +
        'a software device that implements the entire'#10 +
        'Direct3D feature set, but runs very slowly.', 'Error', 1);
  end;
  result := hr;
end;

procedure TOmegaScreen.DrawRectangle(x1, y1, x2, y2: integer; Solid: boolean;
 BorderWidth: integer; Red, Green, Blue: integer);
var rect, rect2: TRect;
begin
  Rect.Left := x1;
  Rect.Right := x2 + 1;
  Rect.Top := y1;
  Rect.Bottom := y2 + 1;

  if Solid = true then
    FDevice.Clear(1, @rect, D3DCLEAR_TARGET,
     D3DColor_RGBA(red, green, blue, 255), 1.0, 0);
  if borderwidth > 0 then
  begin
   Rect.left := Rect.Left - BorderWidth;
   Rect.right := Rect.Right + BorderWidth;
   Rect.top := Rect.Top - BorderWidth;
   Rect.bottom := Rect.Bottom + BorderWidth;
   BorderWidth := -BorderWidth;
  end;
  // Draw top and left
  SetRect(rect2, Rect.left, Rect.top, Rect.right, Rect.top - borderWidth);
  FDevice.Clear(1, @rect2, D3DCLEAR_TARGET,
   D3DColor_RGBA(red, green, blue, 255), 1.0, 0);
  SetRect(rect2, Rect.left, Rect.top, Rect.left - borderWidth, Rect.bottom);
  FDevice.Clear(1, @rect2, D3DCLEAR_TARGET,
   D3DColor_RGBA(red, green, blue, 255), 1.0, 0);
  // Draw bottom and right
  SetRect(rect2, Rect.left, Rect.bottom + borderwidth, Rect.right, Rect.bottom);
  FDevice.Clear(1, @rect2, D3DCLEAR_TARGET,
   D3DColor_RGBA(red, green, blue, 255), 1.0, 0);
  SetRect(rect2, Rect.right + borderwidth, Rect.top, Rect.right, Rect.bottom);
  FDevice.Clear(1, @rect2, D3DCLEAR_TARGET,
   D3DColor_RGBA(red, green, blue, 255), 1.0, 0);
end;

function TOmegaScreen.BuildDeviceList: HRESULT;
const
  NumDeviceTypes: DWORD = 3;
  strDeviceDescs: array[0..2] of string = ('HAL (Hardware Renderer)', 'REF (Reference Renderer)', 'SW (Software Renderer)');
  DeviceTypes: array[0..2] of TD3DDEVTYPE = (D3DDEVTYPE_HAL, D3DDEVTYPE_REF, D3DDEVTYPE_SW);
var
 bHALExists: BOOL;
 bHALIsWindowedCompatible: BOOL;
 bHALIsDesktopCompatible: BOOL;
 bHALIsSampleCompatible: BOOL;
 iAdapter: integer;
 pAdapter: PD3DAdapterInfo;
 modes: array[0..99] of TD3DDISPLAYMODE;
 formats: array[0..19] of TD3DFORMAT;
 NumFormats: DWORD;
 NumModes: DWORD;
 NumAdapterModes: DWORD;
 iMode: Cardinal;
 DisplayMode: TD3DDISPLAYMODE;
 m, f, a, d: DWORD;
 iDevice: Cardinal;
 pDevice: FDeviceInfo;
 bFormatConfirmed: array[0..19] of BOOL;
 Behavior: array[0..19] of DWORD;
 fmtDepthStencil: array[0..19] of TD3DFORMAT;
begin
 bHALExists := FALSE;
 bHALIsWindowedCompatible := FALSE;
 bHALIsDesktopCompatible := FALSE;
 bHALIsSampleCompatible := FALSE;
 // Loop through all the adapters on the system (usually, there's just one
 // unless more than one graphics card is present).
 for iAdapter := 0 to (FD3D8.GetAdapterCount - 1) do
 begin
  // Fill in adapter info
  pAdapter := @Adapters[NumAdapters];
  FD3D8.GetAdapterIdentifier(iAdapter, 0, pAdapter.AdapterIdentifier);
  FD3D8.GetAdapterDisplayMode(iAdapter, pAdapter.Desktop);
  pAdapter.NumDevices := 0;
  pAdapter.CurrentDevice := 0;
  // Enumerate all display modes on this adapter
  NumFormats := 0;
  NumModes := 0;
  NumAdapterModes := FD3D8.GetAdapterModeCount(iAdapter);
  // Add the adapter's current desktop format to the list of formats
  formats[NumFormats] := pAdapter.Desktop.Format;
  inc(NumFormats);
  for iMode := 0 to NumAdapterModes - 1 do
  begin
   // Get the display mode attributes
   FD3D8.EnumAdapterModes(iAdapter, iMode, DisplayMode);

   // Check if the mode already exists (to filter out refresh rates)
   m := 0;
   if NumModes <> 0 then
   for m := 0 to (NumModes - 1) do
   begin
    if ((modes[m].Width = DisplayMode.Width) and
        (modes[m].Height = DisplayMode.Height) and
        (modes[m].Format = DisplayMode.Format)) then break;
   end;
   // If we found a new mode, add it to the list of modes
   if (m = NumModes) then
   begin
    modes[NumModes].Width := DisplayMode.Width;
    modes[NumModes].Height := DisplayMode.Height;
    modes[NumModes].Format := DisplayMode.Format;
    modes[NumModes].RefreshRate := 0;//DisplayMode.RefreshRate;
    inc(NumModes);
    // Check if the mode's format already exists
    for f := 0 to NumFormats - 1 do
     if (DisplayMode.Format = formats[f]) then break;
    // If the format is new, add it to the list
    if (f = NumFormats) then
    begin
     formats[NumFormats] := DisplayMode.Format;
     inc(NumFormats);
    end;
   end;
  end;
  // Sort the list of display modes (by format, then width, then height)
  qsort(@modes, NumModes, sizeof(TD3DDISPLAYMODE), @SortModesCallback);
  // Add devices to adapter
  for iDevice := 0 to NumDeviceTypes - 1 do
  begin
   // Fill in device info
   pDevice := @pAdapter.devices[pAdapter.NumDevices];
   pDevice.DeviceType := DeviceTypes[iDevice];
   FD3D8.GetDeviceCaps(iAdapter, DeviceTypes[iDevice], pDevice.Caps);
   pDevice.strDesc := strDeviceDescs[iDevice];
   pDevice.NumModes := 0;
   pDevice.CurrentMode := 0;
   pDevice.canDoWindowed := FALSE;
   pDevice.isWindowed := FALSE;
   pDevice.MultiSampleType := D3DMULTISAMPLE_NONE;
   // Examine each format supported by the adapter to see if it will
   // work with this device and meets the needs of the application.
   for f := 0 to NumFormats - 1 do
   begin
    bFormatConfirmed[f] := FALSE;
    fmtDepthStencil[f] := D3DFMT_UNKNOWN;
    // Skip formats that cannot be used as render targets on this device
    if (FAILED(FD3D8.CheckDeviceType(iAdapter, pDevice.DeviceType,
     formats[f], formats[f], FALSE))) then continue;
    if (pDevice.DeviceType = D3DDEVTYPE_HAL) then
    begin
     // This system has a HAL device
     bHALExists := TRUE;
     if (pDevice.Caps.Caps2 and D3DCAPS2_CANRENDERWINDOWED) = D3DCAPS2_CANRENDERWINDOWED then
     begin
      // HAL can run in a window for some mode
      bHALIsWindowedCompatible := TRUE;
      if (f = 0) then
      begin
       // HAL can run in a window for the current desktop mode
       bHALIsDesktopCompatible := TRUE;
      end;
     end;
    end;                                               
    // Confirm the device/format for HW vertex processing
    if (pDevice.Caps.DevCaps and D3DDEVCAPS_HWTRANSFORMANDLIGHT) = D3DDEVCAPS_HWTRANSFORMANDLIGHT then
    begin
     if (pDevice.Caps.DevCaps and D3DDEVCAPS_PUREDEVICE) = D3DDEVCAPS_PUREDEVICE then
     begin
      Behavior[f] := D3DCREATE_HARDWARE_VERTEXPROCESSING or D3DCREATE_PUREDEVICE;
      bFormatConfirmed[f] := TRUE;
     end;
     if (FALSE = bFormatConfirmed[f]) then
     begin
      Behavior[f] := D3DCREATE_HARDWARE_VERTEXPROCESSING;
      bFormatConfirmed[f] := TRUE;
     end;
     if (FALSE = bFormatConfirmed[f]) then
     begin
      Behavior[f] := D3DCREATE_MIXED_VERTEXPROCESSING;
      bFormatConfirmed[f] := TRUE;
     end;
    end;
    // Confirm the device/format for SW vertex processing
    if (FALSE = bFormatConfirmed[f]) then
    begin
     Behavior[f] := D3DCREATE_SOFTWARE_VERTEXPROCESSING;
     bFormatConfirmed[f] := TRUE;
    end;

    // Find a suitable depth/stencil buffer format for this device/format
    if (bFormatConfirmed[f] and FZBuffer) then
    begin
     if (not FindDepthStencilFormat(iAdapter, pDevice.DeviceType,
      formats[f], fmtDepthStencil[f])) then bFormatConfirmed[f] := FALSE;
    end;
   end;
   // Add all enumerated display modes with confirmed formats to the
   // device's list of valid modes
   if NumModes <> 0 then
   for m := 0 to NumModes - 1 do
   begin
    for f := 0 to NumFormats - 1 do
    begin
     if (modes[m].Format = formats[f]) then
     begin
      if (bFormatConfirmed[f] = TRUE) then
      begin
       // Add this mode to the device's list of valid modes
       pDevice.modes[pDevice.NumModes].Width := modes[m].Width;
       pDevice.modes[pDevice.NumModes].Height := modes[m].Height;
       pDevice.modes[pDevice.NumModes].Format := modes[m].Format;
       pDevice.modes[pDevice.NumModes].Behavior := Behavior[f];
       pDevice.modes[pDevice.NumModes].DepthStencilFormat := fmtDepthStencil[f];
       inc(pDevice.NumModes);
       if (pDevice.DeviceType = D3DDEVTYPE_HAL) then
        bHALIsSampleCompatible := TRUE;
      end;
     end;
    end;
   end;

   FDeviceListBuilt := true;
   // Select any 800x600 mode for default (but prefer a 16-bit mode)
   if pDevice.NumModes <> 0 then
   for m := 0 to pDevice.NumModes - 1 do
   begin
    if (pDevice.modes[m].Width = 800) and (pDevice.modes[m].Height = 600) then
    begin
     pDevice.CurrentMode := m;
     if (pDevice.modes[m].Format = D3DFMT_R5G6B5) or
        (pDevice.modes[m].Format = D3DFMT_X1R5G5B5) or
        (pDevice.modes[m].Format = D3DFMT_A1R5G5B5) then break;
    end;
   end;
   // Check if the device is compatible with the desktop display mode
   // (which was added initially as formats[0])
   if (bFormatConfirmed[0]) and ((pDevice.Caps.Caps2 and D3DCAPS2_CANRENDERWINDOWED) = D3DCAPS2_CANRENDERWINDOWED) then
   begin
    pDevice.canDoWindowed := TRUE;
    pDevice.isWindowed := TRUE;
   end;
   // If valid modes were found, keep this device
   if (pDevice.NumModes > 0) then inc(pAdapter.NumDevices);
  end;
  // If valid devices were found, keep this adapter
  if (pAdapter.NumDevices > 0) then inc(NumAdapters);
 end;
 // Return an error if no compatible devices were found
 if (NumAdapters = 0) then
 begin
  result := D3DAPPERR_NOCOMPATIBLEDEVICES;
  exit;
 end;
 // Pick a default device that can render into a window
 // This code assumes that the HAL device comes before the REF
 // device in the device array
 for a := 0 to NumAdapters - 1 do
 begin
  for d := 0 to Adapters[a].NumDevices - 1 do
  begin
   if (Adapters[a].devices[d].isWindowed) then
   begin
    Adapters[a].CurrentDevice := d;
    CurrentAdapter := a;
    FFullScreen := false;
    // Display a warning message
    if (Adapters[a].devices[d].DeviceType = D3DDEVTYPE_REF) then
    begin
     if (not bHALExists) then
      DisplayErrorMsg(D3DAPPERR_NOHARDWAREDEVICE, MSGWARN_SWITCHEDTOREF)
     else if (not bHALIsSampleCompatible) then
      DisplayErrorMsg(D3DAPPERR_HALNOTCOMPATIBLE, MSGWARN_SWITCHEDTOREF)
     else if (not bHALIsWindowedCompatible) then
      DisplayErrorMsg(D3DAPPERR_NOWINDOWEDHAL, MSGWARN_SWITCHEDTOREF)
     else if (not bHALIsDesktopCompatible) then
      DisplayErrorMsg(D3DAPPERR_NODESKTOPHAL, MSGWARN_SWITCHEDTOREF)
     else // HAL is desktop compatible, but not sample compatible
      DisplayErrorMsg(D3DAPPERR_NOHALTHISMODE, MSGWARN_SWITCHEDTOREF);
    end;
    result := S_OK;
    exit;
   end;
  end;
 end;
 result := D3DAPPERR_NOWINDOWABLEDEVICES;
end;

procedure TOmegaScreen.Cleanup;
begin
 FWindowReady := FALSE;
 FDXReady := FALSE;
 if (FDevice <> nil) then
 begin
  SubUnInit;
  if assigned(FOnUnInit) then FOnUnInit(self);
  if VB <> nil then VB := nil;
  FDevice := nil;
  FD3D8 := nil;
 end;
end;

function TOmegaScreen.ReInit: HRESULT;
var hr: HRESULT;
    fsok: boolean;
    d3ddm: TD3DDisplayMode;
    behave: cardinal;
    pool: TD3DPOOL;
begin
 if not assigned(FDevice) then begin result:=S_FALSE;exit end;
 CaptureRenderStates;
 SubUnInit;
 if vb <> nil then vb := nil;
 if assigned(FOnUnInit) then FOnUnInit(self);
 FDevice.SetTexture(0, nil);

{ if not FFullScreen then
 begin
  GetWindowRect(FhWnd, rcWindowBounds);
  SetWindowPos(FhWnd, HWND_NOTOPMOST,
    rcWindowBounds.left, rcWindowBounds.top,
   (rcWindowBounds.right - rcWindowBounds.left),
   (rcWindowBounds.bottom - rcWindowBounds.top),
    SWP_SHOWWINDOW);
 end;
}
 fsok:=false;
 if ((FWidth=800)and(FHeight=600)) or
 ((FWidth=640)and(FHeight=400)) or
 ((FWidth=640)and(FHeight=480)) or
 ((FWidth=1024)and(FHeight=768)) or
 ((FWidth=320)and(FHeight=200)) or
 ((FWidth=320)and(FHeight=240)) or
 ((FWidth=1280)and(FHeight=1024)) then fsok:=true;

 with FD3DDEVPPAR8 do
 begin
  Windowed := not FFullScreen;
  if FBBAccess then Flags := D3DPRESENTFLAG_LOCKABLE_BACKBUFFER else Flags := 0;
  if (not Windowed) then
  begin // FullScreen
   if not fsok then
   begin
    beep;
    BackBufferWidth := 800;
    BackBufferHeight := 600;
   end else
   begin
    BackBufferWidth := FWidth;
    BackBufferHeight := FHeight;
   end;
   if Fbpp = 16 then BackBufferFormat := D3DFMT_R5G6B5
   else BackBufferFormat := D3DFMT_A8R8G8B8;
   if FBBCount<1 then FBBCount := 1;
   BackBufferCount := FBBCount;
   FullScreen_RefreshRateInHz := FFrequency;
   if FVSync then
    FullScreen_PresentationInterval := D3DPRESENT_INTERVAL_ONE
   else
    FullScreen_PresentationInterval := D3DPRESENT_INTERVAL_IMMEDIATE;
  end else
  begin // Windowed mode
   BackBufferWidth := FWidth;
   BackBufferHeight := FHeight;
   FD3D8.GetAdapterDisplayMode(D3DADAPTER_DEFAULT, d3ddm);
   BackBufferFormat := d3ddm.Format;
   FullScreen_PresentationInterval := 0;
   FullScreen_RefreshRateInHz := 0;
  end;
 end;

 // Reset the device
 hr := FDevice.Reset(FD3DDEVPPAR8);
 if (FAILED(hr)) then
 begin
  result := hr;
  exit;
 end;

 ShowCursor(FShowMouseCursor);

  if FPool = plDefault then
  begin
   behave := D3DUSAGE_WRITEONLY or D3DUSAGE_DYNAMIC;
   pool := D3DPOOL_DEFAULT;
  end else
  begin
   behave := D3DUSAGE_WRITEONLY;
   pool := D3DPOOL_MANAGED;
  end;
  FDevice.CreateVertexBuffer(sizeof(TOmegaVertex) * maxVertices,
   behave, OmegaVertexDef, pool, VB);

 RestoreRenderStates;
 SubInit;
 if assigned(FOnInit) then FOnInit(self);
 result := S_OK;
end;

// Called when user toggles between fullscreen mode and windowed mode
function TOmegaScreen.ToggleFullscreen: HRESULT;
begin
 FWindowReady := FALSE;
 FFullScreen := not FFullScreen;
 ReInit;
 // When moving from fullscreen to windowed mode, it is important to
 // adjust the window size after resetting the device rather than
 // beforehand to ensure that you get the window size you want.  For
 // example, when switching from 640x480 fullscreen to windowed with
 // a 1000x600 window on a 1024x768 desktop, it is impossible to set
 // the window size to 1000x600 until after the display mode has
 // changed to 1024x768, because windows cannot be larger than the
 // desktop.
 if not FFullScreen then
 begin
  SetWindowPos(FhWnd, HWND_NOTOPMOST,
   rcWindowBounds.left, rcWindowBounds.top,
   (rcWindowBounds.right - rcWindowBounds.left),
   (rcWindowBounds.bottom - rcWindowBounds.top),
   SWP_SHOWWINDOW);
 end;
 sethWnd(FhWndSave);

 FDXReady := TRUE;
 FWindowReady := TRUE;
 result := S_OK;
end;

// Finds a depth/stencil format for the given device that is compatible
// with the render target format.
function TOmegaScreen.FindDepthStencilFormat(iAdapter: UINT; DeviceType: TD3DDEVTYPE;
 TargetFormat: TD3DFORMAT; var pDepthStencilFormat: TD3DFORMAT): BOOL;
begin
 if (dwMinDepthBits <= 16) and (dwMinStencilBits = 0) then
 begin
  if (SUCCEEDED(FD3D8.CheckDeviceFormat(iAdapter, DeviceType,
   TargetFormat, D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D16))) then
  begin
   if (SUCCEEDED(FD3D8.CheckDepthStencilMatch(iAdapter, DeviceType,
    TargetFormat, TargetFormat, D3DFMT_D16))) then
   begin
    pDepthStencilFormat := D3DFMT_D16;
    result := TRUE;
    exit;
   end;
  end;
 end;
 if (dwMinDepthBits <= 15) and (dwMinStencilBits <= 1) then
 begin
  if (SUCCEEDED(FD3D8.CheckDeviceFormat(iAdapter, DeviceType,
   TargetFormat, D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D15S1))) then
  begin
   if (SUCCEEDED(FD3D8.CheckDepthStencilMatch(iAdapter, DeviceType,
    TargetFormat, TargetFormat, D3DFMT_D15S1))) then
   begin
    pDepthStencilFormat := D3DFMT_D15S1;
    result := TRUE;
    exit;
   end;
  end;
 end;
 if (dwMinDepthBits <= 24) and (dwMinStencilBits = 0) then
 begin
  if (SUCCEEDED(FD3D8.CheckDeviceFormat(iAdapter, DeviceType,
   TargetFormat, D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D24X8))) then
  begin
   if (SUCCEEDED(FD3D8.CheckDepthStencilMatch(iAdapter, DeviceType,
    TargetFormat, TargetFormat, D3DFMT_D24X8))) then
   begin
    pDepthStencilFormat := D3DFMT_D24X8;
    result := TRUE;
    exit;
   end;
  end;
 end;
 if (dwMinDepthBits <= 24) and (dwMinStencilBits <= 8) then
 begin
  if (SUCCEEDED(FD3D8.CheckDeviceFormat(iAdapter, DeviceType,
   TargetFormat, D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D24S8))) then
  begin
   if (SUCCEEDED(FD3D8.CheckDepthStencilMatch(iAdapter, DeviceType,
    TargetFormat, TargetFormat, D3DFMT_D24S8))) then
   begin
    pDepthStencilFormat := D3DFMT_D24S8;
    result := TRUE;
    exit;
   end;
  end;
 end;
 if (dwMinDepthBits <= 24) and (dwMinStencilBits <= 4) then
 begin
  if (SUCCEEDED(FD3D8.CheckDeviceFormat(iAdapter, DeviceType,
   TargetFormat, D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D24X4S4))) then
  begin
   if (SUCCEEDED(FD3D8.CheckDepthStencilMatch(iAdapter, DeviceType,
    TargetFormat, TargetFormat, D3DFMT_D24X4S4))) then
   begin
    pDepthStencilFormat := D3DFMT_D24X4S4;
    result := TRUE;
    exit;
   end;
  end;
 end;
 if (dwMinDepthBits <= 32) and (dwMinStencilBits = 0) then
 begin
  if (SUCCEEDED(FD3D8.CheckDeviceFormat(iAdapter, DeviceType,
   TargetFormat, D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D32))) then
  begin
   if (SUCCEEDED(FD3D8.CheckDepthStencilMatch(iAdapter, DeviceType,
    TargetFormat, TargetFormat, D3DFMT_D32))) then
   begin
    pDepthStencilFormat := D3DFMT_D32;
    result := TRUE;
    exit;
   end;
  end;
 end;
 result := FALSE;
end;

function TOmegaScreen.CanDraw: boolean;
begin
 result := FWindowReady and FDXReady;
end;

destructor TOmegaScreen.Destroy;
begin
 ShowCursor(true);
 Cleanup;
 if assigned(FOnDestroy) then FOnDestroy(self);
 SetWindowPos(FhWnd, HWND_NOTOPMOST,
  rcWindowBounds.left, rcWindowBounds.top,
  (rcWindowBounds.right - rcWindowBounds.left),
  (rcWindowBounds.bottom - rcWindowBounds.top),
  SWP_SHOWWINDOW);
 inherited Destroy;
end;

procedure TOmegaScreen.ClearScreen(r, g, b: Byte; a: Byte=255);
begin
 if FZBuffer then
  FDevice.Clear(0, nil, D3DCLEAR_ZBUFFER or D3DCLEAR_TARGET, D3DCOLOR_RGBA(r, g, b, a), 1, 0)
 else
  FDevice.Clear(0, nil, D3DCLEAR_TARGET, D3DCOLOR_RGBA(r, g, b, a), 1, 0);
end;

procedure TOmegaScreen.SetAntiAliasing(activate: boolean);
begin
 if activate then
 begin
  FDevice.SetTextureStageState(0, D3DTSS_MAGFILTER, 2);
  FDevice.SetTextureStageState(0, D3DTSS_MINFILTER, 2);
 end else
 begin
  FDevice.SetTextureStageState(0, D3DTSS_MAGFILTER, 1);
  FDevice.SetTextureStageState(0, D3DTSS_MINFILTER, 1);
 end;
 FAntiAliasing := activate;
end;

procedure TOmegaScreen.SetDithering(activate: boolean);
begin
 if activate then
  FDevice.SetRenderState(D3DRS_DITHERENABLE, Cardinal(TRUE))
 else
  FDevice.SetRenderState(D3DRS_DITHERENABLE, Cardinal(FALSE));
 FDithering := activate;
end;

procedure TOmegaScreen.SetWireframe(activate: boolean);
begin
 if activate then
  FDevice.SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME)
 else
  FDevice.SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
 FWireframe := activate;
end;

procedure TOmegaScreen.SetAlphaBlending(activate: boolean);
begin
 if activate then
  FDevice.SetRenderState(D3DRS_ALPHABLENDENABLE, cardinal(TRUE))
 else
  FDevice.SetRenderState(D3DRS_ALPHABLENDENABLE, cardinal(FALSE));
end;

procedure TOmegaScreen.SetBlendMode(mode: integer);
begin
 if (mode = FBlendMode) or (mode = bmIgnore) then exit;
 case mode of
 bmNormal:
 begin
  FDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
  FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
 end;
 bmAdd:
 begin
  FDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
  FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
 end;
 bmSrcAdd:
 begin
  FDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
  FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
 end;
 bmSrcCol:
 begin
  FDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCCOLOR);
  FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCCOLOR);
 end;                      
 bmSrcColAdd:
 begin
  FDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCCOLOR);
  FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
 end;
 bmMultiply:
 begin
  FDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ZERO);
  FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_SRCCOLOR);
 end;
 bmMultiplyAlpha:
 begin
  FDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ZERO);
  FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_SRCALPHA);
 end;
 bmInvMultiply:
 begin
  FDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ZERO);
  FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCCOLOR);
 end;
 bmInvMultiplyAlpha:
 begin
  FDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ZERO);
  FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
 end;
 bmInvert:
 begin
  FDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_INVDESTCOLOR);
  FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
 end;
 bmSrcBright:
 begin
  FDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCCOLOR);
  FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_SRCCOLOR);
 end;
 bmDestBright:
 begin
  FDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_DESTCOLOR);
  FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_DESTCOLOR);
 end;
 bmInvSrcBright:
 begin
  FDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_INVSRCCOLOR);
  FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCCOLOR);
 end;
 bmInvDestBright:
 begin
  FDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_INVDESTCOLOR);
  FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVDESTCOLOR);
 end
 else
  FDevice.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
  FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
 end;
 FBlendMode := mode;
end;

procedure TOmegaScreen.ScreenShot(FileName: string);
begin
 FTakeScreenShot := True;
 FScreenShotFile := FileName;
end;

procedure TOmegaScreen.TakeScreenShot(FileName: string);
var Modes: TD3DDISPLAYMODE;
  surface: IDIRECT3DSURFACE8;
  pt: TPoint;
  srcCpy: TRect;
begin
 if Failed(FDevice.GetDisplayMode(Modes)) then Exit;
 if Failed(FDevice.CreateImageSurface(modes.width, modes.height, D3DFMT_A8R8G8B8, surface)) then exit;
 if Failed(FDevice.GetFrontBuffer(surface)) then exit;
 Pt.x := 0;
 Pt.Y := 0;
 ClientToScreen(Self.FhWnd, Pt);
 GetClientRect(Self.FhWnd, SrcCpy);
 SrcCpy.top := Pt.y;
 SrcCpy.left := Pt.x;
 SrcCpy.bottom := SrcCpy.Bottom + Pt.y;
 SrcCpy.right := SrcCpy.Right + Pt.x;
 D3DXSaveSurfaceToFile(pChar(fileName), D3DXIFF_BMP, surface, nil, @SrcCpy);
 FTakeScreenShot := False;
end;

procedure TOmegaScreen.BeginRender;
var hr: HRESULT;
//    pAdapterInfo: PD3DAdapterInfo;
    d3ddm: TD3DDisplayMode;
begin
 // Test the cooperative level to see if it's okay to render
 hr := FDevice.TestCooperativeLevel;
 if (FAILED(hr)) then
 begin
  // If the device was lost, do not render until we get it back
  if (hr = D3DERR_DEVICELOST) then exit;
  // Check if the device needs to be resized.
  if (hr = D3DERR_DEVICENOTRESET) then
  begin
   // If we are windowed, read the desktop mode and use the same format for
   // the back buffer
   if (not FFullScreen) then
   begin
    FD3D8.GetAdapterDisplayMode(D3DADAPTER_DEFAULT, d3ddm);
    FD3DDEVPPAR8.BackBufferFormat := d3ddm.Format;
   end;
   hr := ReInit;
   if (FAILED(hr)) then exit;
  end;
  exit;
 end;

 if (FDXReady = FALSE) or (FWindowReady = FALSE) then exit;
 FDevice.BeginScene;
 Rendering := true;
end;

procedure TOmegaScreen.EndRender;
begin
 if not Rendering then exit;
 Rendering := false;
 FDevice.EndScene;
 if FTakeScreenshot then TakeScreenshot(FScreenShotFile);
 FDevice.Present(nil, nil, FhWnd, nil);
end;

procedure TOmegaScreen.SetTarget;
var pBack, pStencil: IDIRECT3DSURFACE8;
begin
 FDevice.GetBackBuffer(0, D3DBackBuffer_Type_Mono, pback);
 FDevice.GetDepthStencilSurface(pStencil);
 FDevice.SetRenderTarget(pBack, pStencil);
 pBack._Release;
 if pStencil <> nil then pStencil._Release;
end;

function TOmegaScreen.GetViewPort: TRect;
var Viewport: TD3DViewport8;
begin
 FillChar(Result, SizeOf(Result), 0);
 FillChar(Viewport, SizeOf(Viewport), 0);
 FDevice.GetViewport(Viewport);
 Result:= Bounds(ViewPort.X, ViewPort.Y, ViewPort.Width, ViewPort.Height);
end;

procedure TOmegaScreen.SetViewPort(Rect: TRect);
begin
 SetViewPort(Rect.Left, Rect.Top, Rect.Right, Rect.Bottom);
end;

procedure TOmegaScreen.SetViewPort(x1, y1, x2, y2: Cardinal);
var Viewport: TD3DViewport8;
begin                 
 oldVP := GetViewPort;
 FillChar(Viewport, SizeOf(Viewport), 0);
 Viewport.X := x1;
 Viewport.Y := y1;
 Viewport.Width := x2 - x1;
 Viewport.Height := y2 - y1;
 FDevice.SetViewport(ViewPort);
end;

procedure TOmegaScreen.ResetViewPort;
begin
 SetViewPort(0, 0, FWidth, FHeight);
end;

procedure TOmegaScreen.SetPreviousViewPort;
begin
 SetViewPort(oldVP);
end;

function TOmegaScreen.GetBackBuffer(n: integer): IDirect3DSurface8;
begin
 Result:= nil;
 if (FDevice = nil) then Exit;
 FDevice.GetBackBuffer(n, D3DBACKBUFFER_TYPE_MONO, Result);
end;

procedure TOmegaScreen.SetBPP(const Value: LongInt);
begin
 if (Value = 32) then FBPP := 32
 else FBPP := 16;
end;

procedure TOmegaScreen.CaptureRenderStates;
begin
 with FDevice do
 begin
  GetRenderState(D3DRS_LIGHTING, FRenderStates[0]);
  GetRenderState(D3DRS_LASTPIXEL, FRenderStates[1]);
  GetRenderState(D3DRS_CULLMODE, FRenderStates[2]);
  GetRenderState(D3DRS_ZENABLE, FRenderStates[3]);
  GetRenderState(D3DRS_ALPHABLENDENABLE, FRenderStates[4]);
  GetRenderState(D3DRS_SRCBLEND, FRenderStates[5]);
  GetRenderState(D3DRS_DESTBLEND, FRenderStates[6]);
  GetRenderState(D3DRS_DITHERENABLE, FRenderStates[7]);
  GetTextureStageState(0, D3DTSS_COLOROP, FRenderStates[8]);
  GetTextureStageState(0, D3DTSS_COLORARG1, FRenderStates[9]);
  GetTextureStageState(0, D3DTSS_COLORARG2, FRenderStates[10]);
  GetTextureStageState(0, D3DTSS_ALPHAOP, FRenderStates[11]);
  GetTextureStageState(0, D3DTSS_ALPHAARG1, FRenderStates[12]);
  GetTextureStageState(0, D3DTSS_ALPHAARG2, FRenderStates[13]);
  GetTextureStageState(0, D3DTSS_MAGFILTER, FRenderStates[14]);
  GetTextureStageState(0, D3DTSS_MINFILTER, FRenderStates[15]);
  GetRenderState(D3DRS_FILLMODE, FRenderStates[16]);
 end;
end;

procedure TOmegaScreen.RestoreRenderStates;
begin
 with FDevice do
 begin
  SetRenderState(D3DRS_LIGHTING, FRenderStates[0]);
  SetRenderState(D3DRS_LASTPIXEL, FRenderStates[1]);
  SetRenderState(D3DRS_CULLMODE, FRenderStates[2]);
  SetRenderState(D3DRS_ZENABLE, FRenderStates[3]);
  SetRenderState(D3DRS_ALPHABLENDENABLE, FRenderStates[4]);
  SetRenderState(D3DRS_SRCBLEND, FRenderStates[5]);
  SetRenderState(D3DRS_DESTBLEND, FRenderStates[6]);
  SetRenderState(D3DRS_DITHERENABLE, FRenderStates[7]);
  SetTextureStageState(0, D3DTSS_COLOROP, FRenderStates[8]);
  SetTextureStageState(0, D3DTSS_COLORARG1, FRenderStates[9]);
  SetTextureStageState(0, D3DTSS_COLORARG2, FRenderStates[10]);
  SetTextureStageState(0, D3DTSS_ALPHAOP, FRenderStates[11]);
  SetTextureStageState(0, D3DTSS_ALPHAARG1, FRenderStates[12]);
  SetTextureStageState(0, D3DTSS_ALPHAARG2, FRenderStates[13]);
  SetTextureStageState(0, D3DTSS_MAGFILTER, FRenderStates[14]);
  SetTextureStageState(0, D3DTSS_MINFILTER, FRenderStates[15]);
  SetRenderState(D3DRS_FILLMODE, FRenderStates[16]);
 end;
end;

procedure TOmegaScreen.ResetRenderStates;
begin
 with FDevice do
 begin
  SetRenderState(D3DRS_LIGHTING, cardinal(FALSE));
  SetRenderState(D3DRS_LASTPIXEL, cardinal(TRUE));
  SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
  SetRenderState(D3DRS_ZENABLE, cardinal(TRUE));
  SetRenderState(D3DRS_ALPHABLENDENABLE, cardinal(TRUE));
  SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
  SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );
  SetRenderState(D3DRS_DITHERENABLE, Cardinal(FALSE));
  SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
  SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
  SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
  SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
  SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);
  SetTextureStageState(0, D3DTSS_MAGFILTER, 1);
  SetTextureStageState(0, D3DTSS_MINFILTER, 1);
  SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_NONE);
  SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
 end;
end;

procedure TOmegaScreen.SubInit;
var i:integer;
begin
 if assigned(FDevice) then
 begin
  FDevice.SetTexture(0, nil);
  FDevice.SetGammaRamp(0, FGammaRamp);
 end;
 if FOwner = nil then exit;
 for i := 0 to FOwner.ComponentCount - 1 do
  if FOwner.Components[i] is TOmegaImageList then TOmegaImageList(FOwner.Components[i]).Init;
 for i := 0 to FOwner.ComponentCount - 1 do
 begin
  if FOwner.Components[i] is TOmegaFlexiGrid then TOmegaFlexiGrid(FOwner.Components[i]).Init;
  if FOwner.Components[i] is TOmegaFont then TOmegaFont(FOwner.Components[i]).Init;
  if FOwner.Components[i] is TOmegaBitmapFont then TOmegaBitmapFont(FOwner.Components[i]).Init;
  if FOwner.Components[i] is TOmegaVideo then TOmegaVideo(FOwner.Components[i]).Init;
  if FOwner.Components[i] is TOmegaGUI then TOmegaGUI(FOwner.Components[i]).Init;
  if FOwner.Components[i] is TOmegaInput then TOmegaInput(FOwner.Components[i]).Initialize;
 end;
end;

procedure TOmegaScreen.SubUnInit;
var i:integer;
begin
 if assigned(FDevice) then FDevice.SetTexture(0, nil);
 if FOwner = nil then exit;
 for i := 0 to FOwner.ComponentCount - 1 do
 begin
  if FOwner.Components[i] is TOmegaBitmapFont then TOmegaBitmapFont(FOwner.Components[i]).UnInit;
  if FOwner.Components[i] is TOmegaVideo then TOmegaVideo(FOwner.Components[i]).UnInit;
 end;
 for i := 0 to FOwner.ComponentCount - 1 do
 begin
  if FOwner.Components[i] is TOmegaFlexiGrid then TOmegaFlexiGrid(FOwner.Components[i]).UnInit;
  if FOwner.Components[i] is TOmegaFont then TOmegaFont(FOwner.Components[i]).UnInit;
  if FOwner.Components[i] is TOmegaImageList then TOmegaImageList(FOwner.Components[i]).UnInit;
  if FOwner.Components[i] is TOmegaGUI then TOmegaGUI(FOwner.Components[i]).Stop;
 end;
end;

procedure TOmegaScreen.Notification(AComponent: TComponent; Operation: TOperation);
begin
 inherited;
 if (Operation = opinsert) then
 begin
  if (AComponent is TOmegaImageList) then
   TOmegaImageList(AComponent).OmegaScreen := self;
  if (AComponent is TOmegaFader) then
   TOmegaFader(AComponent).OmegaScreen := self;
  if (AComponent is TOmegaBitmapFont) then
   TOmegaBitmapFont(AComponent).OmegaScreen := self;
  if (AComponent is TOmegaPrimitives) then
   TOmegaPrimitives(AComponent).OmegaScreen := self;
  if (AComponent is TOmegaVideo) then
   TOmegaVideo(AComponent).OmegaScreen := self;
  if (AComponent is TOmegaSurface) then
   TOmegaSurface(AComponent).OmegaScreen := self;
  if (AComponent is TOmegaFlexiGrid) then
   TOmegaFlexiGrid(AComponent).OmegaScreen := self;
  end;
end;

procedure TOmegaScreen.CopyVertices(pVertices: PByte; numVertices: integer);
var pVB: PByte;

procedure _Modify(CenterX, CenterY, Angle,
 ScaleX, ScaleY, OfsX, OfsY: single; c: TOmegaColor);
var a: integer;
    q, s1, s2, s3, s4: single;
    p: POmegaVertex;
const afac = pi/180;
begin
 q  := Angle * afac;
 s1 := ScaleX * cos(q);
 s2 := ScaleX * sin(q);
 s3 := - ScaleY * sin(q);
 s4 := ScaleY * cos(q);
 p := POmegaVertex(pVertices);
 for a := 0 to numVertices-1 do
 begin
  with p^ do
  begin
{
   Color.Red := round(Color.Red*C.Red/255);
   Color.Green := round(Color.Green*C.Green/255);
   Color.Blue := round(Color.Blue*C.Blue/255);
   Color.Alpha := round(Color.Alpha*C.Alpha/255);
   }
   Color.Red := (Color.Red * C.Red) shr 8;
   Color.Green := (Color.Green * C.Green) shr 8;
   Color.Blue := (Color.Blue * C.Blue) shr 8;
   Color.Alpha := (Color.Alpha * C.Alpha) shr 8;
   q := s1 * x + s3 * y + CenterX - CenterX * s1 - CenterY * s3;
   y := OfsY + s2 * x + s4 * y + CenterY - CenterX * s2 - CenterY * s4;
   x := OfsX + q;
  end;
  inc(p);
 end;
end;

begin
 _Modify(CenterX, CenterY, Rotation, ScaleX, ScaleY, OffsetX, OffsetY, Color);
 Device.SetVertexShader(OmegaVertexDef);
 VB.Lock(0, sizeof(TOmegaVertex) * numVertices, pVB, D3DLOCK_DISCARD);
 Move(pVertices^, pVB^, sizeof(TOmegaVertex) * numVertices);
 VB.Unlock;
 Device.SetStreamSource(0, VB, sizeof(TOmegaVertex));
end;

{
The gamma routines are based on the Microsoft DirectX SDK documentation
(www.msdn.microsoft.com) and PowerDraw v3.0 Pre-Release 3,
written by Lifepower (lifepower@fromru.com), available at
turbo.gamedev.net)
Brightness and Contrast have 128 as default value! Therefore, to
reset the gamma values, call SetGamma(255, 255, 255, 128, 128);
}
procedure TOmegaScreen.SetGamma(Red, Green, Blue, Brightness,
 Contrast: byte);
var k: single;
    k2, i: integer;
begin
 for i := 0 to 255 do
 begin
  FGammaRamp.red[i] := i * (Red + 1);
  FGammaRamp.green[i] := i * (Green + 1);
  FGammaRamp.blue[i] := i * (Blue + 1);
 end;

 with FGammaRamp do
 begin
 k := (Contrast / 128) - 1;
 if (k < 1) then
 for i := 0 to 255 do
 begin
  if (Red[i] > 32767.5) then
   Red[i] := Min(Round(Red[i] + (Red[i] - 32767.5) * k), 65535)
  else Red[i] := Max(Round(Red[i] - (32767.5 - Red[i]) * k), 0);
  if (Green[i] > 32767.5) then
   Green[i]:= Min(Round(Green[i] + (Green[i] - 32767.5) * k), 65535)
  else Green[i] := Max(Round(Green[i] - (32767.5 - Green[i]) * k), 0);
  if (Blue[i] > 32767.5) then
   Blue[i] := Min(Round(Blue[i] + (Blue[i] - 32767.5) * k), 65535)
  else Blue[i] := Max(Round(Blue[i] - (32767.5 - Blue[i]) * k), 0);
 end else
 for i:= 0 to 255 do
 begin
  if (Red[i] > 32767.5) then
   Red[i] := Max(Round(Red[i] - (Red[i] - 32767.5) * k), 32768)
  else Red[i] := Min(Round(Red[i] + (32767.5 - Red[i]) * k), 32768);
  if (Green[i] > 32767.5) then
   Green[i] := Max(Round(Green[i] - (Green[i] - 32767.5) * k), 32768)
  else Green[i] := Min(Round(Green[i] + (32767.5 - Green[i]) * k), 32768);
  if (Blue[i] > 32767.5) then
   Blue[i] := Max(Round(Blue[i] - (Blue[i] - 32767.5) * k), 32768)
  else Blue[i] := Min(Round(Blue[i] + (32767.5 - Blue[i]) * k), 32768);
 end;

 k2 := round(((Brightness / 128) - 1) * 65535);
 if (k2 < 0) then
 for i := 0 to 255 do
 begin
  red[i] := Max(red[i] + k2, 0);
  green[i] := Max(green[i] + k2, 0);
  blue[i] := Max(blue[i] + k2, 0);
 end
 else
 for i := 0 to 255 do
 begin
  red[i] := Min(red[i] + k2, 65535);
  green[i] := Min(green[i] + k2, 65535);
  blue[i] := Min(blue[i] + k2, 65535);
 end;
 end;
 FDevice.SetGammaRamp(0, FGammaRamp);
end;

function OmegaColor(Color: longword): TOmegaColor; overload;
begin
 if Color>$FFFFFF then
 begin
  Result.Blue := Color and $FF;
  Result.Green := (Color shr 8) and $FF;
  Result.Red := (Color shr 16) and $FF;
  Result.Alpha := (Color shr 24) and $FF
 end else
 begin
  Result.Red := Color and $FF;
  Result.Green := (Color shr 8) and $FF;
  Result.Blue := (Color shr 16) and $FF;
  Result.Alpha := 255;
 end;
end;

function OmegaColor(Color: TColor; Alpha: Byte): TOmegaColor; overload;
var RGB: Longint;
begin
 // Convert the color to RGB so we can process it.
 RGB := ColorToRGB(Color);
 Result.Alpha := Alpha;
 // Extract the Red, Green and blue part from the RGB value.
 Result.Red := Byte(RGB);
 Result.Green := RGB shr 8;
 Result.Blue := RGB shr 16;
end;

function OmegaColor(Red, Green, Blue: Byte; Alpha: Byte = 255): TOmegaColor; overload;
begin
 Result.Alpha := Alpha;
 Result.Red := Red;
 Result.Green := Green;
 Result.Blue := Blue;
end;

function OmegaRed: TOmegaColor;
begin
 Result.Alpha := 255;
 Result.Red := 255;
 Result.Green := 0;
 Result.Blue := 0;
end;

function OmegaGreen: TOmegaColor;
begin
 Result.Alpha := 255;
 Result.Red := 0;
 Result.Green := 255;
 Result.Blue := 0;
end;

function OmegaBlue: TOmegaColor;
begin
 Result.Alpha := 255;
 Result.Red := 0;
 Result.Green := 0;
 Result.Blue := 255;
end;

function OmegaBlack: TOmegaColor;
begin
 Result.Alpha := 255;
 Result.Red := 0;
 Result.Green := 0;
 Result.Blue := 0;
end;

function OmegaWhite: TOmegaColor;
begin
 Result.Alpha := 255;
 Result.Red := 255;
 Result.Green := 255;
 Result.Blue := 255;
end;

function OmegaYellow: TOmegaColor;
begin
 Result.Alpha := 255;
 Result.Red := 255;
 Result.Green := 255;
 Result.Blue := 0;
end;

function OmegaMagenta: TOmegaColor;
begin
 Result.Alpha := 255;
 Result.Red := 255;
 Result.Green := 0;
 Result.Blue := 255;
end;

function OmegaCyan: TOmegaColor;
begin
 Result.Alpha := 255;
 Result.Red := 0;
 Result.Green := 255;
 Result.Blue := 255;
end;

function OmegaGray: TOmegaColor;
begin
 Result.Alpha := 255;
 Result.Red := 128;
 Result.Green := 128;
 Result.Blue := 128;
end;

function InvColor(c: TOmegaColor): TOmegaColor; 
begin
 result.Red := 255 - c.Red;
 result.Green := 255 - c.Green;
 result.Blue := 255 - c.Blue;
 result.Alpha := c.Alpha;
end;

procedure TOmegaScreen.SetPixel(x, y: integer; Red, Green, Blue: integer; Alpha: integer=255);
var rect: TRect;
begin
  Rect.Left := x; Rect.Right := x+1;
  Rect.Top := y; Rect.Bottom := y+1;
  FDevice.Clear(1, @rect, D3DCLEAR_TARGET,
   D3DColor_RGBA(red, green, blue, alpha), 1.0, 0);
end;

procedure TOmegaScreen.SetAlpha(const Value: byte);
begin
  FAlpha := Value;
  FColor := OmegaColor(FRed, FGreen, FBlue, FAlpha);
end;

procedure TOmegaScreen.SetBlue(const Value: byte);
begin
  FBlue := Value;
  FColor := OmegaColor(FRed, FGreen, FBlue, FAlpha);
end;

procedure TOmegaScreen.SetColor(const Value: TOmegaColor);
begin
  FColor := Value;
  FRed := FColor.Red;
  FGreen := FColor.Green;
  FBlue := FColor.Blue;
  FAlpha := FColor.Alpha;
end;

procedure TOmegaScreen.SetGreen(const Value: byte);
begin
  FGreen := Value;
  FColor := OmegaColor(FRed, FGreen, FBlue, FAlpha);
end;

procedure TOmegaScreen.SetRed(const Value: byte);
begin
  FRed := Value;
  FColor := OmegaColor(FRed, FGreen, FBlue, FAlpha);
end;

procedure TOmegaScreen.SetFrequency(const Value: Cardinal);
begin
  if not ((Value = 56) or (Value = 60) or (Value = 70) or (Value = 72) or (Value = 75) or
          (Value = 85) or (Value = 90) or (Value = 100) or (Value = 120) or
          (Value = 140) or (Value = 144) or (Value = 150) or (Value = 160) or
          (Value = 170) or (Value = 200) or (Value = 240))
          then FFrequency := D3DPRESENT_RATE_DEFAULT else FFrequency := Value;
end;

procedure TOmegaScreen.SetGrayscale(activate: boolean);
begin
 if activate then
 begin
  FDevice.SetRenderState(D3DRS_TextureFactor,
   Integer(OmegaColor(155,255,155,155)));
  FDevice.SetTextureStageState(0,D3DTSS_COLOROP,D3DTOP_DOTPRODUCT3);
  FDevice.SetTextureStageState(0,D3DTSS_COLORARG2,D3DTA_TFACTOR);
 end
 else
 begin
  FDevice.SetRenderState(D3DRS_TextureFactor,$FFFFFFFF);
  FDevice.SetTextureStageState(0,D3DTSS_COLOROP,D3DTOP_MODULATE);
  FDevice.SetTextureStageState(0,D3DTSS_COLORARG2,D3DTA_DIFFUSE);
 end;
 FGrayscale := activate;
end;

procedure TOmegaScreen.LineH(x1, x2, y: integer; Red, Green, Blue: byte; width: integer = 1);
var rect: TRect;
begin
 rect.Left := x1;
 rect.Top := y;
 rect.Right := x2;
 rect.Bottom := y + width;
 FDevice.Clear(1, @rect, D3DCLEAR_TARGET,
  D3DColor_RGBA(red, green, blue, 255), 1.0, 0);
end;

procedure TOmegaScreen.LineV(x, y1, y2: integer; Red, Green, Blue: byte; width: integer = 1);
var rect: TRect;
begin
 rect.Left := x;
 rect.Top := y1;
 rect.Right := x + width;
 rect.Bottom := y2;
 FDevice.Clear(1, @rect, D3DCLEAR_TARGET,
  D3DColor_RGBA(red, green, blue, 255), 1.0, 0);
end;

function BlendModeString(bm: integer): string;
var bmstr: string;
begin
 case bm of
 bmNormal: bmstr := 'bmNormal';
 bmAdd: bmstr := 'bmAdd';
 bmSrcAdd: bmstr := 'bmSrcAdd';
 bmSrcCol: bmstr := 'bmSrcCol';
 bmSrcColAdd: bmstr := 'bmSrcColAdd';
 bmMultiply: bmstr := 'bmMultiply';
 bmMultiplyAlpha: bmstr := 'bmMultiplyAlpha';
 bmInvMultiply: bmstr := 'bmInvMultiply';
 bmInvMultiplyAlpha: bmstr := 'bmInvMultiplyAlpha';
 bmInvert: bmstr := 'bmInvert';
 bmSrcBright: bmstr := 'bmSrcBright';
 bmDestBright: bmstr := 'bmDestBright';
 bmInvSrcBright: bmstr := 'bmInvSrcBright';
 bmInvDestBright: bmstr := 'bmInvDestBright';
 else
 bmstr := 'bmInvalid';
 end;
 result := bmstr;
end;

end.

