{
//****************************************************************
// OmegaSound3D unit
//****************************************************************

version history:
03/10/04: fixed infinite loop on application exit
          when using MOD files (Tobybear)
02/10/04: adapted unit for latest DX8/DX9 headers (Tobybear)
}

unit OmegaSound3D;
interface

uses
  Windows, SysUtils, Classes, Forms, SyncObjs, ActiveX,
  DirectSound, DirectMusic, ModPlugSDK, OmegaUtils;

type
  TAudioPathListCollection = class;
  TSoundListCollection = class;
  TSoundListItem = class;
  TOmegaSound3D = class;

  TAudioPathType = (apt3D, aptMono, aptStereo);
  TAudioPathListItem = class(TCollectionItem)
  private
    FOmegaSound3D: TOmegaSound3D;
    FName: string;
    FAudioPathType: TAudioPathType;
    FPool: Boolean;
    FChannels: Integer;
    FVolume: Integer;
    FPan: Integer;
    FFrequency: Integer;
    FSoundPosX: Single;
    FSoundPosY: Single;
    FSoundPosZ: Single;
    FAudioPath: IDirectMusicAudioPath8;
    FBuffer: IDirectSoundBuffer8;
    F3DBuffer: IDirectSound3DBuffer8;
    FListener: IDirectSound3DListener8;
    FLastSoundListItem: TSoundListItem;
    FLastSoundTime: Cardinal;
  protected
    procedure SetVolume(Value: Integer);
    function GetVolume: integer;
    procedure SetPan(Value: Integer);
    function GetPan: integer;
    procedure SetFrequency(Value: Integer);
    procedure SetSoundPosX(Value: Single);
    procedure SetSoundPosY(Value: Single);
    procedure SetSoundPosZ(Value: Single);
    procedure SetListenerPos(AX, AY, AZ: Single);
  public
    constructor Create(Collection: TCollection); override;
    procedure Init;
    destructor Destroy; override;
    procedure Play;
    procedure Stop;
    function IsPlaying: Boolean;
    procedure SetSoundPosition(AX, AY, AZ: Single);
    property OmegaSound3D: TOmegaSound3D read FOmegaSound3D;
    property Volume: Integer read GetVolume write SetVolume default 0;
    property Pan: Integer read GetPan write SetPan;
    property SoundPositionX: Single read FSoundPosX write SetSoundPosX;
    property SoundPositionY: Single read FSoundPosY write SetSoundPosY;
    property SoundPositionZ: Single read FSoundPosZ write SetSoundPosZ;

  published
    property Name: string read FName write FName;
    property AudioPathType: TAudioPathType read FAudioPathType write FAudioPathType;
    property Pool: Boolean read FPool write FPool;
    property Channels: Integer read FChannels write FChannels;
    property Frequency: Integer read FFrequency write SetFrequency default 22050;
  end;

  TAudioPathListCollection = class(TOwnedCollection)
  private
    FOmegaSound3D: TOmegaSound3D;
    FLastPoolItem: array[TAudioPathType] of Integer;
  protected
    function GetItem(Index: Integer): TAudioPathListItem;
    procedure SetItem(Index: Integer; Value: TAudioPathListItem);
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TComponent);
    procedure AfterConstruction; override;
    destructor Destroy; override;
    function Add: TAudioPathListItem;
    function Insert(Index: Integer): TAudioPathListItem;
    function Find(AName: string): TAudioPathListItem;
    function FindInPool(AAudioPathType: TAudioPathType): TAudioPathListItem;
    procedure LoadFromFile(const Filename: string);
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToFile(const Filename: string);
    procedure SaveToStream(Stream: TStream);
    property Items[Index: Integer]: TAudioPathListItem read GetItem write SetItem;
    property OmegaSound3D: TOmegaSound3D read FOmegaSound3D;
  end;

  TAudioPathPool = (appUseIndex, app3D, appMono, appStereo);
  TSoundListItem = class(TCollectionItem)
  private
    FOmegaSound3D: TOmegaSound3D;
    //Standard
    FName: string;
    FFileName: string;
    FLoop: Boolean;
    FRepeatCount: DWORD;
    FVolume: Integer;
    FFrequency: Integer;
    FPan: Integer;
    FSoundPosX: Single;
    FSoundPosY: Single;
    FSoundPosZ: Single;
    FAudioPathIndex: Integer;
    FAudioPathPool: TAudioPathPool;
    FSegment: IDirectMusicSegment8;
    FLoaded: Boolean;
    FLastAudioPath: TAudioPathListItem;
  protected
    procedure SetLoop(Value: Boolean);
    procedure SetRepeatCount(Value: DWORD);
    procedure LoadNew(AFileName: string);
    function FindAudioPath: TAudioPathListItem;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Load;
    procedure Unload;
    function Play: TAudioPathListItem;
    procedure Stop;
    function IsPlaying: Boolean;
    procedure SetSoundPosition(AX, AY, AZ: Single);
    procedure SetFileName(const AFileName: string);
    property OmegaSound3D: TOmegaSound3D read FOmegaSound3D;
    property LastAudioPath: TAudioPathListItem read FLastAudioPath;
  published
    property Name: string read FName write FName;
    property FileName: string read FFilename write SetFileName;
    property Loaded: Boolean read FLoaded;
    property Loop: Boolean read FLoop write SetLoop;
    property RepeatCount: DWORD read FRepeatCount write SetRepeatCount;
    property Volume: Integer read FVolume write FVolume;
    property Frequency: Integer read FFrequency write FFrequency;
    property Pan: Integer read FPan write FPan;
    property SoundPositionX: Single read FSoundPosX write FSoundPosX;
    property SoundPositionY: Single read FSoundPosY write FSoundPosY;
    property SoundPositionZ: Single read FSoundPosZ write FSoundPosZ;
    property AudioPathIndex: Integer read FAudioPathIndex write FAudioPathIndex;
    property AudioPathPool: TAudioPathPool read FAudioPathPool write FAudioPathPool;
  end;

  TSoundListCollection = class(TOwnedCollection)
  private
    FOmegaSound3D: TOmegaSound3D;
  protected
    function GetItem(Index: Integer): TSoundListItem;
    procedure SetItem(Index: Integer; Value: TSoundListItem);
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TComponent);
    destructor Destroy; override;
    function Add: TSoundListItem;
    function Insert(Index: Integer): TSoundListItem;
    function Find(AName: string): TSoundListItem;
    procedure LoadFromFile(const Filename: string);
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToFile(const Filename: string);
    procedure SaveToStream(Stream: TStream);
    property Items[Index: Integer]: TSoundListItem read GetItem write SetItem;
    property OmegaSound3D: TOmegaSound3D read FOmegaSound3D;
  end;

  TModPlugEventThread = class(TThread)
  private
    FEventCallback: TThreadMethod;
  public
    procedure Execute; override;
    property EventCallback: TThreadMethod read FEventCallback write FEventCallback;
  end;

  TModListItem = class(TCollectionItem)
  private
    FOmegaSound3D: TOmegaSound3D;
    //Standard
    FName: string;
    FFileName: string;
    FLoop: Boolean;
    FVolume: Integer;
    FLoaded: Boolean;
    //ModPlug
    FModMixer: IModMixer;
    FModPlugBuffer: IDirectSoundBuffer;
    FNotify: IDirectSoundNotify8;
    FNotifyArray: array[0..1] of TDSBPositionNotify;
    FModPlugEventThread: TModPlugEventThread;
    FBufferBytes: Cardinal;
    FCriticalSection: TCriticalSection;
    FRepeatCount: DWORD;
  protected
    procedure SetLoop(Value: Boolean);
    procedure SetRepeatCount(Value: DWORD);
    procedure LoadNew(AFileName: string);
    //ModPlug
    procedure ModPlugThreadCallback;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Load;
    procedure Unload;
    procedure Play;
    procedure Stop;
    function IsPlaying: Boolean;
    procedure SetFileName(const AFileName: string);
    property OmegaSound3D: TOmegaSound3D read FOmegaSound3D;
  published
    property Name: string read FName write FName;
    property FileName: string read FFilename write SetFileName;
    property Loaded: Boolean read FLoaded;
    property Loop: Boolean read FLoop write SetLoop;
    property RepeatCount: DWORD read FRepeatCount write SetRepeatCount;
    property Volume: Integer read FVolume write FVolume;
  end;

  TModListCollection = class(TOwnedCollection)
  private
    FOmegaSound3D: TOmegaSound3D;
  protected
    function GetItem(Index: Integer): TModListItem;
    procedure SetItem(Index: Integer; Value: TModListItem);
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TComponent);
    destructor Destroy; override;
    function Add: TModListItem;
    function Insert(Index: Integer): TModListItem;
    function Find(AName: string): TModListItem;
    procedure LoadFromFile(const Filename: string);
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToFile(const Filename: string);
    procedure SaveToStream(Stream: TStream);
    property Items[Index: Integer]: TModListItem read GetItem write SetItem;
    property OmegaSound3D: TOmegaSound3D read FOmegaSound3D;
  end;

  TOmegaSound3D = class(TComponent)
  private
    FAudioPathList: TAudioPathListCollection;
    FSoundList: TSoundListCollection;
    FModList: TModListCollection;

    FPerformance: IDirectMusicPerformance8;
    FLoader: IDirectMusicLoader8;
    FDirectSound: IDirectSound8;
    FDirectMusic: IDirectMusic8;

    FListenerPosX,
    FListenerPosY,
    FLIstenerPosZ: Single;

    procedure SetAudioPathList(const Value: TAudioPathListCollection);
    procedure SetSoundList(const Value: TSoundListCollection);
    procedure SetModList(const Value: TModListCollection);
  protected
    procedure SetListenerPosX(AX: Single);
    procedure SetListenerPosY(AY: Single);
    procedure SetListenerPosZ(AZ: Single);
    procedure SetListenerPos(AX, AY, AZ: Single);
  public
    constructor Create(AOwner: TComponent); override;
    procedure Init;
    destructor Destroy; override;
    procedure Stop;
    function IsPlaying: Boolean;
  published
    property AudioPathList: TAudioPathListCollection read FAudioPathList write SetAudioPathList;
    property SoundList: TSoundListCollection read FSoundList write SetSoundList;
    property ModList: TModListCollection read FModList write SetModList;
    property ListenerPositionX: Single read FListenerPosX write SetListenerPosX;
    property ListenerPositionY: Single read FListenerPosY write SetListenerPosY;
    property ListenerPositionZ: Single read FListenerPosZ write SetListenerPosZ;
  end;

implementation

uses
  MMSystem, Messages;

{ TAudioPathListItem }

constructor TAudioPathListItem.Create(Collection: TCollection);
begin
  inherited;

  FOmegaSound3D := TAudioPathListCollection(Collection).FOmegaSound3D;
  FAudioPathType := apt3D;
  FPool := False;
  FChannels := 16;
  FFrequency := 22050;
  FVolume := 0;

  FAudioPath := nil;
  FBuffer := nil;
  F3DBuffer := nil;
  FListener := nil;
  FLastSoundListItem := nil;
  FLastSoundTime := 0;
end;

procedure TAudioPathListItem.Init;
var
  apt: Cardinal;
begin
  case FAudioPathType of
    apt3D: apt := DMUS_APATH_DYNAMIC_3D;
    aptMono: apt := DMUS_APATH_DYNAMIC_MONO;
  else
    apt := DMUS_APATH_DYNAMIC_STEREO;
  end;
  //AudioPath
  if Failed(FOmegaSound3D.FPerformance.CreateStandardAudioPath(apt, FChannels, True, FAudioPath)) then
    raise Exception.Create('Failed to create standard audio path. DirectX 8 may not be installed');
  //Buffer
  if Failed(FAudioPath.GetObjectInPath(DMUS_PCHANNEL_ALL, DMUS_PATH_BUFFER, 0, GUID_NULL, 0, IID_IDirectSoundBuffer, FBuffer)) then
    raise Exception.Create('Failed GetObjectInPath. DirectX 8 may not be installed');
  //3DBuffer
  if FAudioPathType = apt3D then
    if Failed(FAudioPath.GetObjectInPath(DMUS_PCHANNEL_ALL, DMUS_PATH_BUFFER, 0, GUID_NULL, 0, IID_IDirectSound3DBuffer8, F3DBuffer)) then
      raise Exception.Create('Failed GetObjectInPath. DirectX 8 may not be installed');
  //Listener
  if Failed(FAudioPath.GetObjectInPath(0, DMUS_PATH_PRIMARY_BUFFER, 0, GUID_NULL, 0, IID_IDirectSound3DListener8, FListener)) then
    raise Exception.Create('Failed GetObjectInPath. DirectX 8 may not be installed');
end;

destructor TAudioPathListItem.Destroy;
begin
  if not (csDesigning in TSoundListCollection(Collection).FOmegaSound3D.ComponentState) then
  begin
    FOmegaSound3D.FPerformance.StopEx(FAudioPath, 0, 0);
    //Release interfaces
    FAudioPath := nil;
    FBuffer := nil;
    F3DBuffer := nil;
    FListener := nil;
  end;
  inherited;
end;

procedure TAudioPathListItem.SetVolume(Value: Integer);
begin
  if Value > 100 then Value := 100;
  if Value < 0 then Value := 0;
  FVolume := -10000 + Value * 100;
  if (FBuffer <> nil) and (Failed(FBuffer.SetVolume(FVolume))) then
    Exception.Create('Error setting volume');
end;

function TAudioPathListItem.GetVolume: integer;
begin
  Result := 0;
  if FBuffer = nil then
    Result := RoundNormal(FVolume / 100) + 100
  else
  begin
    if Failed(FBuffer.GetVolume(Result)) then
      raise Exception.Create('Error getting volume');
    Result := RoundNormal(Result / 100) + 100;
  end;
end;

procedure TAudioPathListItem.SetPan(Value: Integer);
begin
  if Value > 100 then Value := 100;
  if Value < -100 then Value := -100;
  FPan := Value * 100;
  if (FBuffer <> nil) and (Failed(FBuffer.SetPan(FPan))) then
    Exception.Create('Error setting pan');
end;

function TAudioPathListItem.GetPan: integer;
begin
  Result := 0;
  if FBuffer = nil then Result := RoundNormal(FPan / 100)
  else
  begin
    if Failed(FBuffer.GetPan(Result)) then
      raise Exception.Create('Error getting pan');
    Result := RoundNormal(Result / 100);
  end;
end;

procedure TAudioPathListItem.SetFrequency(Value: Integer);
begin
  if Value < 0 then Value := 0;
  if FFrequency <> Value then
  begin
    FFrequency := Value;
    if (FBuffer <> nil) and (Failed(FBuffer.SetFrequency(FFrequency))) then
      raise Exception.Create('Error setting frequency');
  end;
end;

procedure TAudioPathListItem.SetSoundPosX(Value: Single);
begin
  if FSoundPosX <> Value then
    SetSoundPosition(Value, FSoundPosY, FSoundPosZ);
end;

procedure TAudioPathListItem.SetSoundPosY(Value: Single);
begin
  if FSoundPosY <> Value then
    SetSoundPosition(FSoundPosX, Value, FSoundPosZ);
end;

procedure TAudioPathListItem.SetSoundPosZ(Value: Single);
begin
  if FSoundPosZ <> Value then
    SetSoundPosition(FSoundPosX, FSoundPosY, Value);
end;

procedure TAudioPathListItem.SetSoundPosition(AX, AY, AZ: Single);
begin
  FSoundPosX := AX;
  FSoundPosY := AY;
  FSoundPosZ := AZ;
  if (F3DBuffer <> nil) and (Failed(F3DBuffer.SetPosition(AX, AY, AZ, DS3D_IMMEDIATE))) then
    raise Exception.Create('Error setting sound position');
end;

procedure TAudioPathListItem.SetListenerPos(AX, AY, AZ: Single);
begin
  if (FListener <> nil) and (Failed(FListener.SetPosition(AX, AY, AZ, DS3D_IMMEDIATE))) then
    raise Exception.Create('Error setting listener position');
end;

procedure TAudioPathListItem.Play;
begin
  if (FBuffer <> nil) and (Failed(FBuffer.Play(0, 0, 0))) then
    raise Exception.Create('Error playing AudioPath');
end;

procedure TAudioPathListItem.Stop;
begin
  if (FBuffer <> nil) and (Failed(FOmegaSound3D.FPerformance.StopEx(FAudioPath, 0, 0))) then
    raise Exception.Create('Error stopping AudioPath');
end;

function TAudioPathListItem.IsPlaying: Boolean;
begin
  Result :=
    ((FLastSoundListItem <> nil) and (FLastSoundListItem.IsPlaying));
  if not Result then FLastSoundListItem := nil;
end;

{ TAudioPathListCollection }

type //streaming support
  TAudioPathListCollectionComponent = class(TComponent)
  private
    FList: TAudioPathListCollection;
  published
    property List: TAudioPathListCollection read FList write FList;
  end;

constructor TAudioPathListCollection.Create(AOwner: TComponent);
begin
  inherited Create(AOwner, TAudioPathListItem);
  FOmegaSound3D := TOmegaSound3D(AOwner);
  FLastPoolItem[apt3D] := 0;
  FLastPoolItem[aptStereo] := 0;
  FLastPoolItem[aptMono] := 0;
end;

procedure TAudioPathListCollection.AfterConstruction;
begin
  inherited;
end;

destructor TAudioPathListCollection.Destroy;
begin
  inherited;
end;

function TAudioPathListCollection.GetItem(Index: Integer): TAudioPathListItem;
begin
  Result := TAudioPathListItem(inherited GetItem(Index));
end;

procedure TAudioPathListCollection.SetItem(Index: Integer; Value: TAudioPathListItem);
begin
  inherited SetItem(Index, Value);
end;

procedure TAudioPathListCollection.Update(Item: TCollectionItem);
begin
  inherited Update(Item);
end;

function TAudioPathListCollection.Add: TAudioPathListItem;
begin
  Result := TAudioPathListItem(inherited Add);
end;

function TAudioPathListCollection.Insert(Index: Integer): TAudioPathListItem;
begin
  Result := TAudioPathListItem(inherited Insert(Index));
end;

function TAudioPathListCollection.Find(AName: string): TAudioPathListItem;
var
  i: Integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do begin
    if AName = Items[i].FName then begin
      Result := Items[i];
      Break;
    end;
  end;
end;

function TAudioPathListCollection.FindInPool(AAudioPathType: TAudioPathType): TAudioPathListItem;
var
  i, iOffset, iIndex: Integer;
  ap: TAudioPathListItem;
begin
  Result := nil;
  iOffset := FLastPoolItem[AAudioPathType];
  for i := 1 to Count do
  begin
    iIndex := (iOffset + i) mod Count;
    ap := Items[iIndex];
    if (ap.FPool) and (ap.AudioPathType = AAudioPathType) then
    begin
      Result := ap;
      if not ap.IsPlaying then
      begin
        FLastPoolItem[AAudioPathType] := iOffset + i mod Count;
        Break;
      end;
    end;
  end;
end;

procedure TAudioPathListCollection.LoadFromFile(const Filename: string); 
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(Filename, fmOpenRead or fmShareDenyWrite); 
  try 
    LoadFromStream(Stream); 
  finally 
    Stream.Free; 
  end; 
end; 
 
procedure TAudioPathListCollection.LoadFromStream(Stream: TStream); 
var 
  Component: TAudioPathListCollectionComponent; 
begin 
  Clear; 
  Component := TAudioPathListCollectionComponent.Create(nil); 
  try 
    Component.FList := self; 
    Stream.ReadComponentRes(Component); 
  finally
    Component.Free; 
  end; 
end; 
 
procedure TAudioPathListCollection.SaveToFile(const Filename: string); 
var 
  Stream: TFileStream; 
begin 
  Stream := TFileStream.Create(Filename, fmCreate); 
  try 
    SaveToStream(Stream); 
  finally 
    Stream.Free; 
  end; 
end; 
 
procedure TAudioPathListCollection.SaveToStream(Stream: TStream); 
var
  Component: TAudioPathListCollectionComponent; 
begin 
  Component := TAudioPathListCollectionComponent.Create(nil); 
  try 
    Component.FList := self; 
    Stream.WriteComponentRes('TAudioPathListCollection', Component); 
  finally 
    Component.Free; 
  end; 
end; 

{ TModPlugEventThread }

procedure TModPlugEventThread.Execute;
begin
  if not Assigned(EventCallback) then Terminate;
  while not Terminated do
    EventCallback;
exit;
 if not Assigned(EventCallback) then Terminate;
 while (not Terminated) do EventCallback;
end;

{ TSoundListItem }

constructor TSoundListItem.Create(Collection: TCollection);
begin
  inherited;
  FAudioPathIndex := 0;
  FAudioPathPool := appUseIndex;
  FVolume := 100;
  FPan := 0;
  FOmegaSound3D := TSoundListCollection(Collection).FOmegaSound3D;
  FLoaded := False;
end;

destructor TSoundListItem.Destroy;
var
  i: Integer;
begin
  //Make sure no AudioPath remembers us
  with FOmegaSound3D.AudioPathList do
    for i := Count-1 downto 0 do
      if Items[i].FLastSoundListItem = Self then
        Items[i].FLastSoundListItem := nil;
  if not (csDesigning in TSoundListCollection(Collection).FOmegaSound3D.ComponentState) then
    Unload;
  inherited;
end;

function TSoundListItem.FindAudioPath: TAudioPathListItem;
begin
  case FAudioPathPool of
    app3D: Result := FOmegaSound3D.FAudioPathList.FindInPool(apt3D);
    appMono: Result := FOmegaSound3D.FAudioPathList.FindInPool(aptMono);
    appStereo: Result := FOmegaSound3D.FAudioPathList.FindInPool(aptStereo);
  else
    Result := FOmegaSound3D.FAudioPathList.Items[FAudioPathIndex];
  end;
end;

function TSoundListItem.Play: TAudioPathListItem;
var s: idirectmusicsegmentstate;//t
begin
  if not FLoaded then Load;
  Result := FindAudioPath;
  if Result <> nil then
  begin
    Result.SetVolume(Volume);//FVolume/2?
    //Result.SetPan(FPan);
    Result.SetFrequency(Frequency);
    Result.SetSoundPosition(FSoundPosX, FSoundPosY, FSoundPosZ);
    if Failed(FOmegaSound3D.FPerformance.PlaySegmentEx(FSegment, nil, nil, DMUS_SEGF_SECONDARY, 0, s, nil, Result.FAudioPath)) then
//t    if Failed(FOmegaSound3D.FPerformance.PlaySegmentEx(FSegment, nil, nil, DMUS_SEGF_SECONDARY, 0, nil, nil, Result.FAudioPath)) then
      raise Exception.Create('Error playing sound');
    Result.FLastSoundListItem := Self;
    Result.FLastSoundTime := timeGetTime;
  end;
  FLastAudioPath := Result;
end;

procedure TSoundListItem.Stop;
var
  i: Integer;
begin
  if not FLoaded then Exit;
  if Failed(FOmegaSound3D.FPerformance.Stop(FSegment, nil, 0, 0)) then
    raise Exception.Create('Error stopping sound');
  //Make sure no audiopath remembers us
  with FOmegaSound3D.AudioPathList do
    for i := Count-1 downto 0 do
      if Items[i].FLastSoundListItem = Self then
        Items[i].FLastSoundListItem := nil;
end;

function TSoundListItem.IsPlaying: Boolean;
begin
//  Result := FLoaded and (FSegment <> nil) and (FOmegaSound3D.FPerformance.IsPlaying(FSegment, nil) = S_OK);
  Result :=
    FLoaded and
    (FSegment <> nil) and
    (FLastAudioPath <> nil) and
    (FLastAudioPath.FLastSoundListItem = Self) and
    ((timeGetTime - FLastAudioPath.FLastSoundTime < 250) or
    (FOmegaSound3D.FPerformance.IsPlaying(FSegment, nil) = S_OK));
end;

procedure TSoundListItem.Load;
begin
  if csDesigning in TSoundListCollection(Collection).FOmegaSound3D.ComponentState then Exit;
  //Load segment (disabling caching for segments by loader, else sounditems using the same file could share their Segment!)
  FOmegaSound3D.FLoader.ClearCache(CLSID_DirectMusicSegment);
  if Failed(FOmegaSound3D.FLoader.LoadObjectFromFile(CLSID_DirectMusicSegment, IID_IDirectMusicSegment, PWideChar(WideString(FFileName)), FSegment)) then
    raise Exception.Create('Error loading sound: ' + FFileName);
  FLoaded := True;
  //Download segment to DirectMusic synthesizer
  if Failed(FSegment.Download(FOmegaSound3D.FPerformance)) then
    raise Exception.Create('Error downloading sound');
  //Set initial values
  if FLoop then
  begin
    if Failed(FSegment.SetRepeats(DMUS_SEG_REPEAT_INFINITE)) then
      raise Exception.Create('Error looping sound');
  end
  else
  begin
    if Failed(FSegment.SetRepeats(FRepeatCount)) then
      //raise Exception.Create('Error setting number of repeat counts');
  end;
end;

procedure TSoundListItem.Unload;
begin
  if not FLoaded then Exit;
  //Stop all playing instances to be sure
  Stop;
  //Unload segment from synthesizer
  FSegment.Unload(FOmegaSound3D.FPerformance);
  //Release segment from Loader's cache
  FOmegaSound3D.FLoader.ReleaseObjectByUnknown(FSegment);
  FSegment := nil;
  FLoaded := False;
end;

procedure TSoundListItem.LoadNew(AFileName: string);
begin
  Unload;
  FFileName := AFileName;
  Load;
end;

procedure TSoundListItem.SetLoop(Value: Boolean);
begin
  if FLoop = Value then Exit;
  FLoop := Value;
  if not FLoaded then Exit;
  if FLoop then
  begin
    if Failed(FSegment.SetRepeats(DMUS_SEG_REPEAT_INFINITE)) then
      raise Exception.Create('Error looping sound');
  end else
  begin
    if Failed(FSegment.SetRepeats(FRepeatCount)) then
      raise Exception.Create('Error setting number of repeat counts');
  end;
end;

procedure TSoundListItem.SetRepeatCount(Value: DWORD);
begin
  if FRepeatCount = Value then Exit;
  FRepeatCount := Value;
  if not FLoaded then Exit;
  if (not FLoop) and (Failed(FSegment.SetRepeats(FRepeatCount))) then
    raise Exception.Create('Error setting number of repeat counts');
end;

procedure TSoundListItem.SetSoundPosition(AX, AY, AZ: Single);
begin
  FSoundPosX := AX;
  FSoundPosY := AY;
  FSoundPosZ := AZ;
end;

procedure TSoundListItem.SetFileName(const AFileName: string);
begin
  if AFileName = FFileName then Exit;
  FFileName := AFileName;
end;

{ TSoundListCollection }

type
  TSoundListCollectionComponent = class(TComponent)
  private
    FList: TSoundListCollection;
  published
    property List: TSoundListCollection read FList write FList;
  end;

constructor TSoundListCollection.Create(AOwner: TComponent);
begin
  inherited Create(AOwner, TSoundListItem);
  if not (AOwner is TOmegaSound3D) then
    raise Exception.Create('TSoundListCollection may only be created with TOmegaSound3D as owner');
  FOmegaSound3D := TOmegaSound3D(AOwner);
end;

destructor TSoundListCollection.Destroy;
begin
  inherited;
end;

function TSoundListCollection.GetItem(Index: Integer): TSoundListItem;
begin
  Result := TSoundListItem(inherited GetItem(Index));
end;

procedure TSoundListCollection.SetItem(Index: Integer; Value: TSoundListItem);
begin
  inherited SetItem(Index, Value);
end;

procedure TSoundListCollection.Update(Item: TCollectionItem);
begin
  inherited Update(Item);
end;

function TSoundListCollection.Add: TSoundListItem;
begin
  Result := TSoundListItem(inherited Add);
end;

function TSoundListCollection.Insert(Index: Integer): TSoundListItem;
begin
  Result := TSoundListItem(inherited Insert(Index));
end;

function TSoundListCollection.Find(AName: string): TSoundListItem;
var
  i: Integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do begin
    if AName = Items[i].FName then begin
      Result := Items[i];
      Break;
    end;
  end;
end;

procedure TSoundListCollection.LoadFromFile(const Filename: string);
var  
  Stream: TFileStream; 
begin 
  Stream := TFileStream.Create(Filename, fmOpenRead or fmShareDenyWrite); 
  try 
    LoadFromStream(Stream); 
  finally 
    Stream.Free; 
  end; 
end; 
 
procedure TSoundListCollection.LoadFromStream(Stream: TStream); 
var 
  Component: TSoundListCollectionComponent; 
begin 
  Clear; 
  Component := TSoundListCollectionComponent.Create(nil); 
  try
    Component.FList := self; 
    Stream.ReadComponentRes(Component); 
  finally 
    Component.Free; 
  end; 
end; 
 
procedure TSoundListCollection.SaveToFile(const Filename: string); 
var 
  Stream: TFileStream; 
begin 
  Stream := TFileStream.Create(Filename, fmCreate); 
  try 
    SaveToStream(Stream); 
  finally 
    Stream.Free; 
  end; 
end;
 
procedure TSoundListCollection.SaveToStream(Stream: TStream); 
var 
  Component: TSoundListCollectionComponent; 
begin 
  Component := TSoundListCollectionComponent.Create(nil); 
  try 
    Component.FList := self; 
    Stream.WriteComponentRes('TSoundListCollection', Component);
  finally 
    Component.Free; 
  end; 
end;

{ TModListItem }

constructor TModListItem.Create(Collection: TCollection);
begin
  inherited;
  FVolume := 100;
  FModMixer := nil;
  FNotify := nil;
  if csDesigning in TSoundListCollection(Collection).FOmegaSound3D.ComponentState then Exit;
  FOmegaSound3D := TSoundListCollection(Collection).FOmegaSound3D;
  FLoaded := False;
  FCriticalSection := TCriticalSection.Create;
end;

destructor TModListItem.Destroy;
begin
  if not (csDesigning in TSoundListCollection(Collection).FOmegaSound3D.ComponentState) then
  begin
    Unload;
    FCriticalSection.Free;
  end;
  inherited;
end;

procedure TModListItem.Play;
var
  Flags: Cardinal;
  W1, W2: pointer;
  S1, S2: Cardinal;
begin
  if not FLoaded then Load;

  //Set loop
  Flags := FModMixer.GetMixerOptions;
  if FLoop then Flags := Flags or MPPMIX_LOOP
  else Flags := Flags and (not MPPMIX_LOOP);
  FModMixer.SetMixerOptions(Flags);

  //Clear soundbuffer
  if FModPlugBuffer.Lock(0, 0, w1, s1, w2, s2, DSBLOCK_ENTIREBUFFER) = 0 then
//t  if FModPlugBuffer.Lock(0, 0, @w1, @s1, @w2, @s2, DSBLOCK_ENTIREBUFFER) = 0 then
//  if FModPlugBuffer.Lock(0, 0, W1, S1, W2, S2, DSBLOCK_ENTIREBUFFER) = 0 then
  begin
    FillMemory(W1, S1, 0);
    if W2 <> nil then FillMemory(W2, S2, 0);
    FModPlugBuffer.Unlock(W1, S1, W2, S2);
  end;

  //Start ModPlug thread
  FModPlugEventThread.Resume;

  //Start playing DirectSound buffer
  FModPlugBuffer.Play(0, 0, DSBPLAY_LOOPING);
end;

procedure TModListItem.Stop;
begin
  if not FLoaded then Exit;
  try
    if Failed(FModPlugBuffer.Stop) then
      raise Exception.Create('Error stopping .mod sound buffer');
  finally
    FModPlugEventThread.Suspend;
  end;
end;

function TModListItem.IsPlaying: Boolean;
begin
  Result := False;
  if not FLoaded then Exit;
  Result := True;
end;

procedure TModListItem.Load;
var
  Stream: TMemoryStream;
  WaveFormat: TWaveFormatEx;
  Index: Integer; Offset: Integer;
  BufferDesc: TDSBufferDesc;
begin
  if csDesigning in TSoundListCollection(Collection).FOmegaSound3D.ComponentState then Exit;

  if not InitModPlugSDKLibrary(FModMixer) then
    raise Exception.Create('Error initializing ModPlug library, '+
    'please make sure that mppsdk.dll is somewhere in your path!');

  //Create seperate audio buffer for .mod
  WaveFormat.wFormatTag := WAVE_FORMAT_PCM;
  WaveFormat.nChannels := 2;
  WaveFormat.nSamplesPerSec := 22050;
  WaveFormat.wBitsPerSample := 16;
  WaveFormat.nBlockAlign := (WaveFormat.wBitsPerSample div 8) * WaveFormat.nChannels;
  WaveFormat.nAvgBytesPerSec := WaveFormat.nSamplesPerSec * WaveFormat.nBlockAlign;

  ZeroMemory(@BufferDesc, Sizeof(TDSBufferDesc));
  BufferDesc.dwSize := Sizeof(TDSBufferDesc);
  BufferDesc.dwFlags := DSBCAPS_GLOBALFOCUS or DSBCAPS_CTRLPAN or DSBCAPS_CTRLVOLUME or DSBCAPS_CTRLFREQUENCY
    or DSBCAPS_GETCURRENTPOSITION2 or DSBCAPS_CTRLPOSITIONNOTIFY or DSBCAPS_LOCSOFTWARE; //!
  BufferDesc.dwBufferBytes := WaveFormat.nAvgBytesPerSec * 2;
  BufferDesc.lpwfxFormat := @WaveFormat;
  BufferDesc.guid3DAlgorithm := DS3DALG_DEFAULT;

  FOmegaSound3D.FDirectSound.CreateSoundBuffer(BufferDesc, FModPlugBuffer, nil);

  //Set ModPlug mix output to DirectSoundBuffer's wave format
  FModMixer.SetWaveFormat(WaveFormat.nSamplesPerSec, WaveFormat.nChannels, WaveFormat.wBitsPerSample);
  FBufferBytes := BufferDesc.dwBufferBytes;

  //Create IDirectSoundNotify environment
  if Failed(FModPlugBuffer.QueryInterface(IID_IDirectSoundNotify, FNotify)) then
    raise Exception.Create('Error querying for Notify interface');
  Offset := 0;
  for Index := 0 to 1 do
  begin
    FNotifyArray[Index].dwOffset := OffSet;
    FNotifyArray[Index].hEventNotify := CreateEvent(nil, True, False, nil);
    Inc(Offset, WaveFormat.nAvgBytesPerSec);
  end;
  if FNotify.SetNotificationPositions(2, @(FNotifyArray[0])) <> 0 then
    raise Exception.Create('Error creating Notify environment');

  //Create EventThread
  if not Assigned(FModPlugEventThread) then
  begin
    FModPlugEventThread := TModPlugEventThread.Create(True);
    FModPlugEventThread.Priority := tpNormal;
    FModPlugEventThread.EventCallback := ModPlugThreadCallback;
  end;

  //Load .mod into memory
  Stream := TMemoryStream.Create;
  try
    Stream.LoadFromFile(FFileName);
    FModMixer.LoadSong(Stream.Memory, Stream.Size);
    FLoaded := True;
  finally
    Stream.Free;
  end;
end;

procedure TModListItem.Unload;
begin
  if not FLoaded then Exit;
  if FModMixer = nil then Exit;
  Stop;

  //Free ModPlug thread
  if (Assigned(FModPlugEventThread)) then
  begin
     FModPlugEventThread.Terminate;
     FModPlugEventThread.Free;
     FModPlugEventThread := nil;
  end;

  //Free Notify environment
  FNotify := nil;
  //Release .mod from memory
  FModMixer.FreeSong;
  //Release library
  DeInitModPlugSDKLibrary(FModMixer);

  FLoaded := False;
end;

procedure TModListItem.LoadNew(AFileName: string);
begin
  Unload;
  FFileName := AFileName;
  Load;
end;

procedure TModListItem.SetLoop(Value: Boolean);
begin
  if FLoop = Value then Exit;
  FLoop := Value;
end;

procedure TModListItem.SetRepeatCount(Value: DWORD);
begin
  if FRepeatCount = Value then Exit;
  FRepeatCount := Value;
  if not FLoaded then Exit;
end;

procedure TModListItem.SetFileName(const AFileName: string);
begin
  if AFileName = FFileName then Exit;
  FFileName := AFileName;
end;

procedure TModListItem.ModPlugThreadCallback;
var
  Msg: TMsg;
  CurrentEvent: Integer;
  Events: array[0..1] of Cardinal;

  procedure ReadData(Event: Integer);
  var
    W1, W2: pointer;
    S1, S2: Cardinal;
    p1,p2:pdword;
    NumWrite: Longint;
    Pos: Cardinal;
    Written1: Integer;
  begin
    if Event = 1 then
    begin
      Pos := FNotifyArray[0].dwOffset;
      NumWrite := FNotifyArray[1].dwOffset - Pos;
    end
    else
    begin
      Pos := FNotifyArray[1].dwOffset;
      NumWrite := FNotifyArray[0].dwOffset - Pos;
    end;
    if (NumWrite < 0) then Inc(NumWrite, FBufferBytes);

    if FModPlugBuffer.Lock(Pos, NumWrite, W1, S1, W2, S2, 0) = 0 then
//t    if FModPlugBuffer.Lock(Pos, NumWrite, @W1, @S1, @W2, @S2, 0) = 0 then
//    if FModPlugBuffer.Lock(Pos, NumWrite, W1, S1, W2, S2, 0) = 0 then
    begin
      FCriticalSection.Acquire;
      Written1 := FModMixer.Render(W1, S1);
      if W2 <> nil then Written1 := FModMixer.Render(W2, S2);
      FCriticalSection.Release;
      FModPlugBuffer.Unlock(W1, S1, W2, S2);
      if (Written1 = 0) then Stop;
    end;
  end;

begin
  Events[0] := FNotifyArray[0].hEventNotify;
  Events[1] := FNotifyArray[1].hEventNotify;
 // Toby's changes: (500 ms)
  CurrentEvent := MsgWaitForMultipleObjects(2, Events[0],
   False, 500, QS_ALLINPUT);
  Dec(CurrentEvent, WAIT_OBJECT_0);
  if CurrentEvent > 1 then
  begin
    while (PeekMessage(Msg, 0, 0, 0, PM_REMOVE)) do
    begin
      if Msg.Message = WM_QUIT then Stop else
      begin
        TranslateMessage(Msg);
        DispatchMessage(Msg);
      end;
    end;
  end
  else                  
  begin
    ResetEvent(Events[CurrentEvent]);
    ReadData(CurrentEvent);
  end;
end;

{ TModListCollection }

type
  TModListCollectionComponent = class(TComponent)
  private
    FList: TModListCollection;
  published
    property List: TModListCollection read FList write FList;
  end;

constructor TModListCollection.Create(AOwner: TComponent);
begin
  inherited Create(AOwner, TModListItem);
  if not (AOwner is TOmegaSound3D) then
    raise Exception.Create('TModListCollection may only be created with TOmegaSound3D as owner');
  FOmegaSound3D := TOmegaSound3D(AOwner);
end;

destructor TModListCollection.Destroy;
begin
  inherited;
end;

function TModListCollection.GetItem(Index: Integer): TModListItem;
begin
  Result := TModListItem(inherited GetItem(Index));
end;

procedure TModListCollection.SetItem(Index: Integer; Value: TModListItem);
begin
  inherited SetItem(Index, Value);
end;

procedure TModListCollection.Update(Item: TCollectionItem);
begin
  inherited Update(Item);
end;

function TModListCollection.Add: TModListItem;
begin
  Result := TModListItem(inherited Add);
end;

function TModListCollection.Insert(Index: Integer): TModListItem;
begin
  Result := TModListItem(inherited Insert(Index));
end;

function TModListCollection.Find(AName: string): TModListItem;
var
  i: Integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do begin
    if AName = Items[i].FName then begin
      Result := Items[i];
      Break;
    end;
  end;
end;

procedure TModListCollection.LoadFromFile(const Filename: string); 
var  
  Stream: TFileStream; 
begin 
  Stream := TFileStream.Create(Filename, fmOpenRead or fmShareDenyWrite); 
  try 
    LoadFromStream(Stream); 
  finally 
    Stream.Free; 
  end; 
end; 
 
procedure TModListCollection.LoadFromStream(Stream: TStream); 
var 
  Component: TModListCollectionComponent; 
begin 
  Clear; 
  Component := TModListCollectionComponent.Create(nil); 
  try 
    Component.FList := self; 
    Stream.ReadComponentRes(Component); 
  finally 
    Component.Free; 
  end; 
end; 
 
procedure TModListCollection.SaveToFile(const Filename: string); 
var 
  Stream: TFileStream; 
begin 
  Stream := TFileStream.Create(Filename, fmCreate); 
  try 
    SaveToStream(Stream); 
  finally 
    Stream.Free; 
  end; 
end; 
 
procedure TModListCollection.SaveToStream(Stream: TStream); 
var 
  Component: TModListCollectionComponent;
begin 
  Component := TModListCollectionComponent.Create(nil); 
  try 
    Component.FList := self; 
    Stream.WriteComponentRes('TModListCollection', Component);
  finally 
    Component.Free; 
  end; 
end; 

{ TOmegaSound3D }

constructor TOmegaSound3D.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FLoader := nil;
  FPerformance := nil;
  FDirectSound := nil;
  FDirectMusic := nil;

  if not (csDesigning in ComponentState) then begin

    //Initialise COM
    if Failed(CoInitialize(nil)) then
      raise Exception.Create('Failed to initialise COM');

    //Loader
    if Failed(CoCreateInstance(CLSID_DirectMusicLoader, nil, CLSCTX_INPROC, IID_IDirectMusicLoader8, FLoader)) then
      raise Exception.Create('Failed to initialise DirectX Audio. DirectX 8 may not be installed');
    //Performance
    if Failed(CoCreateInstance(CLSID_DirectMusicPerformance, nil, CLSCTX_INPROC, IID_IDirectMusicPerformance8, FPerformance)) then
      raise Exception.Create('Failed to initialise DirectMusic. DirectX 8 may not be installed');
    //InitAudio without default path!             //DMUS_APATH_DYNAMIC_STEREO, 32
    if Failed(FPerformance.InitAudio(@FDirectMusic, @FDirectSound, 0, 0, 0, DMUS_AUDIOF_ALL or DMUS_AUDIOF_STREAMING, nil)) then
      raise Exception.Create('Failed to initialise DirectX Audio. DirectX 8 may not be installed');

  end;

  FSoundList := TSoundListCollection.Create(Self);
  FAudioPathList := TAudioPathListCollection.Create(Self);
  FModList := TModListCollection.Create(Self);
end;

procedure TOmegaSound3D.Init;
var
  i: Integer;
begin
  for i := 0 to FAudioPathList.Count - 1 do
    FAudioPathList.Items[i].Init;
end;

destructor TOmegaSound3D.Destroy;
begin
  if FAudioPathList <> nil then FAudioPathList.Free;
  if FSoundList <> nil then FSoundList.Free;
  if FModList <> nil then FModList.Free;
  if FDirectMusic <> nil then FDirectMusic := nil;
  if FDirectSound <> nil then FDirectSound := nil;
  if FLoader <> nil then FLoader := nil;
  if FPerformance <> nil then
  begin
    FPerformance.CloseDown;
    FPerformance := nil;
  end;

  if not (csDesigning in ComponentState) then
    //End COM
    CoUninitialize;

  inherited;
end;

procedure TOmegaSound3D.SetAudioPathList(const Value: TAudioPathListCollection);
begin
  FAudioPathList.Assign(Value);
end;

procedure TOmegaSound3D.SetSoundList(const Value: TSoundListCollection);
begin
  FSoundList.Assign(Value);
end;

procedure TOmegaSound3D.SetModList(const Value: TModListCollection);
begin
  FModList.Assign(Value);
end;

procedure TOmegaSound3D.SetListenerPosX(AX: Single);
begin
  if FListenerPosX <> AX then
  begin
    FListenerPosX := AX;
    SetListenerPos(FListenerPosX, FListenerPosY, FListenerPosZ);
  end;
end;

procedure TOmegaSound3D.SetListenerPosY(AY: Single);
begin
  if FListenerPosY <> AY then
  begin
    FListenerPosY := AY;
    SetListenerPos(FListenerPosX, FListenerPosY, FListenerPosZ);
  end;
end;

procedure TOmegaSound3D.SetListenerPosZ(AZ: Single);
begin
  if FListenerPosZ <> AZ then
  begin
    FListenerPosZ := AZ;
    SetListenerPos(FListenerPosX, FListenerPosY, FListenerPosZ);
  end;
end;

procedure TOmegaSound3D.SetListenerPos(AX, AY, AZ: Single);
var
  i: Integer;
begin
  for i := 0 to FAudioPathList.Count - 1 do
    FAudioPathList.Items[i].SetListenerPos(AX, AY, AZ);
end;

procedure TOmegaSound3D.Stop;
var
  i: Integer;
begin
  for i := FAudioPathList.Count - 1 downto 0 do
    FAudioPathList.Items[i].Stop;
  for i := FModList.Count - 1 downto 0 do
    FModList.Items[i].Stop;
end;

function TOmegaSound3D.IsPlaying: Boolean;
var
  i: Integer;
begin
  Result := False;
  for i := FSoundList.Count - 1 downto 0 do
  begin
    if FSoundList.Items[i].IsPlaying then
    begin
      Result := True;
      Break;
    end;
  end;
  if not Result then
  begin
    for i := FModList.Count - 1 downto 0 do
    begin
      if FModList.Items[i].IsPlaying then
      begin
        Result := True;
        Break;
      end;
    end;
  end;
end;

end.
