unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, FolderBrowser;

type
  TfrmMain = class(TForm)
    memoLog: TMemo;
    btGo: TButton;
    dlgFolder: TFolderBrowser;
    edOmega: TEdit;
    btOmega: TSpeedButton;
    Label1: TLabel;
    Label2: TLabel;
    edDirectX: TEdit;
    btDirectX: TSpeedButton;
    btGoForReal: TButton;
    procedure btGoClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btOmegaClick(Sender: TObject);
    procedure btDirectXClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btGoForRealClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure Log(const Msg: String);
    function FixDelphiPath(const Path, DelphiPath: String): String;
    function ValidateOmegaPath(const Path: String): Boolean;
    function ValidateDirectXPath(const Path: String): Boolean;
    procedure Prepare(const bForReal: Boolean);
    procedure PrepareDelphi(const sDelphiKey, sDelphiPath: String; const bForReal: Boolean);
  end;

var
  frmMain: TfrmMain;
  bDelphiInstalled: array[5..7] of Boolean;
  sDelphiPath: array[5..7] of String;
  sDelphiKey: array[5..7] of String;

implementation

{$R *.DFM}

uses
  Registry, IniFiles;

procedure TfrmMain.Log(const Msg: String);
begin
  memoLog.Lines.Add(Msg);
end;

function TfrmMain.FixDelphiPath(const Path, DelphiPath: String): String;
begin
  Result := StringReplace(Path, '$(DELPHI)', DelphiPath, [rfIgnoreCase]);
end;

procedure TfrmMain.FormCreate(Sender: TObject);
var
  iDelphi: Integer;
  ini: TIniFile;
begin
  //Predefine registry keys
  for iDelphi := 5 to 7 do
    sDelphiKey[iDelphi] := 'SOFTWARE\Borland\Delphi\' + inttostr(iDelphi) + '.0';
  //Get possible settings from .ini file
  ini := TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  try
    edOmega.Text   := ini.ReadString('Paths', 'Omega',   ExtractFilePath(Application.ExeName));
    edDirectX.Text := ini.ReadString('Paths', 'DirectX', ExtractFilePath(Application.ExeName) + 'directx8');
  finally
    ini.Free;
  end;
end;

procedure TfrmMain.btGoClick(Sender: TObject);
begin
  Prepare(False);
end;

procedure TfrmMain.btGoForRealClick(Sender: TObject);
var
  sWindowsPath: String[255];
  i: Integer;
  s: String;
  ss: String;
  src, dst: String;
begin
  Prepare(True);

  ss := ExtractFilePath(Application.ExeName) + 'DLLs\';
  if not FileExists(ss + 'D3DX81ab.dll') then Exit;
  if MessageDlg('Do you want to copy the needed DLLs to your System32 directory now?', mtConfirmation, [mbYes, mbNo], 0) = mrNo then
    Exit;

  i := GetWindowsDirectory(@(sWindowsPath[1]), 255);
  s := IncludeTrailingBackslash(Copy(sWindowsPath, 1, i));

  src := ss + 'D3DX81ab.dll';
  dst := s + 'D3DX81ab.dll';
  CopyFile(PChar(src), PChar(dst), False);
  src := ss + 'MPPSDK.dll';
  dst := s + 'MPPSDK.dll';
  CopyFile(PChar(src), PChar(dst), False);
end;

procedure TfrmMain.Prepare(const bForReal: Boolean);
var
  iDelphi: Integer;
  reg: TRegistry;
begin
  if not ValidateOmegaPath(edOmega.Text) then Exit;
  if not ValidateDirectXPath(edDirectX.Text) then Exit;

  if bForReal then
    if MessageDlg('You are about to change your Delphi set up. We do not guarantee anything from this point on since actually installed packages and paths will be deleted and changed. Are you sure you want to continue?', mtConfirmation, [mbYes, mbNo], 0) = mrNo then
      Exit;

  memoLog.Clear;
  memoLog.Alignment := taLeftJustify;

  Screen.Cursor := crHourGlass;
  reg := TRegistry.Create;
  try
    reg.RootKey := HKEY_LOCAL_MACHINE;

    //Define installed versions and paths of Delphi
    for iDelphi := 5 to 7 do begin
      bDelphiInstalled[iDelphi] := reg.OpenKey(sDelphiKey[iDelphi], False);
      if bDelphiInstalled[iDelphi] then begin
        sDelphiPath[iDelphi] := reg.ReadString('RootDir');
        Log(Format('Delphi %d installation found in %s', [iDelphi, sDelphiPath[iDelphi]]));
        reg.CloseKey;
        PrepareDelphi(sDelphiKey[iDelphi], sDelphiPath[iDelphi], bForReal);
        Log('');
      end;
    end;

    Log('All done.');
  finally
    reg.Free;
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmMain.PrepareDelphi(const sDelphiKey, sDelphiPath: String; const bForReal: Boolean);
var
  reg: TRegistry;
  slNames: TStringList;
  slValues: TStringList;
  i: Integer;
  s: String;
begin
  reg := TRegistry.Create;
  slNames := TStringList.Create;
  slValues := TStringList.Create;
  try
    reg.RootKey := HKEY_CURRENT_USER;

    //Scan known packages for DelphiX
    if reg.OpenKey(sDelphiKey + '\Known Packages', False) then begin
      reg.GetValueNames(slNames);
      for i := slNames.Count-1 downto 0 do begin
        s := reg.ReadString(slNames[i]);
        if Pos('delphix', LowerCase(s)) > 0 then begin
          if bForReal then begin
            Reg.DeleteValue(slNames[i]);
            Log(Format('DelphiX package "%s" found. Removed from Delphi config.', [s]));
          end else
            Log(Format('DelphiX package "%s" found. Would be removed from Delphi.', [s]));
        end;
        if Pos('omega', LowerCase(s)) > 0 then begin
          if bForReal then begin
            Reg.DeleteValue(slNames[i]);
            Log(Format('Omega package "%s" found. Removed from Delphi config.', [s]));
          end else
            Log(Format('Omega package "%s" found. Would be removed from Delphi.', [s]));
        end;
      end;
      reg.CloseKey;
    end;

    //Scan disabled packages for DelphiX
    if reg.OpenKey(sDelphiKey + '\Disabled Packages', False) then begin
      reg.GetValueNames(slNames);
      for i := slNames.Count-1 downto 0 do begin
        s := reg.ReadString(slNames[i]);
        if Pos('delphix', LowerCase(s)) > 0 then begin
          if bForReal then begin
            Reg.DeleteValue(slNames[i]);
            DeleteFile(FixDelphiPath(slNames[i], sDelphiPath));
            Log(Format('DelphiX disabled package "%s" found. BPL removed and Delphi config adjusted.', [s]));
          end else
            Log(Format('DelphiX disabled package "%s" found. BPL would be removed and Delphi config would be adjusted.', [s]));
        end;
        if Pos('omega', LowerCase(s)) > 0 then begin
          if bForReal then begin
            Reg.DeleteValue(slNames[i]);
            DeleteFile(FixDelphiPath(slNames[i], sDelphiPath));
            Log(Format('Omega disabled package "%s" found. BPL removed and Delphi config adjusted.', [s]));
          end else
            Log(Format('Omega disabled package "%s" found. BPL would be removed and Delphi config would be adjusted.', [s]));
        end;
      end;
      reg.CloseKey;
    end;

    //Scan palette entries for DelphiX
    if reg.OpenKey(sDelphiKey + '\Palette', False) then begin
      reg.GetValueNames(slNames);
      for i := slNames.Count-1 downto 0 do begin
        s := reg.ReadString(slNames[i]);
        if Pos('delphix', LowerCase(s)) > 0 then begin
          if bForReal then begin
            Reg.DeleteValue(slNames[i]);
            Log(Format('DelphiX palette entry "%s" found. Delphi config adjusted.', [slNames[i]]));
          end else
            Log(Format('DelphiX palette entry "%s" found. Delphi config would be adjusted.', [slNames[i]]));
        end;
        if Pos('omega', LowerCase(s)) > 0 then begin
          if bForReal then begin
            Reg.DeleteValue(slNames[i]);
            Log(Format('Omega palette entry "%s" found. Delphi config adjusted.', [slNames[i]]));
          end else
            Log(Format('Omega palette entry "%s" found. Delphi config would be adjusted.', [slNames[i]]));
        end;
      end;
      reg.CloseKey;
    end;

    //Scan Library settings for DelphiX and DirectX headers
    if reg.OpenKey(sDelphiKey + '\Library', False) then begin
      s := reg.ReadString('Browsing Path');
      slValues.Text := StringReplace(s, ';', #13#10, [rfReplaceAll]);
      for i := slValues.Count-1 downto 0 do begin
        s := slValues[i];
        if Copy(s, Length(s), 1) <> '\' then s := s + '\';
        if FileExists(s + 'DelphiXcfg.inc') then begin
          if bForReal then begin
            slValues.Delete(i);
            Log(Format('DelphiX library path "%s" found. Delphi config adjusted.', [s]));
          end else
            Log(Format('DelphiX library path "%s" found. Delphi config would be adjusted.', [s]));
        end;
        if FileExists(s + 'DirectDraw.pas') then begin
          if bForReal then begin
            slValues.Delete(i);
            Log(Format('DirectX JEDI library path "%s" found. Delphi config adjusted.', [s]));
          end else
            Log(Format('DirectX JEDI library path "%s" found. Delphi config would be adjusted.', [s]));
        end;
        if FileExists(s + 'OmegaRegister.pas') then begin
          if bForReal then begin
            slValues.Delete(i);
            Log(Format('Omega library path "%s" found. Delphi config adjusted.', [s]));
          end else
            Log(Format('Omega library path "%s" found. Delphi config would be adjusted.', [s]));
        end;
      end;
      slValues.Append(edOmega.Text);
      slValues.Append(edDirectX.Text);
      s := StringReplace(slValues.Text, #13#10, ';', [rfReplaceAll]);
      if bForReal then begin
        reg.WriteString('Browsing Path', s);
        Log('New Omega and DirectX JEDI paths added to Delphi Library path');
      end else
        Log('New Omega and DirectX JEDI paths would be added to Delphi Library path');
      reg.CloseKey;
    end;

    //Scan Search settings for DelphiX and DirectX headers
    if reg.OpenKey(sDelphiKey + '\Library', False) then begin
      s := reg.ReadString('Search Path');
      slValues.Text := StringReplace(s, ';', #13#10, [rfReplaceAll]);
      for i := slValues.Count-1 downto 0 do begin
        s := slValues[i];
        if Copy(s, Length(s), 1) <> '\' then s := s + '\';
        if FileExists(s + 'DelphiXcfg.inc') then begin
          if bForReal then begin
            slValues.Delete(i);
            Log(Format('DelphiX search path "%s" found. Delphi config adjusted.', [s]));
          end else
            Log(Format('DelphiX search path "%s" found. Delphi config would be adjusted.', [s]));
        end;
        if FileExists(s + 'DirectDraw.pas') then begin
          if bForReal then begin
            slValues.Delete(i);
            Log(Format('DirectX JEDI search path "%s" found. Delphi config adjusted.', [s]));
          end else
            Log(Format('DirectX JEDI search path "%s" found. Delphi config would be adjusted.', [s]));
        end;
        if FileExists(s + 'OmegaRegister.pas') then begin
          if bForReal then begin
            slValues.Delete(i);
            Log(Format('Omega search path "%s" found. Delphi config adjusted.', [s]));
          end else
            Log(Format('Omega search path "%s" found. Delphi config would be adjusted.', [s]));
        end;
      end;
      slValues.Append(edOmega.Text);
      slValues.Append(edDirectX.Text);
      s := StringReplace(slValues.Text, #13#10, ';', [rfReplaceAll]);
      if bForReal then begin
        reg.WriteString('Search Path', s);
        Log('New Omega and DirectX JEDI paths added to Delphi Search  path');
      end else
        Log('New Omega and DirectX JEDI paths would be added to Delphi Search path');
      reg.CloseKey;
    end;

  finally
    reg.Free;
    slNames.Free;
    slValues.Free;
  end;
end;

procedure TfrmMain.btOmegaClick(Sender: TObject);
begin
  dlgFolder.Title := 'Select Omega source path';
  dlgFolder.Folder := edOmega.Text;
  if dlgFolder.Execute then
    if ValidateOmegaPath(dlgFolder.Folder) then
      edOmega.Text := dlgFolder.Folder;
end;

procedure TfrmMain.btDirectXClick(Sender: TObject);
begin
  dlgFolder.Title := 'Select DirectX JEDI path';
  dlgFolder.Folder := edDirectX.Text;
  if dlgFolder.Execute then
    if ValidateDirectXPath(dlgFolder.Folder) then
      edDirectX.Text := dlgFolder.Folder;
end;

function TfrmMain.ValidateOmegaPath(const Path: String): Boolean;
begin
  Result := False;
  if not FileExists(IncludeTrailingBackslash(Path) + 'OmegaRegister.pas') then begin
    ShowMessage('Incorrect Omega path specified: OmegaRegister.pas not found!');
    Exit;
  end;
  Result := True;
end;

function TfrmMain.ValidateDirectXPath(const Path: String): Boolean;
var
  sl: TStringList;
begin
  Result := False;
  if not FileExists(IncludeTrailingBackslash(Path) + 'DirectMusic.pas') then begin
    ShowMessage('Incorrect DirectX path specified: DirectMusic.pas not found!');
    Exit;
  end;
  sl := TStringList.Create;
  try
    sl.LoadFromFile(IncludeTrailingBackslash(Path) + 'DirectMusic.pas');
    if Pos('ppSegmentState: PIDirectMusicSegmentState', sl.Text) = 0 then begin
      ShowMessage('Incorrect DirectX path specified: Please make sure you point to the Omega-fixed version of the JEDI DirectX 8 headers!');
      Exit;
    end;
  finally
    sl.Free;
  end;
  Result := True;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
var
  ini: TIniFile;
begin
  //Get possible settings from .ini file
  ini := TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  try
    ini.WriteString('Paths', 'Omega', edOmega.Text);
    ini.WriteString('Paths', 'DirectX', edDirectX.Text);
  finally
    ini.Free;
  end;
end;

end.
