(*----------------------------------------------------------------------------*
 *  Direct3D sample for DirectX 8.1 demostrating how to create and render     *
 *  Dot product (Dot3) bumpmapped / modulated font. Font normal map generated *
 *  with D3DX8.1 function D3DXComputeNormalMap which itself requires height   *
 *  map. So height map is generated from font bitmap by averaging intensivity *
 *  of 3x3 square of pixels.                                                  *
 *                                                                            *
 *  Note: This sample requires DirectX 8.1 to compile and run!!!              *
 *                                                                            *
 *  Copyright (c) 2002 by Alexey Barkovoy                                     *
 *  E-Mail: clootie@reactor.ru                                                *
 *                                                                            *
 *  Modified: 29-May-2002                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru                                                *
 *----------------------------------------------------------------------------*)
unit Unit1;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, OmegaImageList, OmegaScreen, OmegaInput,
  OmegaBitmapFont, OmegaTimer, OmegaUtils, D3DFont, CommDlg,
  D3DFontDot3, D3DUtil, DXUtil, D3DX8, {$I UseD3D8.inc};

type
  TForm1 = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaTimer1: TOmegaTimer;
    OmegaInput1: TOmegaInput;
    procedure FormCreate(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
   private
    m_pFont: CD3DFont;
    m_pDot3Font: TD3DFontDot3;
    m_pStatsFont: CD3DFont;
    m_strFont: array[0..99] of Char;
    m_dwFontSize: DWORD;
    MouseNormal, CircularNormal: TD3DXVector3;
    Hue: Byte;
    procedure ChangeFont;
    function FrameMove: HRESULT;
    function ResetData: HRESULT;
  end;

var Form1: TForm1;

implementation
{$R *.DFM}

// Additional routines
type
  TRGBRec = packed record
    B, G, R, a: Byte;
  end;

  THSBRec = packed record
    H, S, B, a: Byte;
  end;

type
  TToneArray = array [0..255] of TRGBRec;

function HSB(H, S, B: Byte): THSBRec;
begin
  Result.H:= H;
  Result.S:= S;
  Result.B:= B;
  Result.a:= 0;
end;

procedure DivMod(Dividend: Integer; Divisor: Word;
  var Result, Remainder: Word);
asm
        PUSH    EBX
        MOV     EBX,EDX
        MOV     EDX,EAX
        SHR     EDX,16
        DIV     BX
        MOV     EBX,Remainder
        MOV     [ECX],AX
        MOV     [EBX],DX
        POP     EBX
end;

function HSBtoRGB(HSB: THSBRec): TRGBRec;
 procedure DoLuminescency(var V: Byte; L: Byte);
 begin
   if L > 120
   then V:= V + ((L - 120)*(255 - V)) div 120
   else V:= V - ((120 - L)*V) div 120;
 end;
var
  ColInd, Rest: Word;
begin
  Assert(not ((HSB.H > 240) or (HSB.S > 240) or (HSB.B > 240)),
    'HSB value out of range');

  // calculate Hue
  DivMod(HSB.H, 40, ColInd, Rest);
  case ColInd of
    0: begin
       Result.R:= 255;
       Result.G:= (255*Rest) div 40;
       Result.B:= 0;
     end;
    1: begin
       Result.R:= (255*(40 - Rest)) div 40;
       Result.G:= 255;
       Result.B:= 0;
     end;
    2: begin
       Result.R:= 0;
       Result.G:= 255;
       Result.B:= (255*Rest) div 40;
     end;
    3: begin
       Result.R:= 0;
       Result.G:= (255*(40 - Rest)) div 40;
       Result.B:= 255;
     end;
    4: begin
       Result.R:= (255*Rest) div 40;
       Result.G:= 0;
       Result.B:= 255;
     end;
    5: begin
       Result.R:= 255;
       Result.G:= 0;
       Result.B:= (255*(40 - Rest)) div 40;
     end;
  end;

  // add Saturation
  Result.R:= Result.R + ((240 - HSB.S)*(127 - Result.R)) div 240;
  Result.G:= Result.G + ((240 - HSB.S)*(127 - Result.G)) div 240;
  Result.B:= Result.B + ((240 - HSB.S)*(127 - Result.B)) div 240;

  // add Luminescency
  DoLuminescency(Result.R, HSB.B);
  DoLuminescency(Result.G, HSB.B);
  DoLuminescency(Result.B, HSB.B);
end;

// function compares 2 HSB records to verify equality
function IsEqualHSB(HSB1, HSB2: THSBRec): Boolean;
begin
  Result:= (HSB1.H = HSB2.H) and (HSB1.S = HSB2.S) and (HSB1.B = HSB2.B);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 self.ClientWidth := 800;
 self.ClientHeight := 600;
 OmegaScreen1.Width := self.ClientWidth;
 OmegaScreen1.Height := self.ClientHeight;
 OmegaScreen1.Init;

 // Create fonts
 m_strFont    := 'Arial';
 m_dwFontSize := 48;
 m_pFont      := CD3DFont.Create(m_strFont, m_dwFontSize);
 m_pDot3Font  := TD3DFontDot3.Create(m_strFont, m_dwFontSize);
 m_pStatsFont := CD3DFont.Create('Arial', 12, D3DFONT_BOLD);
 Hue          := 50; // initially display text in green
 m_pFont.InitDeviceObjects(OmegaScreen1.Device);
 m_pDot3Font.InitDeviceObjects(OmegaScreen1.Device);
 m_pStatsFont.InitDeviceObjects(OmegaScreen1.Device);
 m_pFont.RestoreDeviceObjects;
 m_pDot3Font.RestoreDeviceObjects;
 m_pStatsFont.RestoreDeviceObjects;

 ResetData;
 DXUtil_Timer(TIMER_START);

 OmegaTimer1.Enabled := True;
end;

function TForm1.ResetData: HRESULT;
var
  vEyePt: TD3DXVector3;
  vLookatPt: TD3DXVector3;
  vUpVec: TD3DXVector3;
  matWorld, matView, matProj: TD3DXMatrix;
  fAspect: Single;
begin
  // Restore the textures
  OmegaScreen1.Device.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  OmegaScreen1.Device.SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
  OmegaScreen1.Device.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
  OmegaScreen1.Device.SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
  OmegaScreen1.Device.SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
  OmegaScreen1.Device.SetRenderState(D3DRS_DITHERENABLE, 1);
  OmegaScreen1.Device.SetRenderState(D3DRS_SPECULARENABLE, 1);
  OmegaScreen1.Device.SetRenderState(D3DRS_AMBIENT, $00ffffff);
  // Set the transform matrices
  vEyePt    := D3DXVector3(0.0,-5.0,-10.0);
  vLookatPt := D3DXVector3(0.0, 0.0,  0.0);
  vUpVec    := D3DXVector3(0.0, 1.0,  0.0);
  D3DXMatrixIdentity(matWorld);
  D3DXMatrixLookAtLH(matView, vEyePt, vLookatPt, vUpVec);
  fAspect := ClientWidth / ClientHeight;
  D3DXMatrixPerspectiveFovLH(matProj, D3DX_PI/4, fAspect, 1.0, 100.0);
  OmegaScreen1.Device.SetTransform(D3DTS_WORLD,      matWorld);
  OmegaScreen1.Device.SetTransform(D3DTS_VIEW,       matView);
  OmegaScreen1.Device.SetTransform(D3DTS_PROJECTION, matProj);
  Result:= S_OK;
end;

function TForm1.FrameMove: HRESULT;
var
  n: TD3DXVector3;
  pt: TPoint;
  m_fTime: single;
begin
  m_fTime := DXUtil_Timer(TIMER_GETAPPTIME);
  // Compute the light vector from the cursor position
  if GetFocus <> 0 then
  begin
    GetCursorPos(pt);
    pt := ScreenToClient(pt);
    n.x := (((2.0 * pt.x ) / ClientWidth ) - 1);
    n.y := (((2.0 * pt.y ) / ClientHeight ) - 1);
    n.z := 0.0;
    if (D3DXVec3Length(n) <= 1.0) then
     n.z := Sqrt(1.0 - n.x*n.x - n.y*n.y);
    MouseNormal:= n;
  end;
  // Calculate circular vector (for 3'rd text line)
  CircularNormal.x:= Cos(m_fTime * 4);
  CircularNormal.y:= Sin(m_fTime * 4);
  CircularNormal.z:= 1- (pt.y / ClientHeight);
  D3DXVec3Normalize(CircularNormal, CircularNormal);
  Result:= S_OK;
end;

procedure TForm1.OmegaTimer1Timer(Sender: TObject);
var
  rgb: TRGBRec;
begin
 if not OmegaScreen1.CanDraw then exit;
 OmegaInput1.Update;
 if oisbutton1 in OmegaInput1.keyboard.statesclicked then
 begin
  m_pFont.InvalidateDeviceObjects;
  m_pStatsFont.InvalidateDeviceObjects;
  m_pDot3Font.InvalidateDeviceObjects;
  m_pFont.DeleteDeviceObjects;
  m_pStatsFont.DeleteDeviceObjects;
  m_pDot3Font.DeleteDeviceObjects;
  OmegaScreen1.ToggleFullScreen;
  m_pFont.InitDeviceObjects(OmegaScreen1.Device);
  m_pStatsFont.InitDeviceObjects(OmegaScreen1.Device);
  m_pDot3Font.InitDeviceObjects(OmegaScreen1.Device);
  m_pFont.RestoreDeviceObjects;
  m_pStatsFont.RestoreDeviceObjects;
  m_pDot3Font.RestoreDeviceObjects;
  ResetData;
 end;
 OmegaScreen1.ClearScreen(0, 160, 60);
 FrameMove;
 OmegaScreen1.BeginRender;

 // Draw CD3DFont in 2D (red)
 m_pFont.DrawText(20, 75, D3DCOLOR_ARGB(255,255,0,0), 'CD3DFont.DrawText');
 m_pDot3Font.SetLightNormal(MouseNormal);
 m_pDot3Font.DrawText(20, 190, D3DCOLOR_ARGB(255, rgb.r, rgb.g, rgb.b),
   'TD3DFontDot3.DrawText');
 m_pDot3Font.DrawText(20, 135, D3DCOLOR_ARGB(255,0,0,255), 'TD3DFontDot3.DrawText');
 rgb := HSBtoRGB(HSB(Hue, 240, 100));
 m_pDot3Font.SetLightNormal(CircularNormal);
 // Show keyboard commands hints
 m_pStatsFont.DrawText(5, 20, D3DCOLOR_ARGB(255,255,255,255),
 'F'#10 + 'X'#10 + 'A / Z');
 m_pStatsFont.DrawText(45, 20, D3DCOLOR_ARGB(255,255,255,0),
  '- Choose different font'#10 +
  '- Modulate 2X'#10 +
  '- Modify color hue of lowest text');
 m_pStatsFont.DrawText(305, 20, D3DCOLOR_ARGB(255,255,255,255),
  'D'#10 + 'C');
 m_pStatsFont.DrawText(320, 20, D3DCOLOR_ARGB(255,255,255,0),
  '- Switch Bumped / "normal texture" view'#10 +
  '- Blend to Black / Back buffer');
 OmegaScreen1.EndRender;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 OmegaTimer1.Enabled := False;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if key = 27 then close;
 if (Ord('D') = key) then
  m_pDot3Font.fOnlyDot3:= not m_pDot3Font.fOnlyDot3;
 if (Ord('X') = key) then
  m_pDot3Font.fModulate2X := not m_pDot3Font.fModulate2X;
 if (Ord('C') = key) then
  m_pDot3Font.fBlendToBlack:= not m_pDot3Font.fBlendToBlack;
 if (Ord('F') = key) then ChangeFont;
 if (Ord('A') = key) then
 begin
  Inc(Hue, 5);
  if Hue > 235 then Hue:= 0;
 end;
 if (Ord('Y') = key) then
  if Hue > 5 then Dec(Hue, 5) else Hue:= 235;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 m_pFont.DeleteDeviceObjects;
 m_pDot3Font.DeleteDeviceObjects;
 m_pStatsFont.DeleteDeviceObjects;
 m_pFont := nil;
 m_pDot3Font := nil;
 m_pStatsFont := nil;
end;

procedure TForm1.ChangeFont;
var
  hdc_: HDC;
  lHeight: Longint;
  lf: TLogFont;
  cf: TChooseFont;
begin
  hdc_ := GetDC(handle);
  lHeight := -MulDiv(m_dwFontSize, GetDeviceCaps(hdc_, LOGPIXELSY), 72);
  ReleaseDC(handle, hdc_);
  // hdc_ := 0;
  lstrcpy(lf.lfFaceName, m_strFont);
  lf.lfHeight := lHeight;
  ZeroMemory(@cf, SizeOf(cf));
  cf.lStructSize := SizeOf(cf);
  cf.hwndOwner   := handle;
  cf.lpLogFont   := @lf;
  cf.Flags       := CF_SCREENFONTS or CF_INITTOLOGFONTSTRUCT;
  if ChooseFont(cf) then
  begin
    SAFE_DELETE(m_pFont);
    SAFE_DELETE(m_pDot3Font);
    lstrcpy(m_strFont, lf.lfFaceName);
    m_dwFontSize := Trunc(cf.iPointSize / 10);
    m_pFont := CD3DFont.Create(m_strFont, m_dwFontSize);
    m_pFont.InitDeviceObjects(OmegaScreen1.Device);
    m_pFont.RestoreDeviceObjects;
    m_pDot3Font:= TD3DFontDot3.Create(m_strFont, m_dwFontSize);
    m_pDot3Font.InitDeviceObjects(OmegaScreen1.Device);
    m_pDot3Font.RestoreDeviceObjects;
  end;
end;

end.





